// CellSelectionPanel.java
// Started on June 17th, 2001
// Copyright (c) Virtu Halttunen

//import TheGame.*;
import java.awt.*;
import javax.swing.*;
import java.util.Vector;

public class CellSelectionPanel extends JPanel implements Scrollable
{
    private Vector images;
    private int cellSize;
    private Dimension viewportSize;
    private int padding;
    private int visibleHorizontal;
    private int visibleVertical;
    private int yCells = 1;

    public CellSelectionPanel( Vector imageVector, int cellSize, int visibleHorizontal, int visibleVertical, int padding )
    {
	super();
	images = imageVector;
	this.cellSize = cellSize;
	this.visibleHorizontal = visibleHorizontal;
	this.visibleVertical = visibleVertical;
	this.padding = padding;
	
	viewportSize = new Dimension( visibleHorizontal*cellSize+(visibleHorizontal+1)*padding, visibleVertical*cellSize+(visibleVertical+1)*padding );
	validateSize();
    }
    
    public void appendImages( Vector newImages )
    {
	images.addAll( newImages );
	validateSize();
    }
    
    public void setImages( Vector newImages )
    {
	images = newImages;
	validateSize();
    }
	
    
    public void validateSize()
    {
	Dimension mySize;
	if( images.size() > 0 )
	{
	    yCells = images.size()/visibleHorizontal;
	    System.out.println( "CellSelectionpanel: vertical cells: "+ yCells );
	    mySize = new Dimension ( visibleHorizontal*cellSize+(visibleHorizontal+1)*padding, yCells*cellSize+(yCells+1)*padding );
	    setMinimumSize( mySize );
	    setPreferredSize( mySize );
	    setMaximumSize( mySize );
	}
    }
    
    protected void paintComponent( Graphics g )
    {
	/*
	super.paintComponent( g );
	Rectangle newRect = g.getClipBounds();
	
	Point min = getCell( new Point( newRect.x, newRect.y ) );
	int minX = min.x;
	int minY = min.y;
	Point max = getCell( new Point( newRect.x+newRect.width, newRect.y+newRect.height ) );
	int maxX = max.x;
	int maxY = max.y;
	
	if( maxX == visibleHorizontal ) maxX = maxX-1;
	if( maxY == visibleVertical ) maxY = maxY-1;
	  
	for( int i = minX; i <= maxX; i++ )
	    for( int j = minY; j <= maxY; j++ )
	    {
		g.drawImage( (Image)images.elementAt( i+(i*j) ), i*(cellSize+padding), j*(cellSize+padding), cellSize, cellSize, this );
	    }	
	*/
	g.setColor( Color.white );
	g.fillRect( 0, 0, getWidth(), getHeight() );
	int cellNum = 0;
	for( int i = 0; i < yCells; i++ )
	    for( int j = 0; j < visibleHorizontal; j++ )
	    {
		g.drawImage( (Image)images.elementAt( cellNum++ ), j*(cellSize+padding), i*(cellSize+padding), cellSize, cellSize, this );
	    }
	
    }
    
    public int getCellType( Point point )
    {
	int x=0;
	int y=0;
	
	x = point.x/(cellSize+padding);
	y = point.y/(cellSize+padding);
	if( x < 0 ) x = 0;
	if( x > visibleHorizontal ) x = visibleHorizontal-1;
	if( y < 0 ) y = 0;
	if( y > yCells ) y = yCells-1;
	
	return x+y*visibleHorizontal;   
    }
    
    public Image getCellImage( int cell )
    {
	return (Image)images.elementAt( cell );
    }

    public Point getCell( Point point )
    {
	int x=0;
	int y=0;
	
	x = point.x/(cellSize+padding);
	y = point.y/(cellSize+padding);
	if( x < 0 ) x = 0;
	if( x > visibleHorizontal+visibleHorizontal*2*padding ) x = visibleHorizontal-2*padding;
	if( y < 0 ) y = 0;
	if( y > visibleVertical+visibleVertical*2*padding ) y = visibleVertical-2*padding;
	
	return new Point( x, y );
    }
	
    public Vector getCellImages()
    {
	return images;
    }	

    public Dimension getPreferredScrollableViewportSize()
    {
	return viewportSize;
    }
    
    public int getScrollableUnitIncrement( Rectangle visibleRect, int orientation, int direction ) 
    {
	//get the current position
	int currentPosition = 0;
	if( orientation == SwingConstants.HORIZONTAL )
	    currentPosition = visibleRect.x;
	else
	    currentPosition = visibleRect.y;
    
	//return the number of pixels between currentPosition
	//and the nearest tick mark in the indicated direction
	if ( direction < 0 ) 
	{
	    int newPosition = currentPosition - (currentPosition / (cellSize+padding) ) * (cellSize+padding);
	    return (newPosition == 0) ? cellSize+padding : newPosition;
	} 
	else 
	{
	    return ((currentPosition / (cellSize+padding) ) + 1) * (cellSize+padding) - currentPosition;
	}
    }
    
    public int getScrollableBlockIncrement( Rectangle visibleRect, int orientation, int direction ) 
    {
	int amount = 0;
	if( orientation == SwingConstants.HORIZONTAL )
	{
	    amount = getScrollableUnitIncrement( visibleRect, orientation, direction );
	    if( amount < cellSize+padding )
		amount = amount+cellSize+padding;
	    return visibleRect.width - amount;
	}
	else
	{
	    amount = getScrollableUnitIncrement( visibleRect, orientation, direction );
	    if( amount < cellSize+padding )
		amount = amount+cellSize+padding;
	    return visibleRect.height - amount;
	}
    }
    
    public boolean getScrollableTracksViewportWidth()
    {
	return false;
    }
    
    public boolean getScrollableTracksViewportHeight()
    {
	return false;
    }

}