// OverviewMapPanel.java
// Started on June 17th, 2001
// Copyright (c) Virtu Halttunen

//import TheGame.*;
import java.awt.*;
import javax.swing.*;

public class OverviewMapPanel extends JPanel
{
    private Color backgroundColor = Color.black;
    private Color floorColor = Color.darkGray;
    private Color wallColor = Color.lightGray;
    private Color doorColor = Color.yellow;
    private Color friendlyColor = Color.green;
    private Color hostileColor = Color.red;
    
    private int cellSize = 2;
    private Map map;
    
    public OverviewMapPanel( Map map )
    {
	super();
	this.map = map;
	Dimension mySize = new Dimension ( map.getHorizontalSize()*cellSize, map.getVerticalSize()*cellSize );
	setMinimumSize( mySize );
	setPreferredSize( mySize );
	setMaximumSize( mySize );
    }
    
    protected void paintComponent( Graphics g )
    {
	super.paintComponent( g );
	for( int i = 0; i < map.getHorizontalSize(); i++ )
	    for( int j = 0; j < map.getVerticalSize(); j++ )
	    {
		drawCell( g, i, j );
	    }	
    }
    
    private void drawCell( Graphics g, int x, int y )
    {
	MapCell cell =  map.getCell( x, y );
        g.setColor( backgroundColor );
        if( cell != null )
        {
            if( cell.getFloor() > -1 )
            {
                g.setColor( floorColor );
            }
            if( cell.getWall() > -1 )
            {
                g.setColor( wallColor );
            }
            if( cell.hasDoor() )
            {
                if( !map.getDoor( x, y ).isOpen() )
                    g.setColor( doorColor );
            }
        }
	g.fillRect( x*cellSize, y*cellSize, cellSize, cellSize );
    }
    
    public void setMap( Map map )
    {
	this.map = map;
        Dimension mySize = new Dimension ( map.getHorizontalSize()*cellSize, map.getVerticalSize()*cellSize );
        setMinimumSize( mySize );
        setPreferredSize( mySize );
        setMaximumSize( mySize );
	invalidate();
        repaint();
    }
    
    public Point getCell( Point point )
    {
	return new Point( point.x/cellSize, point.y/cellSize );    
    }
}
