// DirectionPanel.java
// Started on April 13th, 2002
// Copyright (c) Virtu Halttunen

import TheGame.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Vector;

public class DirectionPanel extends JPanel implements MouseListener
{
    private int direction = Map.NORTH;
    private Dimension size;
    private Vector listeners;
    private int eventIndex = 0;

    public DirectionPanel( int width, int height )
    {
        size = new Dimension( width, height );
        setMinimumSize( size );
        setMaximumSize( size );
        setPreferredSize( size );
        addMouseListener( this );
        listeners = new Vector();
    }

    public void addActionListener( ActionListener listener )
    {
        listeners.add( listener );
    }

    public void removeActionListener( ActionListener listener )
    {
        listeners.remove( listener );
    }

    public int getDirection()
    {
        return direction;
    }

    public void paintComponent( Graphics g )
    {
	if( g instanceof Graphics2D )
	{
	    ((Graphics2D)g).setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
	}
        g.setColor( Color.lightGray );
        g.fillRect( 0, 0, size.width, size.height );
        g.setColor( Color.black );
        g.drawOval( 2, 2, size.width-4, size.height-4 );
        g.setColor( Color.blue );
        int xOffset = 0;
        int yOffset = 0;
        int angle = 0;
        int endAngle = 90;
        //int diagonalOffset = (int)(0.75/Math.sqrt(2));
        switch( direction )
        {
            case Map.NORTH:
                angle = 225; yOffset=-(int)(size.height*0.675); xOffset=-(int)(size.width*0.25); break;
            case Map.NORTH_EAST:
                angle = 180;
                yOffset=-(int)(size.height*0.5); xOffset=(int)(size.width*0.125);
                //yOffset = diagonalOffset-(int)(size.height*0.25);
                //xOffset = diagonalOffset-(int)(size.width*0.25);
               break;
            case Map.EAST:
                angle = 135; xOffset=(int)(size.width*0.125); yOffset=-(int)(size.width*0.25); break;
            case Map.SOUTH_EAST:
                angle = 90; xOffset=0; yOffset=0; break;
            case Map.SOUTH:
                angle = 45; yOffset=(int)(size.height*0.125); xOffset=-(int)(size.width*0.25); break;
            case Map.SOUTH_WEST:
                angle = 0; yOffset=0; xOffset=-(int)(size.width*0.5); break;
            case Map.WEST:
                angle = 315; xOffset=-(int)(size.width*0.675); yOffset=-(int)(size.width*0.25); break;
            case Map.NORTH_WEST:
                angle = 270; xOffset=-(int)(size.width*0.5); yOffset=-(int)(size.height*0.5); break;
        }
        angle = angle+30;
        endAngle = endAngle-60;
        g.fillArc( xOffset, yOffset, (int)(size.width*1.5), (int)(size.height*1.5), angle, endAngle );
    }

    public void mouseClicked( MouseEvent e )
    {
        if( e.getSource() == this )
        {
            Point point = e.getPoint();
            direction = EditorMap.getDirection( new Point( size.width/2, size.height/2), point, direction );
            repaint();
            ActionEvent event = new ActionEvent( this, eventIndex++, "Direction", e.getModifiers() );
            fireActionPerformed( event );
        }
    }

    protected void fireActionPerformed( ActionEvent event )
    {
        ActionListener listener;
        for( int i=0; i < listeners.size(); i++ )
        {
            listener = (ActionListener)listeners.elementAt( i );
            listener.actionPerformed( event );
        }
    }

    public void mousePressed( MouseEvent e )
    {}

    public void mouseReleased( MouseEvent e )
    {}

    public void mouseEntered( MouseEvent e )
    {}

    public void mouseExited( MouseEvent e )
    {}
    
}