// AlarmDrone.java
// Started on September 8th, 2003
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class AlarmDrone extends Monster implements Robot
{
    public static double ALARM_DISTANCE = 10.0;

    private boolean hasMoreActions = false;
    private boolean rightHanded = true;
    private boolean alarmed = false;
    Character target = null;
    private boolean disabled = false;

    public AlarmDrone()
    {
        location = new Point( 0, 0 );
        direction = Map.NORTH;
        alive = true;
        lastAction = new MoveAction( this, location, location );
        maximumActionPoints = 6;
        currentActionPoints = 6;
        maximumHitPoints = 6;
        currentHitPoints = 6;
        rightHanded = (Math.random() < 0.5 );
        if( Math.random() < 0.25 )
        {
            direction = Map.NORTH_EAST;
        }
    }


    public void setSpottedPlayer( Character character, boolean canSee )
    {
        target = character;
        if( canSee )
            alarmed = true;
    }
    
    public GameAction playAction()
    {
        Point cell;

        Monster character;
        Character[] enemies;

        if( alive && !disabled )
        {
            if( currentActionPoints > 0 )
            {
                // Check alarms
                if( alarmed && target != null )
                {
                    for( int i=0; i < activeGame.getMonsters().size(); i++ )
                    {
                        character = activeGame.getMonster( i );
                        if( character instanceof BattleRobot || character instanceof AlarmDrone || character instanceof AssaultDrone)
                        {
                            if( character.getLocation().distance( location ) <= ALARM_DISTANCE )
                            {
                                character.setSpottedPlayer( target, false );
                            }
                        }
                    }
                }
                enemies = activeGame.getVisibleEnemies( this );
                alarmed = ( enemies.length > 0 );
                if( !alarmed )
                {
                    target = null;
                }
                // Move
                if( activeGame.canCharacterMove( this, direction, false ) )
                {
                    cell = activeGame.getMap().getNeighbor( location, direction );
                    lastAction = new MoveAction( this, new Point( location ), new Point( cell ) );
                    return lastAction;
                }
                if( rightHanded )
                {
                    direction = (direction+2)%8;
                }
                else
                {
                    direction = (direction+6)%8;
                }
                lastAction = new WaitAction( this, new Point( location ) );
            }
        }
        return lastAction;
    }

    public GameAction getLastAction()
    {
        return lastAction;
    }

    public void resetActionsPoints()
    {
        currentActionPoints = maximumActionPoints;
    }

    public boolean hasMoreActions()
    {
        if( currentActionPoints > 0 )
        {
            return true;
        }
        return false;
    }

    public boolean applyDamage( int damage )
    {
        damage = damage-1;
        if( damage < 0 )
        {
            damage = 0;
        }
        currentHitPoints = currentHitPoints-damage;
        lastAction = new WoundAction( this, location );
        if( currentHitPoints <= 0 )
        {
            alive = false;
            lastAction = new DeadAction( this, new Point( location ) );
            maximumActionPoints = 0;
            currentActionPoints = 0;
        }
        return alive;
    }

    public void setDisabled( boolean disabled )
    {
        this.disabled = disabled;
        if( disabled )
        {
            alive = false;
            lastAction = new DeadAction( this, new Point( location ) );
        }
    }

    public boolean getDisabled()
    {
        return disabled;
    }
        
    public int getSize()
    {
        return 45;
    }
    
    public int getMeleeAbility()
    {
	return 2;
    }
    
}