// AssaultDrone.java
// Started on September 8th, 2003
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class AssaultDrone extends Monster implements Robot
{
    public static final int ROAMING = 0;
    public static final int SEEKING = 1;
    public static final int ATTACKING = 2;
    
    private int state = ROAMING;
    private Character target = null;
    private boolean hasMoreActions = false;
    private boolean rightHanded;
    private boolean disabled = false;

    public AssaultDrone()
    {
        location = new Point( 0, 0 );
        direction = Map.NORTH;
        alive = true;
        lastAction = new MoveAction( this, location, location );
        maximumActionPoints = 4;
        currentActionPoints = 4;
        maximumHitPoints = 6;
        currentHitPoints = 6;
        rightHanded = (Math.random() < 0.5 );
        if( Math.random() < 0.25 )
        {
            direction = Map.NORTH_EAST;
        }        
    }

    public void setSpottedPlayer( Character character, boolean canSee )
    {
        if( canSee )
        {
            target = character;
            state = SEEKING;
            //System.out.println( "AssaultDrone spotted player: " + index );
        }
    }
    
    
    public GameAction playAction()
    {
        int x;
        int y;

        Point cell;

        if( alive )
        {
            if( currentActionPoints > 0 )
            {
                hasMoreActions = true;
                // If next to a player, attack
                for( int i=0; i < 8; i++ )
                {
                    for( int j=0; j < activeGame.getPlayerCharacters().size(); j++ )
                    {
                        if( activeGame.getPlayerCharacter(j).isAlive() )
                        {
                            cell = activeGame.getPlayerCharacter(j).getLocation();
                            if( cell.equals( activeGame.getMap().getNeighbor( location, i ) ) )
                            {
                                if( currentActionPoints >= 2 )
                                {
                                    direction = i;
                                    lastAction = new MeleeAction( this, activeGame.getPlayerCharacter(j) );
                                    state = ATTACKING;
                                    return lastAction;
                                }
                                else
                                {
                                    lastAction = new MoveAction( this, new Point( location ), new Point( location ) );
                                    hasMoreActions = false;
                                    return lastAction;
                                }
                            }
                        }
                     }
                }
                // If can see a player, move closer and attack
                for( int i=0; i < activeGame.getPlayerCharacters().size(); i++ )
                {
                    if( activeGame.getMap().hasLineOfSight( location, activeGame.getPlayerCharacter( i ).getLocation() ) && activeGame.getPlayerCharacter( i ).isAlive() )
                    {
                        state = SEEKING;
                        target = activeGame.getPlayerCharacter( i );
                        break;
                    }
                }
                if( target != null )
                {
                    Vector path = activeGame.getMap().findShortestPath( this, target.getLocation(), false );
                    if( path.size() >= 2 )
                    {
                        cell = ((MapNode)path.elementAt( 1 )).getPoint();
                        direction = Map.getDirection( location, cell, direction );
                        lastAction = new MoveAction( this, new Point( location ), new Point( cell ) );
                        return lastAction;
                    }
                    else
                    {
                        state = ROAMING;
                        target = null;
                    }
                }
                if( state == ROAMING )
                {
                    // Move
                    if( activeGame.canCharacterMove( this, direction, false ) )
                    {
                        cell = activeGame.getMap().getNeighbor( location, direction );
                        lastAction = new MoveAction( this, new Point( location ), new Point( cell ) );
                        return lastAction;
                    }
                    if( rightHanded )
                    {
                        direction = (direction+2)%8;
                    }
                    else
                    {
                        direction = (direction+6)%8;
                    }
                }
            }
            lastAction = new WaitAction( this, new Point( location ) );
        }
        return lastAction;
    }

    public GameAction getLastAction()
    {
        return lastAction;
    }

    public int getMeleeAbility()
    {
	return 5;
    }
    
    public int getMeleeCost()
    {
	return 2;
    }
    
    public void resetActionsPoints()
    {
        currentActionPoints = maximumActionPoints;
    }

    public boolean hasMoreActions()
    {
        if( currentActionPoints > 0 )
        {
            return true;
        }
        return false;
    }

    public boolean applyDamage( int damage )
    {
        damage = damage-1;
        if( damage < 0 )
        {
            damage = 0;
        }
        currentHitPoints = currentHitPoints-damage;
        lastAction = new WoundAction( this, location );
        if( currentHitPoints <= 0 )
        {
            alive = false;
            lastAction = new DeadAction( this, new Point( location ) );
            maximumActionPoints = 0;
            currentActionPoints = 0;
        }
        return alive;
    }

    public void setDisabled( boolean disabled )
    {
        this.disabled = disabled;
        if( disabled )
        {
            alive = false;
            lastAction = new DeadAction( this, new Point( location ) );
        }
    }

    public boolean getDisabled()
    {
        return disabled;
    }
        
    public int getSize()
    {
        return 45;
    }
    
}