// BattleRobotSprite.java
// Started on March 20th, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import java.util.Vector;

public class BattleRobotSprite extends CharacterSprite
{

    private static Image [][] images = null;
    private static Image [] corpses = null;

    private int corpse = -1;
    private int shootFrame=0;

    public BattleRobotSprite()
    {
        if( images == null )
        {
            images = new Image[8][5];
            Vector temp = new Vector();
            ResourceManager.cropImage( "BattleRobot.png", temp, 96 );
            // Walking frames
            images[ Map.NORTH ][0] = (Image)temp.elementAt( 1 );
            images[ Map.NORTH_WEST ][0] = (Image)temp.elementAt( 0 );
            images[ Map.NORTH_EAST ][0] = (Image)temp.elementAt( 2 );
            images[ Map.WEST ][0] = (Image)temp.elementAt( 6 );
            images[ Map.EAST ][0] = (Image)temp.elementAt( 8 );
            images[ Map.SOUTH_WEST ][0] = (Image)temp.elementAt( 12 );
            images[ Map.SOUTH ][0] = (Image)temp.elementAt( 13 );
            images[ Map.SOUTH_EAST ][0] = (Image)temp.elementAt( 14 );
            // Shooting frames
            images[ Map.NORTH ][1] = (Image)temp.elementAt( 4 );
            images[ Map.NORTH_WEST ][1] = (Image)temp.elementAt( 3 );
            images[ Map.NORTH_EAST ][1] = (Image)temp.elementAt( 5 );
            images[ Map.WEST ][1] = (Image)temp.elementAt( 9 );
            images[ Map.EAST ][1] = (Image)temp.elementAt( 11 );
            images[ Map.SOUTH_WEST ][1] = (Image)temp.elementAt( 15 );
            images[ Map.SOUTH ][1] = (Image)temp.elementAt( 16 );
            images[ Map.SOUTH_EAST ][1] = (Image)temp.elementAt( 17 );
            // Shooting frames (2)
            images[ Map.NORTH ][2] = (Image)temp.elementAt( 19 );
            images[ Map.NORTH_WEST ][2] = (Image)temp.elementAt( 18 );
            images[ Map.NORTH_EAST ][2] = (Image)temp.elementAt( 20 );
            images[ Map.WEST ][2] = (Image)temp.elementAt( 24 );
            images[ Map.EAST ][2] = (Image)temp.elementAt( 26 );
            images[ Map.SOUTH_WEST ][2] = (Image)temp.elementAt( 30 );
            images[ Map.SOUTH ][2] = (Image)temp.elementAt( 31 );
            images[ Map.SOUTH_EAST ][2] = (Image)temp.elementAt( 32 );
             // Melee frames
            images[ Map.NORTH ][3] = (Image)temp.elementAt( 36 );
            images[ Map.NORTH_WEST ][3] = (Image)temp.elementAt( 37 );
            images[ Map.NORTH_EAST ][3] = (Image)temp.elementAt( 38 );
            images[ Map.WEST ][3] = (Image)temp.elementAt( 42 );
            images[ Map.EAST ][3] = (Image)temp.elementAt( 44 );
            images[ Map.SOUTH_WEST ][3] = (Image)temp.elementAt( 48 );
            images[ Map.SOUTH ][3] = (Image)temp.elementAt( 49 );
            images[ Map.SOUTH_EAST ][3] = (Image)temp.elementAt( 50 );
            // Wounded frames
            images[ Map.NORTH ][4] = (Image)temp.elementAt( 22 );
            images[ Map.NORTH_WEST ][4] = (Image)temp.elementAt( 21 );
            images[ Map.NORTH_EAST ][4] = (Image)temp.elementAt( 23 );
            images[ Map.WEST ][4] = (Image)temp.elementAt( 27 );
            images[ Map.EAST ][4] = (Image)temp.elementAt( 29 );
            images[ Map.SOUTH_WEST ][4] = (Image)temp.elementAt( 33 );
            images[ Map.SOUTH ][4] = (Image)temp.elementAt( 34 );
            images[ Map.SOUTH_EAST ][4] = (Image)temp.elementAt( 35 );
            corpses = new Image[4];
            corpses[0] = (Image)temp.elementAt( 7 );
            corpses[1] = (Image)temp.elementAt( 10 );
            corpses[2] = (Image)temp.elementAt( 25 );
            corpses[3] = (Image)temp.elementAt( 28 );
        }
        location = new Point( 0, 0 );
        bounds = new Rectangle( 0, 0, 96, 96 );
        direction = Map.NORTH;
    }

    public Image getSprite( GameAction action )
    {
        if( action instanceof ShootAction )
        {
            int theFrame=1+(shootFrame % 2 );
            return images[ direction ][ theFrame ];
        }
        if( action instanceof MeleeAction )
        {
            return images[ direction ][3];
        }
        if( action instanceof WoundAction )
        {
            return images[ direction ][4];
        }
        if( action instanceof DeadAction )
        {
            if( corpse < 0 )
            {
                corpse = (int)(Math.random()*3);
            }
            return corpses[ corpse ];
        }
        return images[ direction ][0];

    }

    public Image getSprite()
    {
        // Ok, this is ugly. We only want to show the corpse for BattleRobots until they reactivate.
        if( ((BattleRobot)character).hasReactivated() )
        {
            corpse = 3;
        }
        if( corpse < 0 )
        {
            corpse = (int)(Math.random()*3);
        }
        return corpses[ corpse ];        
    }

    public boolean isAnimated()
    {
        return true;
    }

    public void setFrame( int frame )
    {
        shootFrame = frame;
    }

    public void setLocation( Point location)
    {
        this.location.x = location.x-24;
        this.location.y = location.y-24;
        bounds.x = location.x;
        bounds.y = location.y;
    }

    public void setLocation( int x, int y )
    {
        location.x = x-24;
        location.y = y-24;
        bounds.x = x-24;
        bounds.y = y-24;
    }

    public int getNumberOfRangedAttackFrames()
    {
        return 7;
    }
}