// Bob.java
// Started on February 28th 5th, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class Bob extends Monster implements Subordinate
{
    public static final int ROAMING = 0;
    public static final int SHELTERING = 1;
    public static final int ESCAPING = 2;
    public static final int WAITING = 3;
    public static final int PANICKING = 4;
    public static final int DISABLED = -1;

    public static final int FOLLOW = 0;
    public static final int ESCAPE = 1;
    public static final int AVOID = 2;

    public static final int FOLLOW_DISTANCE = 11;
    public static final int PROXIMITY_ROAM_THRESHOLD = 2;
    
    //private static int [][] helper = null;
    
    private String [] foeTypes = { "Ripper", "Brute", "BomberBrute", "CyberBrute", "ShotgunBrute", "Slasher", "Juve", "FlamerBrute" };
    private int state = ROAMING;
    private boolean hasMoreActions = false;
    private int behavior = FOLLOW;
    private Point exit = null;
    private Point target = null;
    private Character leader = null;
    private Character [] oldVisibleEnemies = new Character[0];
    private String type = "Bob";

    public Bob()
    {
        location = new Point( 0, 0 );
        direction = Map.NORTH;
        alive = true;
        lastAction = new WaitAction( this, location );
        maximumActionPoints = 6;
        currentActionPoints = 6;
        maximumHitPoints = 8;
        currentHitPoints = 8;
    }

    public int getBehavior()
    {
        return behavior;
    }
    
    public void setBehavior( int behavior )
    {
        this.behavior = behavior;
    }

    public void disable()
    {
        location = new Point( -10, -10 );
        maximumActionPoints = 0;
        currentActionPoints = 0;
        hasMoreActions = false;
        state = DISABLED;
        lastAction = new MoveAction( this, location, location );
    }
    
    public boolean move()
    {	
	int newDirection = direction;
	boolean moved = false;

	if( target != null)
	{
	    // Run to the target
	    GameAction action = SeekerAI.seek( this, target );
	    if( action != null )
	    {
		if( action instanceof MoveAction )
		{
		    newDirection = Map.getDirection( location, ((MoveAction)action).getEndPoint(), direction );
		    activeGame.handleMessage( new Message( this, "\"Yes, ser!\"" ) );
		    activeGame.moveCharacter( this, newDirection );
		    moved = true;
		}
	    }
	}
	else
	{
	    if( Math.random() < 0.4 )
	    {
		newDirection = (int)(Math.random()*8);   
	    }
	    for( int i=0; (i < 8) && !moved; i++ )
	    {
		if( activeGame.canCharacterMove( this, newDirection, false ) )
		{
		    moved = true;
		    activeGame.handleMessage( new Message( this, "\"Yes, ser!\"" ) );
		    activeGame.moveCharacter( this, newDirection );
		}
		newDirection = (newDirection+1)%8;
	    }
	}
	if( !moved )
	{
	    activeGame.handleOutOfAP( this );
	}
	return moved;
    }
    
    public void setSpottedPlayer( Character character, boolean canSee )
    {
	super.setSpottedPlayer( character, canSee );

        if( behavior == ESCAPE && canSee )
        {
	    if( exit == null )
	    {            
		state = ESCAPING;
		exit = new Point( ((MapNode)activeGame.getPlayerStartingPoints().elementAt( 0 )).getPoint() );
		target = exit;
	    }
        }
        if( behavior == FOLLOW && canSee )
        {
            if( leader == null )
            {
                state = SHELTERING;
                leader = character;
            }
        }
    }

    public void setTraitor()
    {
	type = "TraitorBob";
	String [] newFoes = { "Marine" };
        foeTypes = newFoes;
    }
    
    public boolean isFoe( Character character )
    {
        for( int i = 0; i < foeTypes.length; i++ )
        {
            if( character.getType().equalsIgnoreCase( foeTypes[ i ] ) )
	    {
                return true;
	    }
        }
        return false;
    }
    
    public void preAction()
    {
	if( alive && state != DISABLED )
	{
	    Character [] visibleEnemies = activeGame.getVisibleEnemies( this );
	    if( visibleEnemies.length > 0 )
	    {
		if( behavior != ESCAPE )
		{
		    state = SHELTERING;
		    target = SeekerAI.getSafePointForCharacter( this );
		    if( target == null )
		    {
			state = PANICKING;
		    }
		}
	    }
	    else
	    {
		if( state != ESCAPING )
		{
		    state = ROAMING;
		}
	    }
	    if( behavior == ESCAPE && exit != null )
	    {
		state = ESCAPING;
		target = exit;
	    }
	}
    }
    
    public GameAction playAction()
    {
        int x;
        int y;

        Point cell;
        Character [] visibleEnemies;
        if( state == DISABLED )
        {
            return lastAction;
        }
        if( alive )
        {
            hasMoreActions = true;
            // Preconditions
            if( currentActionPoints <= 0 )
            {
                hasMoreActions = false;
                lastAction = new WaitAction( this, location );
                return lastAction;
            }
            visibleEnemies = activeGame.getVisibleEnemies( this );
            if( behavior == FOLLOW && visibleEnemies.length < 1 )
            {
                if( activeGame.isCellVisibleToPlayer( location ) )
                {
                    state = WAITING;
                    if( leader != null )
                    {
                        if( location.distance( leader.getLocation() ) > FOLLOW_DISTANCE )
                        {
                            state = SHELTERING;
                            target = leader.getLocation();
                        }
                    }
                }
                else
                {
                    if( leader != null )
                    {
                        if( leader.isAlive() )
                        {
                            state = SHELTERING;
                            target = leader.getLocation();
                        }
                        else
                        {
                            leader = null;
                            state = ROAMING;
                        }
                    }
                    else
                    {
                        state = ROAMING;
                    }
                }
            }
	    if( state == WAITING || state == ROAMING )
            {
                if( visibleEnemies.length > 0 )
                {
                    state = SHELTERING;
		    target = SeekerAI.getSafePointForCharacter( this );
		    if( target == null )
		    {
			//System.out.println( "No safe point found for Bob on " + Objective.pointToString( location ) );
			state = PANICKING;
		    }
                }
            }	    
            if( state == ESCAPING || state == SHELTERING )
            {
                if( target == null || location.equals( target ) )
                {
                    state = WAITING;
                }
            }
            if( state == PANICKING )
            {
                if( visibleEnemies.length == 0 )
                {
                    state = ROAMING;
                }
            }
            // Actual actions
            if( state == ESCAPING || state == SHELTERING )
            {
                // Run to the target
                GameAction action = SeekerAI.seek( this, target );
                if( action != null )
                {
		    //System.out.println( "Bob sheltering to " + Objective.pointToString( target ) );
                    lastAction = action;
                    return lastAction;
                }
                else
                {
		    lastAction = RoamerAI.roam( this, 0.15 );
                    return lastAction;
                }
            }
            if( state == WAITING )
            {
                // Generally stand in place, sometimes moving around
                if( Math.random() < 0.25 )
                {
		    lastAction = RoamerAI.roam( this, 0.15 );
                }
                else
                {
                    // Turn to face the closest friend
                    Character friend = null;
		    friend = activeGame.getClosestCharacter( this, activeGame.getPlayerCharacters() );
		    if( friend != null && !friend.isHostile( this ) )
		    {
			direction = Map.getDirection( location, friend.getLocation(), direction );
			double range = friend.getLocation().distance( location );
			if( ( range <= PROXIMITY_ROAM_THRESHOLD ) && ( Math.random() < 0.75 ) )
			{
			    // Roam around a little, so less likely to stop path
			    //System.out.println( "Bob: Proximity roam" );
			    lastAction = RoamerAI.roam( this, 0.75 );
			}
                    }
		    //System.out.println( "Bob waiting" );
		    lastAction = new WaitAction( this, new Point( location ) );
		    hasMoreActions = false;
		    currentActionPoints = 0;
                }
            }
            if( state == PANICKING )
            {
                // Run away from the closest enemy
                Character check;
                Character target = this;
                double range = Double.MAX_VALUE;
                double checkRange;
                int newDirection = -1;
                int tempDirection = -1;
                for( int i = 0; i < visibleEnemies.length; i++ )
                {
                    check = visibleEnemies[ i ];
                    checkRange = location.distance( check.getLocation() );
                    if( checkRange < range && check.isAlive() )
                    {
                        target = check;
                        range = checkRange;
                    }
                }
                tempDirection = Map.getDirection( location, target.getLocation(), direction );
                // Check the opposite direction
                newDirection = (tempDirection+4)%8;
                if( activeGame.getMap().isPassable( location, newDirection ) )
                {
                    // This will do
                    direction = newDirection;
                }
                else
                {
                    newDirection = (tempDirection+3)%8;
                    if( activeGame.getMap().isPassable( location, newDirection ) )
                    {
                        direction = newDirection;
                    }
                    else
                    {
                        newDirection = (tempDirection+5)%8;
                        if( activeGame.getMap().isPassable( location, newDirection ) )
                        {
                            direction = newDirection;
                        }
                        else
                        {
                            newDirection = -1;
                        }
                    }
                }
                // If found suitable escape direction, run there
                if( newDirection > -1 )
                {
                    System.out.println( "Bob panicking from (" + location.x +","+location.y +") towards " +newDirection );
                    lastAction = new MoveAction( this, new Point( location ), activeGame.getMap().getNeighbor( location, direction ));
                    return lastAction;
                }
                else
                {
                    state = ROAMING;
                }
            }
            if( state == ROAMING )
            {
		lastAction = RoamerAI.roam( this, 0.15 );
            }
        }
        return lastAction;
    }
    
    public void postAction()
    {
	if( state == SHELTERING || state == PANICKING )
	{
	    Character [] visibleEnemies = activeGame.getVisibleEnemies( this );
	    if( visibleEnemies.length <= 0 )
	    {
		state = ROAMING;
		target = null;
	    }
	}
    }
    
    /*

    private Point [] findSafeSpots( Character [] visibleEnemies )
    {
        resetHelper();
        Vector safeArea = new Vector();
        Vector front = new Vector();
        Vector newFront;
        front.add( new Point( location ) );
        helper[ location.x ][ location.y ] = 0;
        Point point;
        Point check;

        //System.out.println( "Bob has " + visibleEnemies.length +" visible enemies" );
        int suitableRange = maximumActionPoints*2;
        for( int p=0; p < suitableRange; p++ )
        {
            newFront = new Vector();
	    boolean safe = true;
            for( int i=0; i < front.size(); i++ )
            {
                point = (Point)front.elementAt( i );
                // Check if the point is safe
		safe = true;
                for( int enemy = 0; enemy < visibleEnemies.length; enemy++ )
                {
                    if( activeGame.isCellVisibleToCharacter( visibleEnemies[ enemy ], point ) )
                    {
			safe = false;
                    }
                }
		if( safe )
		{
		    safeArea.add( new Point( point ) );   
		}
                // Check the neighbours and mark them if necessary
                for( int j = 0; j < 8; j++ )
                {
                    check = activeGame.getMap().getNeighbor( point, j );
                    if( activeGame.getMap().isPassable( check, true ) && helper[ check.x ][ check.y ] == -1 )
                    {
                        newFront.add( new Point( check ) );
                        helper[ check.x ][ check.y ] = p;
                    }
                }
            }
            front = newFront;
        }
        if( safeArea.size() > 0 )
        {
            Point [] safePoints = new Point[ safeArea.size() ];
            for( int i = 0; i < safeArea.size(); i++ )
            {
                safePoints[i] = (Point)safeArea.elementAt(i);
            }
            return safePoints;
        }
        return null;
    }*/

    /*
    public void resetHelper()
    {
        for( int i=0; i < activeGame.getMap().getHorizontalSize(); i++ )
            for( int j=0; j < activeGame.getMap().getVerticalSize(); j++ )
                helper[i][j]=-1;
    }*/
        
    public void setTarget( Point point )
    {
	target = point;
	if( target != null )
	{
	    state = SHELTERING;
	}
	else
	{
	    state = ROAMING;   
	}
    }
    
    public Point getTarget()
    {
	return target;
    }
    
    public String getType()
    {
	return type;
    }

    public GameAction getLastAction()
    {
        //System.out.println( "Get action from Bob" );
        return lastAction;
    }
    
    public int getMeleeAbility()
    {
	return 1;
    }
    
    public int getMeleeCost()
    {
	return 1;
    }

    public void resetActionsPoints()
    {
        currentActionPoints = maximumActionPoints;
	oldVisibleEnemies = activeGame.getVisibleEnemies( this );
    }

    public boolean hasMoreActions()
    {
        if( currentActionPoints > 0 )
        {
            //System.out.println( "Bob has " + currentActionPoints + " action points left, attacking: " + attacking +", seeking: " + seeking );
            return true;
        }
        return false;
    }

    public boolean applyDamage( int damage )
    {
        currentHitPoints = currentHitPoints-damage;
        lastAction = new WoundAction( this, location );
        if( currentHitPoints <= 0 )
        {
            alive = false;
            lastAction = new DeadAction( this, new Point( location ) );
            maximumActionPoints = 0;
            currentActionPoints = 0;
        }
        return alive;
    }

    public int getSize()
    {
        return 70;
    }

}