// CommSentry.java
// Started on November 24th, 2001
// Copyright (c) Virtu Halttunen

// Can only be used with SabotageCommObjective

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class CommSentry extends Monster
{
    private boolean alarmed = false;
    private Point start;
    private Point target;
    private Point objective;

    public CommSentry()
    {
	location = new Point( 0, 0 );
	direction = Map.NORTH;
	alive = true;
	lastAction = new MoveAction( this, location, location );
	maximumActionPoints = 5;
	currentActionPoints = 5;
	maximumHitPoints = 3;
	currentHitPoints = 3;
    }
    
    public void setPath( Point start, Point target )
    {
        this.start = new Point( start );
        this.target = new Point ( target );
        this.objective = target;
    }

    public boolean isAlarmed()
    {
        return alarmed;
    }
    
    public void setObjective( Point objective )
    {
        this.objective = objective;
    }
  
    public void setSpottedPlayer( Character character, boolean canSee )
    {
	super.setSpottedPlayer( character, canSee );

        if( canSee )
            alarmed = true;
    }
    
    public GameAction playAction()
    {
	int x; 
	int y;

	Point cell;
	
        Monster monster;
	
	if( alive )
	{
            if( currentActionPoints > 0 )
            {
                // Check if needs to be alarmed
                // First, check the players
                if( activeGame.isCellVisibleToPlayer( location ) )
                {
                    alarmed = true;
                }
                // Then check for Sentry corpses
                for( int i=0; i < activeGame.getMonsters().size(); i++ )
                {
                    monster = activeGame.getMonster( i );
                    if( monster instanceof CommSentry )
                    {
                        if( !monster.isAlive() )
                        {
                            if( activeGame.getMap().hasLineOfSight( location, monster.getLocation() ) )
                            {
                                alarmed = true;
                            }
                        }
                    }
                }
                if( !alarmed )
   		{
                    // If has reached target location, change start to target
                    if( location.x == target.x && location.y == target.y )
                    {
                        Point temp = target;
                        target = start;
                        start = temp;
                    }
                }
                if( alarmed )
                {
                    target = objective;
                    // If next to comm device, wait
                    if( ( Math.abs( location.x - target.x ) <= 1 ) && ( Math.abs( location.y - target.y ) <= 1 ) )
                    {
                        currentActionPoints = 0;
                        lastAction = new MoveAction( this, new Point( location ), new Point( location ) );
                        return lastAction;
                    }
                }
                // Walk towards target
                //System.out.println( "CommSentry walking from: " + location.toString() + " to " + target.toString() );
                Vector path = activeGame.getMap().findShortestPath( this, target, true );
                if( path.size() >= 2 )
                {
                    cell = ((MapNode)path.elementAt( 1 )).getPoint();
                    if( activeGame.getMap().isClosedDoor( cell ) )
                    {
                        //lastAction = new GameAction( this, Game.USE, new Point( location ), new Point( cell ) );
                        //activeGame.openDoor( cell );
			lastAction = new UseAction( this, cell );
                        return lastAction; 
                    }

                    direction = Map.getDirection( location, cell, direction );
                    lastAction = new MoveAction( this, new Point( location ), new Point( cell ) );
                    //System.out.println( "CommSentry walking from " + location.toString() + " to " + cell.toString() );
                    location = new Point( cell );
                    return lastAction;
                }
                else
                {
                    currentActionPoints = 0;
                    lastAction = new MoveAction( this, new Point( location ), new Point( location ) );
                }
            }
	    else
	    {
                currentActionPoints = 0;
		lastAction = new MoveAction( this, new Point( location ), new Point( location ) );
	    }
	}
	return lastAction;
    }
    
    public GameAction getLastAction()
    {
	//System.out.println( "Get action from CommSentry " +index );
	return lastAction;
    }
    
    public int getMeleeAbility()
    {
	return 2;
    }
        
    public void resetActionsPoints()
    {
	currentActionPoints = maximumActionPoints;
    }
    
    public boolean hasMoreActions()
    {
	if( currentActionPoints > 0 )
	{
	    //System.out.println( "ComMSentry has " + currentActionPoints + " action points left" );
	    return true;
	}
	return false;   
    }

    public boolean applyDamage( int damage )
    {
	currentHitPoints = currentHitPoints-damage;
	lastAction = new MoveAction( this, location, location );
	if( currentHitPoints <= 0 )
	{
	    alive = false;
	    lastAction = new DeadAction( this, new Point( location ) );
	    maximumActionPoints = 0;
	    currentActionPoints = 0;
	}
	return alive;
    }


}