// CommandBobsObjective.java
// Started on February 21st, 2004
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;

public class CommandBobsObjective extends Objective
{
    private Character selected;
    private long lastSelectionTime = System.currentTimeMillis();
    private CommandBobsObjectiveSprite targetSprite;
    private CommandBobsObjectiveSprite targetPointSprite;
    
    public CommandBobsObjective()
    {
	targetSprite = new CommandBobsObjectiveSprite( true );
	targetPointSprite = new CommandBobsObjectiveSprite( false );
    }
    
    public boolean isLocatedIn( Point point )
    {
	if( selected != null && selected instanceof Subordinate )
	{
	    if( point.equals( ((Subordinate)selected).getTarget() ) )
	    {
		return true;
	    }
	}
	return false;
	
    }
    
    public int getNumberOfSprites()
    {
	return 2;
    }
    
    public Sprite getSprite( int x, int y )
    {
	if( ( selected != null ) && ( selected instanceof Subordinate ) )
	{
	    Point target = ((Subordinate)selected).getTarget();
	    if( target != null )
	    {
		if( ( target.x == x ) && ( target.y == y ) )
		{
		    return targetPointSprite;
		}
	    }
	    if( ( selected.getLocation().x == x ) && ( selected.getLocation().y == y ) )
	    {
		return targetSprite;
	    }	    
	}
        return null;
    }
    
    public boolean isLocatedIn( int x, int y )
    {
	if( ( selected != null ) && ( selected instanceof Subordinate ) )
	{
	    Point target = ((Subordinate)selected).getTarget();
	    if( target != null )
	    {
		if( ( target.x == x ) && ( target.y == y ) )
		{
		    return true;
		}
	    }
	    if( ( selected.getLocation().x == x ) && ( selected.getLocation().y == y ) )
	    {
		return true;
	    }	    
	}
	return false;
    }
    
    public boolean isShownOnMap()
    {
	if( selected != null && selected instanceof Subordinate )
	{
	    return true;
	}
	return false;
    }
    
    public boolean isUsable()
    {
	return true;
    }
    
    public boolean isRemoteControlled()
    {
	return true;
    }
    
    public boolean handleAction( GameAction action )
    {
        boolean handled = false;
        Marine user;
	Character character = null;
	Character chosen = null;
        if( action instanceof UseAction )
        {
            if( ((UseAction)action).getActor() instanceof Marine )
            {
                user = (Marine)((UseAction)action).getActor();
		Point location = ((UseAction)action).getPoint();
		
		for( int i=0; (i < activeGame.getMonsters().size() ) && (chosen == null ); i++ )
		{
		    character = activeGame.getMonster( i );
		    if( character instanceof Subordinate && character.getLocation().equals( location ) && activeGame.getMap().hasLineOfSight( user.getLocation(), location ) )
		    {
			if( !character.isHostile( user ) && character.isAlive() )
			{
			    System.out.println( "Commanding " + character.getClass().getName() );
			    chosen = character;
			}
		    }
		}
		if( chosen != null )
		{
		    long currentTime = System.currentTimeMillis();
		    boolean clear = false;
		    if( chosen == selected )
		    {
			if( currentTime-lastSelectionTime < 750 )
			{
			    ((Subordinate)chosen).move();
			}
		    }
		    selected = chosen;
		    lastSelectionTime = currentTime;
		    ((UseAction)action).setConsumed( true );
		    handled = true;
		    return true;
		}
		if( chosen == null && selected != null && selected instanceof Subordinate )
		{
		    if( activeGame.getMap().isPassable( location ) )
		    {
			((Subordinate)selected).setTarget( location );
			activeGame.handleMessage( new Message( user, "New target given" ) );
		    }
		    else
		    {
			((Subordinate)selected).setTarget( null );
			activeGame.handleMessage( new Message( user, "Target cleared" ) );
			selected = null;
		    }
		    ((UseAction)action).setConsumed( true );
		    handled = true;
		}
	    }
        }
	else
	{
	    selected = null;
	}
        return handled;
    }
    
    public String [] getPropertyNames()
    {
	return null;
    }
    
    public String getProperty( String key )
    {
	return null;
    }
    
    public void setProperty( String key, String value )
    {}
}