// DestroyerRobot.java
// Started on September 13th, 2003
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class DestroyerRobot extends Monster implements Armed, Robot
{
    public static final int ROAMING = 0;
    public static final int SEEKING = 1;
    public static final int ATTACKING = 2;
    public static final int SHOOTING = 3;

    private int state = ROAMING;
    private Character target = null;
    private boolean hasMoreActions = false;
    private boolean blasted = false;
    private boolean disabled = false;
    private int skill = 2;
    private Weapon weapon;

    public DestroyerRobot()
    {
        location = new Point( 0, 0 );
        direction = Map.NORTH;
        alive = true;
        lastAction = new MoveAction( this, location, location );
        maximumActionPoints = 4;
        currentActionPoints = 4;
        maximumHitPoints = 10;
        currentHitPoints = 10;
        weapon = new SmartGun();
        if( Math.random() < 0.25 )
        {
            direction = Map.NORTH_EAST;
        }
    }

    public void setSpottedPlayer( Character character, boolean canSee )
    {
	super.setSpottedPlayer( character, canSee );
        if( canSee )
        {
            target = character;
            state = SEEKING;
            //System.out.println( "DestroyerRobot spotted player: " + index );
        }
    }


    public GameAction playAction()
    {
        int x;
        int y;

        Point cell;
        Character[] enemies;

        if( alive && !disabled )
        {
            if( currentActionPoints > 0 )
            {
                hasMoreActions = true;
                enemies = activeGame.getVisibleEnemies( this );
                if( target != null )
                {
                    if( !target.isAlive() )
                    {
                        target = null;
                        state = ROAMING;
                    }
                    else
                    {
                        state = SEEKING;
                    }
                }
                if( target == null && enemies.length > 0 )
                {
                    // Select an enemy and start seeking
                    int newTarget = (int)(Math.random()*enemies.length);
                    target = enemies[ newTarget ];
                    state = SEEKING;
                 }
                
                // If not next to an enemy, but can see one, shoot it
                if( enemies.length > 0 )
                {
                    target = activeGame.getClosestCharacter( this, enemies );
                    if( activeGame.getMap().getLineOfFire( location, target.getLocation(), weapon.getBlockingSize() ).equals( target.getLocation() ) && ( location.distance( target.getLocation() ) <= 16 ) && currentActionPoints > 2 )
                    {
                        state = SHOOTING;
                        currentActionPoints--; // Let's decrease this here, so the robot will shoot less
                        direction = Map.getDirection( location, target.getLocation(), direction );
                        lastAction = new ShootAction( this, weapon, target.getLocation(), true );
                        return lastAction;
                    }
                    else
                    {
                        state = SEEKING;
                    }
                }                    
                if( target != null )
                {
                    Vector path = activeGame.getMap().findShortestPath( this, target.getLocation(), false );
                    if( path.size() >= 4 )	// Destroyer robots try to avoid close combat
                    {
                        cell = ((MapNode)path.elementAt( 1 )).getPoint();
                        direction = Map.getDirection( location, cell, direction );
                        lastAction = new MoveAction( this, new Point( location ), new Point( cell ) );
                        return lastAction;
                    }
                    else
                    {
                        state = ROAMING;
                        target = null;
                    }
                }
                else
                {
                    state = ROAMING;
                }
                if( state == ROAMING )
                {
                    // Move
                    if( activeGame.canCharacterMove( this, direction, false ) )
                    {
                        cell = activeGame.getMap().getNeighbor( location, direction );
                        lastAction = new MoveAction( this, new Point( location ), new Point( cell ) );
                        return lastAction;
                    }
                    direction = (int)(Math.random()*8);
                }
            }
            lastAction = new MoveAction( this, new Point( location ), new Point( location ) );
        }
        return lastAction;
    }

    public GameAction getLastAction()
    {
        return lastAction;
    }

    public void resetActionsPoints()
    {
        currentActionPoints = maximumActionPoints;
    }

    public boolean hasMoreActions()
    {
        if( currentActionPoints > 0 )
        {
            return true;
        }
        return false;
    }

    public boolean applyDamage( int damage )
    {
        damage = damage-1;
        if( damage < 0 )
        {
            damage = 0;
        }
        currentHitPoints = currentHitPoints-damage;
        lastAction = new WoundAction( this, location );
        if( currentHitPoints <= 0 )
        {
            alive = false;
            if( !blasted && Math.random() < 0.33 )
            {
                weapon = new AssaultRifle();
                weapon.setFiringMode( AssaultRifle.GRENADE );
                currentActionPoints = weapon.getActionsToFire();
                activeGame.handleRangedAttack( this, location );
            }
            blasted = true;
            lastAction = new DeadAction( this, new Point( location ) );
            maximumActionPoints = 0;
            currentActionPoints = 0;
        }
        return alive;
    }

    public int getWeaponSkill()
    {
        return skill;
    }

    public Weapon getWeapon()
    {
        return weapon;
    }

    public void setWeapon( Weapon weapon )
    {}

    public void registerHit()
    {}
    
    public void addKill()
    {}
    
    public void setDisabled( boolean disabled )
    {
        this.disabled = disabled;
        if( disabled )
        {
            alive = false;
            lastAction = new DeadAction( this, new Point( location ) );
        }
    }

    public boolean getDisabled()
    {
        return disabled;
    }
    
    public int getSize()
    {
        return 90;
    }

}