// DisplayMessageObjective.java
// Started on August 2nd, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.io.*;
import java.util.Vector;

/**
 *
 *  This Objective can be used to display a message when the player sees a certain spot.
 *
 */
public class DisplayMessageObjective extends Objective
{    
    public static final String MESSAGE_KEY = "Message";
    public static final String ANNOYING_KEY = "Annoying";
    public static final String [] properties = { Objective.LOCATION_KEY, MESSAGE_KEY, ANNOYING_KEY };
    
    private boolean triggered = false;
    private Point location = new Point( -1, -1 );
    private String message = "It's a trap!";
    private boolean annoying = false;
    
    public DisplayMessageObjective()
    {
    }
    
    public void setGame( Game activeGame )
    {
	this.activeGame = activeGame;
    }
        
    public int getNumberOfSprites()
    {
	return 0;
    }    
    
    public Sprite getSprite( int x, int y)
    {
	return null;
    }
    
    public String [] getPropertyNames()
    {
	return properties;
    }
    
    public Class getPropertyClass( String key )
    {
	if( key.equals( Objective.LOCATION_KEY ) )
	{
	    return Point.class;
	}
	return String.class;
    }
    
    public String getProperty( String key )
    {
	if( key.equals( Objective.LOCATION_KEY ) )
	{
	    return Objective.pointToString( location );
	}
	if( key.equals( MESSAGE_KEY ) )
	{
	    return message;
	}
	if( key.equals( ANNOYING_KEY ) )
	{
	    return String.valueOf( annoying );
	}
	return null;
    }
    
    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
	if( key.equals( Objective.LOCATION_KEY ) )
	{
	    location = Objective.pointFromString( value );
	}	
	if( key.equals( MESSAGE_KEY ) )
	{
	    if( value != null )
	    {
		message = value;
	    }
	}
	if( key.equals( ANNOYING_KEY ) )
	{
	    if( value != null )
	    {
		annoying = new Boolean( value ).booleanValue();
	    }
	}
    }    
    
    public boolean isCompleted()
    {
	return true;
    }
    
    public int getSize()
    {
	return 0;
    }
    
    public boolean isLocatedIn( Point point )
    {
	return location.equals( point );
    }
    
    public boolean isLocatedIn( int x, int y )
    {
	return isLocatedIn( new Point( x, y ) );
    }
    
    public boolean isDestructible()
    {
	return false;
    }
    
    public boolean isShownOnMap()
    {
	return false;
    }
    
    public boolean handleDamage( Point point, int damage )
    {
	return false;
    }
    
    public boolean handleAction( GameAction action )
    {
	if( action instanceof MoveAction && !triggered )
	{
	    MoveAction mAction = (MoveAction)action;
	    if( activeGame.isPlayerCharacter( mAction.getActor() ) )
	    {
		if( activeGame.isCellVisibleToCharacter( mAction.getActor(), location ) )
		{
		    String messageString = message.replaceAll( "\\\\n", "\n" );
		    Message theMessage = new Message( mAction.getActor(), messageString );
		    theMessage.setAnnoying( annoying );
		    activeGame.handleMessage( theMessage );
		    triggered = true;
		}
	    }
	}
	return false;
    }
}