// FlamerBrute.java
// Started on September 5th, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class FlamerBrute extends Monster implements Armed
{
    public static final int ROAMING = 0;
    public static final int SEEKING = 1;
    public static final int ATTACKING = 2;
    public static final int SHOOTING = 3;
    public static final int WAITING = 4;

    private Character target = null;
    private boolean hasMoreActions = false;
    private int state = ROAMING;
    private Weapon weapon;
    private int skill = 3;
    private boolean blasted = false;

    public FlamerBrute()
    {
        location = new Point( 0, 0 );
        direction = Map.NORTH;
        alive = true;
        lastAction = new MoveAction( this, location, location );
        maximumActionPoints = 5;
        currentActionPoints = 5;
        maximumHitPoints = 6;
        currentHitPoints = 6;
        weapon = new Flamer();
    }

    public void setSpottedPlayer( Character character, boolean canSee )
    {
	super.setSpottedPlayer( character, canSee );
        if( canSee )
        {
            target = character;
            state = SEEKING;
            //System.out.println( "Brute spotted player: " + index );
        }
    }

    public void setState( int state )
    {
        this.state = state;
    }

    public Weapon getWeapon()
    {
        return weapon;
    }

    public void setWeapon( Weapon weapon )
    {}    
    
    public GameAction playAction()
    {
        int x;
        int y;

        Point cell;
        Vector enemies = new Vector();
        Character check;

        hasMoreActions = true;

        if( alive )
        {
            for( int i=0; i < activeGame.getPlayerCharacters().size(); i++ )
            {
                check = activeGame.getPlayerCharacter( i );
                if( activeGame.getMap().hasLineOfSight( check.getLocation(), location ) )
                    enemies.add( check );
            }
            //System.out.println( "ShotgunBrute: playAction" );
            if( target != null )
                if( !target.isAlive() )
                {
                    target = null;
                    state = ROAMING;
                }
                else
                {
                    state = SEEKING;
                }
            if( currentActionPoints <= 0 )
            {
                lastAction = new MoveAction( this, new Point( location ), new Point( location ) );
                return lastAction;
            }
            // If next to an enemy, attack
            //System.out.println( "Brute checking neighbours" );
            for( int i=0; i < enemies.size(); i++ )
            {
                check = (Character)enemies.elementAt(i);
                if( check.isAlive() )
                {
                    cell = check.getLocation();
                    if( activeGame.getMap().isNeighbor( location, cell ) )
                    {
                        if( currentActionPoints >= getMeleeCost() )
                        {
                            state = ATTACKING;
                            direction = Map.getDirection( location, cell, direction );
                            lastAction = new MeleeAction( this, check );
                            return lastAction;
                        }
                        else
                        {
                            state = WAITING;
                            lastAction = new MoveAction( this, new Point( location ), new Point( location ) );
                            hasMoreActions = false;
                            return lastAction;
                        }
                    }
                }
            }
            // If not next to an enemy, but can see one, shoot it
            if( enemies.size() > 0 )
            {
                target = activeGame.getClosestCharacter( this, enemies );
                if( activeGame.getMap().getLineOfFire( location, target.getLocation(), weapon.getBlockingSize() ).equals( target.getLocation() ) && ( location.distance( target.getLocation() ) <= 6 ) && currentActionPoints > 2 )
                {
                    if( location.distance( target.getLocation() ) <= 2 )
                    {
                        state = ROAMING;
                    }
                    else
                    {
                        state = SHOOTING;
                        direction = Map.getDirection( location, target.getLocation(), direction );
                        lastAction = new ShootAction( this, weapon, target.getLocation(), true );
                        return lastAction;
                    }
                }
                else
                {
                    state = SEEKING;
                }
            }
            if( state == SEEKING )
            {
                //System.out.println( "Brute seeking targets" );
                if( target != null )
                {
                    GameAction action = SeekerAI.seek( this, target.getLocation() );
                    if( action != null )
                    {
                        lastAction = action;
                        return lastAction;
                    }
                    else
                    {
                        state = ROAMING;
                        target = null;
                    }
                }
            }
            if( state == ROAMING )
            {
                // Roam around mindlessly like a common Oid
                do
                {
                    x = location.x;
                    y = location.y;
                    x = x + (int)(Math.random()*3)-1;
                    y = y + (int)(Math.random()*3)-1;
                } while( !activeGame.getMap().isPassable( x, y, false ) );
		lastAction = new MoveAction( this, new Point( location ), new Point( x, y ) );
                //System.out.println( "Brute roaming" );
            }
        }
        return lastAction;
    }

        public GameAction getLastAction()
        {
            //System.out.println( "Get action from Brute " +index );
            return lastAction;
        }

        public void resetActionPoints()
        {
            currentActionPoints = maximumActionPoints;
        }

        public boolean hasMoreActions()
        {
            if( currentActionPoints > 0 && hasMoreActions )
            {
                //System.out.println( "Brute has " + currentActionPoints + " action points left, attacking: " + attacking +", seeking: " + seeking );
                return true;
            }
            return false;
        }

	public int getMeleeAbility()
	{
	    return 3;   
	}
	
	public int getMeleeCost()
	{
	    return 2;
	}
	
        public int getWeaponSkill()
        {
            return skill;
        }

        public boolean isAlive()
        {
            return alive;
        }

        public boolean isBlasted()
        {
            return blasted;
        }

	public void addKill()
	{}
	
	public void registerHit()
	{}	
	
        public boolean applyDamage( int damage )
        {
            currentHitPoints = currentHitPoints-damage;
            lastAction = new WoundAction( this, location );
            if( ( damage > 4+(int)(Math.random()*6) ) && !blasted )
            {
                // We'll explode!
                blasted = true;
                currentActionPoints = weapon.getActionsToFire();
                currentHitPoints = 0;
                activeGame.handleRangedAttack( this, location );
            }
            if( currentHitPoints <= 0 )
            {
                alive = false;
                lastAction = new DeadAction( this, new Point( location ) );
                maximumActionPoints = 0;
                currentActionPoints = 0;
            }
            return alive;
        }

        public int getSize()
        {
            return 75;
        }
}