// FlamerSprite.java
// Started on June 23rd, 2003
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Image;
import java.util.Vector;
import java.awt.Point;
import java.awt.Graphics;
import java.applet.AudioClip;
import javax.swing.RepaintManager;
import javax.swing.ImageIcon;


public class FlamerSprite extends WeaponSprite
{
    private static AudioClip fire = null;

    private static Image flamerFlame = null;
    private static Image [] stains = null;

    private Flamer weapon;
    private Point fireTo = new Point( 0, 0 );
    private Point [] area = null;
    

    public FlamerSprite()
    {
        if( fire == null && GameMapPanel.playSounds )
        {
            fire = ResourceManager.loadAudioClip( "grenade.wav" );
        }
        if( flamerFlame == null )
        {
            flamerFlame = ResourceManager.loadImage( "WeaponGraphics/FlamerFlame.png" );
            Vector temp = new Vector();
            ResourceManager.cropImage( "stains.png", temp, 48 );
            stains = new Image[4];
            stains[0] = (Image)temp.elementAt( 0 );
            stains[1] = (Image)temp.elementAt( 1 );
            stains[2] = (Image)temp.elementAt( 6 );
            stains[3] = (Image)temp.elementAt( 7 );        
        }
    }

    public void setWeapon( Weapon weapon )
    {
        this.weapon = (Flamer)weapon;
    }


    public boolean affectsLights( )
    {
        return true;
    }

    public void prepareFrame( Point fireFrom, Point fireTo, int frame, GameMapPanel panel )
    {
        area = weapon.getLastArea();
        int minX = Integer.MAX_VALUE;
        int minY = Integer.MAX_VALUE;
        int maxX = -1;
        int maxY = -1;

        Point point;
        for( int i = 0; i < area.length; i++ )
        {
            point = area[i];
            if( minX > point.x ) { minX = point.x; }
            if( minY > point.y ) { minY = point.y; }
            if( maxX < point.x ) { maxX = point.x; }
            if( maxY < point.y ) { maxY = point.y; }
            
            if( ( frame == 0 ) && ( Math.random() < 0.25 ) )
            {
                int cellSize = panel.getCellSize();
                ImageStain stain = new ImageStain( stains[ (int)(Math.random()*4) ] );
                stain.setLocation( point.x, point.y );
                stain.setBounds( point.x*cellSize, point.y*cellSize, cellSize, cellSize );
                panel.addStain( stain );
            }
            
        }
        
        this.fireTo = fireTo;
        int cellSize = panel.getCellSize();
        RepaintManager.currentManager( panel ).addDirtyRegion( panel, minX*cellSize-cellSize, minY*cellSize-cellSize, maxX*cellSize+2*cellSize, maxY*cellSize+2*cellSize );
    }


    public void drawFrame( Graphics g, GameMapPanel host, int frame )
    {
        Point point;
        int cellSize = host.getCellSize();
        if( frame == 9 )
        {
            return;
        }
        for( int i = 0; i < area.length; i++ )
        {
            point = area[i];
            //System.out.println( "Animating flamer at " + point.toString() );
            g.drawImage( flamerFlame, point.x*cellSize-cellSize/2, point.y*cellSize-cellSize/2, host );
        }
    }

    public int getFrames()
    {
        return 10;
    }

    public AudioClip getFireSoundEffect()
    {
        return fire;
    }

    public AudioClip getMissSoundEffect()
    {
        return null;
    }

}