// GameOverviewMapPanel.java
// Started on June 29th, 2001
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.*;
import java.util.*;

public class GameOverviewMapPanel extends JPanel implements Observer
{
    private Color backgroundColor = Color.black;
    private Color floorColor = Color.darkGray;
    private Color wallColor = Color.lightGray;
    private Color doorColor = Color.yellow;
    private Color friendlyColor = Color.green;
    private Color neutralColor = Color.blue;
    private Color hostileColor = Color.red;
    private Color blipColor = Color.cyan;
    private Color objectiveColor = Color.orange;
    
    private int cellSize = 2;
    private Game game = null;
    private Map map = null;
    private boolean showFriendly = true;
    private boolean showHostile = true;
    private boolean showBlip = false;
    private Insets insets;
    
    public GameOverviewMapPanel()
    {
	super();
	Dimension mySize = new Dimension ( 50*cellSize, 50*cellSize );
	setMinimumSize( mySize );
	setPreferredSize( mySize );
	setMaximumSize( mySize );
	insets = new Insets( 5, 5, 5, 5 );
    }
    
    protected void paintComponent( Graphics g )
    {
	//super.paintComponent( g );
	int x=0, y=0;
	Character character;
	
	if( map == null )
	{
	    g.setColor( Color.black );
	    g.fillRect( 0, 0, getWidth(), getHeight() );
	    return;
	}
	
	// Draw the general map features
	for( int i = 0; i < map.getHorizontalSize(); i++ )
	{
	    for( int j = 0; j < map.getVerticalSize(); j++ )
	    {
		drawCell( g, i, j );
	    }
	}
	if( game != null )
	{
	    // Draw the monsters
	    for( int i = 0; i < game.getMonsters().size(); i++ )
	    {
		character = game.getMonster( i );
		if( character.isAlive() && character.isShownOnMap() )
		{
		    x = character.getLocation().x;
		    y = character.getLocation().y;
		    if( showBlip )
		    {
			g.setColor( blipColor );
			g.fillRect( x*cellSize, y*cellSize, cellSize, cellSize );
		    }
		    if( showHostile && game.isMonsterVisibleToPlayer( i ) )
		    {
			if( character.isHostile( game.getPlayerCharacter( 0 ) ) )
			    g.setColor( hostileColor );
			else
			    g.setColor( neutralColor );
			g.fillRect( x*cellSize, y*cellSize, cellSize, cellSize );
		    }
		}
	    }
	    // Draw the player characters
	    if( showFriendly )
	    {
		g.setColor( friendlyColor );
		for( int i = 0; i < game.getPlayerCharacters().size(); i++ )
		{
		    character = game.getPlayerCharacter( i );
		    if( character.isAlive() )
		    {
			x = character.getLocation().x;
			y = character.getLocation().y;
			g.fillRect( x*cellSize, y*cellSize, cellSize, cellSize );
		    }
		}
	    }
	}
    }

    private void drawCell( Graphics g, int x, int y )
    {
        Objective objective;
        MapCell cell =  map.getCell( x, y );
        g.setColor( backgroundColor );
        if( cell != null )
        {
            if( cell.getFloor() > -1 )
            {
                g.setColor( floorColor );
            }
            if( cell.getWall() > -1 )
            {
                g.setColor( wallColor );
            }
            if( cell.hasDoor() )
            {
                if( !map.getDoor( x, y ).isOpen() )
                    g.setColor( doorColor );
            }
	    if( game != null )
	    {
		for( int i = 0; i < game.getNumberOfObjectives(); i++ )
		{
		    objective = game.getObjective( i );
		    if( objective.isShownOnMap() && objective.isLocatedIn( x, y ) )
			g.setColor( objectiveColor );
		}
	    }
        }
        g.fillRect( x*cellSize, y*cellSize, cellSize, cellSize );
    }
        
    public void setGame( Game game )
    {
	setMap( game.getMap() );
	game.addObserver( this );
	this.game = game;
    }
    
    public void setMap( Map map )
    {
	this.map = map;
	this.game = null;
	if( map != null )
	{
	    Dimension mySize = new Dimension ( map.getHorizontalSize()*cellSize, map.getVerticalSize()*cellSize );
	    setMinimumSize( mySize );
	    setPreferredSize( mySize );
	    setMaximumSize( mySize );
	    invalidate();
	}
	repaint();
    }
	
	
    public int getCellSize()
    {
	return cellSize;
    }
    
    public Insets getInsets()
    {
	return insets;
    }
    
    public Insets getInsets( Insets newInsets )
    {
	newInsets.top = insets.top;
	newInsets.bottom = insets.bottom;
	newInsets.left = insets.left;
	newInsets.right = insets.right;
	return newInsets;
    }
    
    public void update( Observable o, Object arg )
    {
	if( o == game )
	{
	    repaint();
	}
    }
}
