// GameWindow.java
// Started on June 20th, 2001
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import java.awt.event.*;
import java.applet.*;
import java.io.*;
import java.util.*;
import java.text.*;

/**
 * The main game window. Handles the user interface, as well as controls
 * the Game it shows.
 *
 * @author	Virtu Halttunen
 * @version	0.1
 */
public class GameWindow extends JFrame implements KeyListener, MouseListener, ActionListener, WindowListener
{    
    private Game game;
    private TeamManagerWindow teamWindow;
    private GameMapPanel mapPanel;
    private GameOverviewMapPanel overview;
    private JButton walkButton;
    private JButton runButton;
    private JButton shootButton;
    private JButton useButton;
    private JButton nextRoundButton;
    private MarineStatisticsPanel stats;
    private Cursor myCrosshairCursor;
    private JButton [] playerButtons;
    private ImageIcon skullIcon;
    private MarineInfoDialog marineDialog;
    private Properties preferences;

    private final String MARINE_INFO_DIALOG_LOCATION_KEY = "MarineInfoDialogLocation";
    private final String MAIN_WINDOW_LOCATION_KEY = "MainWindowLocation";
    private final String MAIN_WINDOW_SIZE_KEY = "MainWindowSize";
    private final String SHOW_MARINE_INFO_DIALOG_KEY = "ShowMarineInfoDialog";

    private JComboBox weaponModes;
    
    private boolean shiftDown = false;
    
    private int mode = -1;
    private int tempWeaponMode = 0;
    private boolean hasMoved = false;
    private boolean walkingPath = false;
    private boolean finished = false;
    
    public GameWindow( Game game, TeamManagerWindow teamWindow ) 
    {
        super("TheGame: " + game.getMissionName() );
	System.out.println( "Creating game window" );
	this.game = game;
        this.teamWindow = teamWindow;
	Box base = new Box( BoxLayout.X_AXIS );
	Box left = new Box( BoxLayout.Y_AXIS );
	mapPanel = new GameMapPanel( game, 48, 16, 11 );
	JScrollPane scroller = new JScrollPane( mapPanel );
	left.add( scroller );
	JPanel buttonPanel = new JPanel();
	walkButton = new JButton( new ImageIcon( ResourceManager.loadImage( "WalkIcon.png" ) ) );
        walkButton.setSelectedIcon( new ImageIcon( ResourceManager.loadImage( "WalkIconSelected.png" ) ) );
	walkButton.addActionListener( this );
        walkButton.addKeyListener( this );
	walkButton.setToolTipText( "Walk" );
        walkButton.setFocusPainted(false);
	buttonPanel.add( walkButton );
	runButton = new JButton( new ImageIcon( ResourceManager.loadImage( "RunIcon.png" ) ) );
        runButton.setSelectedIcon( new ImageIcon( ResourceManager.loadImage( "RunIconSelected.png" ) ) );
	runButton.addActionListener( this );
        runButton.addKeyListener( this );
	runButton.setToolTipText( "Run" );
        runButton.setFocusPainted(false);
        runButton.setEnabled( false );
	//buttonPanel.add( runButton );
	useButton = new JButton( new ImageIcon( ResourceManager.loadImage( "UseIcon.png" ) ) );
        useButton.setSelectedIcon( new ImageIcon( ResourceManager.loadImage( "UseIconSelected.png" ) ) );
	useButton.addActionListener( this );
        useButton.addKeyListener( this );
	useButton.setToolTipText( "Use" );
        useButton.setFocusPainted(false);
	buttonPanel.add( useButton );
	shootButton = new JButton( new ImageIcon( ResourceManager.loadImage( "ShootIcon.png" ) ) );
        shootButton.setSelectedIcon( new ImageIcon( ResourceManager.loadImage( "ShootIconSelected.png" ) ) );
	shootButton.addActionListener( this );
        shootButton.addKeyListener( this );
	shootButton.setToolTipText( "Shoot" );
        shootButton.setFocusPainted(false);
	buttonPanel.add( shootButton );
	
	buttonPanel.add( Box.createHorizontalStrut( 50 ) );
	JButton playerButton;
	playerButtons = new JButton[ game.getPlayerCharacters().size() ];
	skullIcon = new ImageIcon( ResourceManager.loadImage( "SkullIcon.png" ) );
	for( int i = 0; i < playerButtons.length; i++ )
	{
	    playerButton = new JButton( new ImageIcon( ResourceManager.loadImage( "SoldierFaceIcon.png" ) ) );
            playerButton.setSelectedIcon( new ImageIcon( ResourceManager.loadImage( "SoldierFaceIconSelected.png" ) ) );
	    playerButton.addActionListener( this );
            playerButton.addKeyListener( this );
	    playerButton.setToolTipText( ((Marine)game.getPlayerCharacter( i )).getName() );
            playerButton.setFocusPainted(false);
	    playerButtons[ i ] = playerButton;
	    buttonPanel.add( playerButton );
	}
	playerButtons[0].setSelected( true );
	
	left.add( buttonPanel );	
	base.add( left );
	
	Box right = new Box( BoxLayout.Y_AXIS );
	overview = new GameOverviewMapPanel();
	overview.setGame( game );
	overview.addMouseListener( this );
        overview.addKeyListener( this );
	Box overviewPanel = new Box( BoxLayout.Y_AXIS );
	overviewPanel.add( overview );
	overviewPanel.setBackground( Color.darkGray );
	right.add( overviewPanel );
	stats = new MarineStatisticsPanel();
        stats.setShowWeapon( true );
	stats.setMarine( (Marine)game.getPlayerCharacter(0) );
        stats.addMouseListener( this );
        stats.addKeyListener( this );
        stats.setCursor( Cursor.getPredefinedCursor( Cursor.HAND_CURSOR ) );
        stats.setToolTipText( "Click for marine info" );
	right.add( stats );

        weaponModes = new JComboBox( ((Marine)game.getPlayerCharacter(0)).getWeapon().getFiringModeNames() );
        weaponModes.setEditable( false );
        weaponModes.addActionListener( this );
        weaponModes.addKeyListener( this );
        weaponModes.setKeySelectionManager( null );
        weaponModes.setRequestFocusEnabled( false );
        weaponModes.setToolTipText( "Weapon firing mode" );

        right.add( weaponModes );
        
	right.add( Box.createVerticalGlue( ) );
	nextRoundButton = new JButton( new ImageIcon( ResourceManager.loadImage( "NextRoundIcon.png" ) ) );
	nextRoundButton.addActionListener( this );
        nextRoundButton.addKeyListener( this );
	nextRoundButton.setToolTipText( "Next round" );
	right.add( nextRoundButton );
	base.add( right );
	
	getContentPane().add( base );
	setIconImage( ResourceManager.loadImage( "FubarIcon.png" ) );
	setDefaultCloseOperation( JFrame.DO_NOTHING_ON_CLOSE );
	addWindowListener( this );
	pack();

	// Set the fixed dimensions for panels. Yeah, I know this is supposedly unsafe
	Dimension temp;
	temp = overview.getPreferredSize();
	temp.height = temp.height+5;
	temp.width = temp.width+5;
	//overviewPanel.setPreferredSize( temp );
	//overviewPanel.setMinimumSize( temp );
	//overviewPanel.setMaximumSize( temp );
	temp = buttonPanel.getPreferredSize();
	buttonPanel.setMinimumSize( temp );
	buttonPanel.setMaximumSize( temp );

        temp = weaponModes.getSize();
        weaponModes.setMaximumSize( temp );
	weaponModes.invalidate();
	validate();

	mapPanel.centerOnCharacter( game.getActiveCharacter() );
	addKeyListener( this );
	mapPanel.addMouseListener( this );
        mapPanel.addKeyListener( this );

        System.out.println( "Maximum cursor colours: " + Toolkit.getDefaultToolkit().getMaximumCursorColors() );
	Image cursorImage = ResourceManager.loadImage( "MyCrosshairCursor16.png" );
	Dimension blah = Toolkit.getDefaultToolkit().getBestCursorSize( 24, 24 );
        System.out.println( "Best fit for 24: " + blah.width + "x" + blah.height );
 	myCrosshairCursor = Toolkit.getDefaultToolkit().createCustomCursor( cursorImage, new Point( blah.width/2, blah.height/2 ), "MyCrosshairCursor" );
	System.out.println( "Initialized cursors" );

	marineDialog = new MarineInfoDialog( this );
        marineDialog.setMarine( (Marine)game.getActiveCharacter() );
        marineDialog.addKeyListener( this );

        System.out.println( "Loading preferences" );
        loadPreferences();
	System.out.println( "Showing game window" );
        mapPanel.checkMonsterVisibility();
        setVisible(true);
        setGameMode( Game.MOVE );
        showMissionBrief();
        requestFocus();
    }

    public void showMissionBrief()
    {
        String missionDesc = game.getMissionDescription();
        JOptionPane.showMessageDialog( this, missionDesc, "Mission briefing", JOptionPane.INFORMATION_MESSAGE );
    }
    
    public void keyTyped( KeyEvent e )
    {
    }
    
    public void keyReleased( KeyEvent e )
    {
	int key = e.getKeyCode();
	switch( key )
	{
	    case KeyEvent.VK_META:
		((Marine)game.getActiveCharacter()).getWeapon().setFiringMode( tempWeaponMode );
		stats.repaint();
		break;
	}
	if( !e.isControlDown() )
	{
	    //hasReticle = false;
	}
	
    }
    
    public void keyPressed( KeyEvent e )
    {
	Rectangle rect;
	int player;
	Point cell;
    
	mapPanel.setDrawPlayerHint( false );
	int key = e.getKeyCode();
	boolean move = false;
	int direction = -1;
	
	if( mapPanel.isAnimating() )
	{
	    e.consume();
	    return;
	}
	
	switch( key )
	{
	    case KeyEvent.VK_NUMPAD8:
            case KeyEvent.VK_W:
		direction = Map.NORTH;
		move = true;
		e.consume();
		break;
	    case KeyEvent.VK_NUMPAD9:
            case KeyEvent.VK_E:
		direction = Map.NORTH_EAST;
		move = true;
		e.consume();
		break;
	    case KeyEvent.VK_NUMPAD6:
            case KeyEvent.VK_D:
		direction = Map.EAST;
		move = true;
		e.consume();
		break;
	    case KeyEvent.VK_NUMPAD3:
            case KeyEvent.VK_C:
		direction = Map.SOUTH_EAST;
		move = true;
		e.consume();
		break;
	    case KeyEvent.VK_NUMPAD2:
            case KeyEvent.VK_X:
		direction = Map.SOUTH;
		move = true;
		e.consume();
		break;
	    case KeyEvent.VK_NUMPAD1:
            case KeyEvent.VK_Z:
		direction = Map.SOUTH_WEST;
		move = true;
		e.consume();
		break;
	    case KeyEvent.VK_NUMPAD4:
            case KeyEvent.VK_A:
		direction = Map.WEST;
		move = true;
		e.consume();
		break;
	    case KeyEvent.VK_NUMPAD7:
            case KeyEvent.VK_Q:
		direction = Map.NORTH_WEST;
		move = true;
		e.consume();
		break;
	    case KeyEvent.VK_UP:
		if( !e.isControlDown() )
		{
		    rect = mapPanel.getVisibleRect();
		    rect.y = rect.y-mapPanel.getScrollableUnitIncrement( rect, SwingConstants.VERTICAL, -1 );
		    mapPanel.scrollRectToVisible( rect );
		}
		else
		{
		    move = true;
		    direction = Map.NORTH;
		}
		e.consume();
		break;
	    case KeyEvent.VK_DOWN:
		if( !e.isControlDown() )
		{
		    rect = mapPanel.getVisibleRect();
		    rect.y = rect.y+mapPanel.getScrollableUnitIncrement( rect, SwingConstants.VERTICAL, 1 );
		    mapPanel.scrollRectToVisible( rect );
		}
		else
		{
		    move = true;
		    direction = Map.SOUTH;
		}
		e.consume();
		break;
	    case KeyEvent.VK_LEFT:
		if( !e.isControlDown() )
		{
		    rect = mapPanel.getVisibleRect();
		    rect.x = rect.x-mapPanel.getScrollableUnitIncrement( rect, SwingConstants.HORIZONTAL, -1 );
		    mapPanel.scrollRectToVisible( rect );
		}
		else
		{
		    move = true;
		    direction = Map.WEST;
		}
		e.consume();
		break;
	    case KeyEvent.VK_RIGHT:
		if( !e.isControlDown() )
		{
		    rect = mapPanel.getVisibleRect();
		    rect.x = rect.x+mapPanel.getScrollableUnitIncrement( rect, SwingConstants.HORIZONTAL, -1 );
		    mapPanel.scrollRectToVisible( rect );
		}
		else
		{
		    move = true;
		    direction = Map.EAST;
		}
		e.consume();
		break;
	    case KeyEvent.VK_PAGE_DOWN:
		playerButtons[ game.getActiveCharacterIndex() ].setSelected( false );
		player = game.getActiveCharacterIndex();
		do
		{
		    player = (player+1)%game.getPlayerCharacters().size();
		} while( !game.getPlayerCharacter( player ).isAlive() );
		mapPanel.clearMessageString();
                // Don't center the map on the new character if shift is pressed
                setActiveCharacter( player, ( !( ( e.getModifiers() & KeyEvent.SHIFT_MASK ) == KeyEvent.SHIFT_MASK ) ) );
                e.consume();
		break;
	    case KeyEvent.VK_PAGE_UP:
		playerButtons[ game.getActiveCharacterIndex() ].setSelected( false );
		player = game.getActiveCharacterIndex();
		do
		{
		    player = (player+game.getPlayerCharacters().size()-1)%game.getPlayerCharacters().size();
		} while( !game.getPlayerCharacter( player ).isAlive() );
		mapPanel.clearMessageString();
                setActiveCharacter( player, ( !( ( e.getModifiers() & KeyEvent.SHIFT_MASK ) == KeyEvent.SHIFT_MASK ) ) );
		e.consume();
		break;
	    case KeyEvent.VK_HOME:
		mapPanel.setDrawPlayerHint( true );
		mapPanel.clearMessageString();
		mapPanel.centerOnCharacter( game.getActiveCharacter() );
		mapPanel.repaint();
		e.consume();
		break;
            case KeyEvent.VK_ASTERISK:
                 e.consume();
                break;
            case KeyEvent.VK_SPACE:
                System.out.println( "Space" );
                e.consume();
                break;
            case KeyEvent.VK_INSERT:
            case KeyEvent.VK_HELP:
	    case KeyEvent.VK_DELETE:
	    case KeyEvent.VK_H:
                showMissionBrief();
                e.consume();
                break;
            case KeyEvent.VK_ENTER:
                endTurn();
                e.consume();
                break;
	    case KeyEvent.VK_META:
		tempWeaponMode = ((Marine)game.getActiveCharacter()).getWeapon().getFiringMode();
		((Marine)game.getActiveCharacter()).getWeapon().setFiringMode( 1 );
		stats.repaint();
		break;
	    case KeyEvent.VK_CONTROL:
		Point targetPoint = Map.getNeighbor( game.getActiveCharacter().getLocation(), game.getActiveCharacter().getDirection() );
		Character [] visibleEnemies = game.getVisibleEnemies( game.getActiveCharacter() );
		if( visibleEnemies.length > 0 )
		{
		    Character monster = game.getClosestCharacter( game.getActiveCharacter(), visibleEnemies );
		    targetPoint = monster.getLocation();
		}
		mapPanel.setTargetReticleLocation( targetPoint );
		break;
	    case KeyEvent.VK_P:
		mapPanel.showPreviousMessage();
		break;
	    case KeyEvent.VK_N:
		mapPanel.showNextMessage();
		break;
	    case KeyEvent.VK_F1:
		((Marine)game.getActiveCharacter()).getWeapon().setFiringMode( 0 );
		weaponModes.setSelectedIndex( 0 );
		stats.repaint();
		mapPanel.grabFocus();	// Workaround for the combobox hogging focus
		break;
	    case KeyEvent.VK_F2:
		((Marine)game.getActiveCharacter()).getWeapon().setFiringMode( 1 );
		weaponModes.setSelectedIndex( 1 );
		stats.repaint();
		mapPanel.grabFocus();	// Workaround for the combobox hogging focus
		break;
	    case KeyEvent.VK_F8:
		mapPanel.setSlowify();
		break;
	    case KeyEvent.VK_U:
		if( mapPanel.getTargetReticleLocation().x >= 0 )
		{
		    cell = mapPanel.getTargetReticleLocation();
		    handleUse( game.getActiveCharacter(), new Point( cell ) );
		}
		break;

	    case KeyEvent.VK_F:
		hasMoved = true;
                if( mapPanel.getTargetReticleLocation().x < 0  || !game.getActiveCharacter().isAlive() )
                {
                    // Dead people don't shoot...
                    return;
                }
		mapPanel.setDrawPlayerHint( false );
		
                int temp = ((Marine)game.getActiveCharacter()).getWeapon().getFiringMode();
                if( e.isMetaDown() )
                {
                    ((Marine)game.getActiveCharacter()).getWeapon().setFiringMode( 1 );
                    System.out.println( "Second trigger" );
                }
		cell = mapPanel.getTargetReticleLocation();
		boolean valid = game.handleRangedAttack( (Marine)game.getActiveCharacter(), new Point( cell ) );
		
                if( valid )
                {
                    //System.out.println( "Valid attack" );
		    mapPanel.update( game, new SyncEvent() );
                    for( int i=0; i < game.getPlayerCharacters().size(); i++ )
                    {
                        if( !game.getPlayerCharacter( i ).isAlive() )
                        {
                            playerButtons[i].setEnabled( false );
                        }
                    }
                    overview.repaint();
                    stats.repaint();
                    marineDialog.repaint();
                }
		else
		{
		    //mapPanel.animateString( "Not a \nvalid attack" );
		    System.out.println( "Not a valid attack" );
		}
		((Marine)game.getActiveCharacter()).getWeapon().setFiringMode( temp );
                e.consume();
		
		break;
	    case KeyEvent.VK_G:
		if( e.isMetaDown() || e.isControlDown() )
		{
		    regroup();
		}
		e.consume();
		break;
		
 	}
	if( move )
	{
	    Point targetPoint;
	    if( e.isControlDown() )
	    {
		targetPoint = Map.getNeighbor( mapPanel.getTargetReticleLocation(), direction );
		mapPanel.setTargetReticleLocation( targetPoint );
	    }
	    else
	    {
		if( e.isShiftDown() )
		{
		    handleUse( game.getActiveCharacter(), direction );
		}
		else
		{
		    moveCharacter( true, game.getActiveCharacterIndex(), direction );
		}
	    }
	}
    }

    public void setGameMode( int mode )
    {
	this.mode = mode;
	if( mode == game.MOVE )
	{
	    mapPanel.setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
	    mapPanel.setDrawShootingHints( false );
	    shootButton.setSelected( false );
	    walkButton.setSelected( true );
	    runButton.setSelected( false );
            useButton.setSelected( false );
	}
	if( mode == game.ATTACK_RANGED )
	{
	    mapPanel.setCursor( myCrosshairCursor );
	    mapPanel.setDrawShootingHints( true );
	    shootButton.setSelected( true );
	    walkButton.setSelected( false );
	    runButton.setSelected( false );
            useButton.setSelected( false );
	}
        if( mode == game.USE )
        {
 	    mapPanel.setCursor( Cursor.getPredefinedCursor( Cursor.HAND_CURSOR ) );
	    mapPanel.setDrawShootingHints( false );
	    shootButton.setSelected( false );
	    walkButton.setSelected( false );
	    runButton.setSelected( false );
            useButton.setSelected( true );       
        }
        mapPanel.repaint();
    }
    
    public int getGameMode()
    {
        return mode;
    }
    
    public boolean moveCharacter( boolean player, int index, int direction )
    {
    	boolean success = false;
	Character character = null; 

	if( player )
	{
	    mapPanel.setTargetReticleLocation( new Point( -1, -1 ) );
	    character = game.getPlayerCharacter( index );
            if( !character.isAlive() )
	    {
		// Dead people don't move...
		return false;
	    }
	    hasMoved = true;
	    success = game.movePlayer( index, direction );
	}

	if( !success )
	{
	    // Might have been blocked by a door. Try to open it
	    if( game.getMap().isClosedDoor( game.getPlayerCharacter( index ).getLocation(), direction ) )
	    {
		success = game.handleUse( character, Map.getNeighbor( character.getLocation(), direction ) );
//		openDoor( Map.getNeighbor( game.getPlayerCharacter( index ).getLocation(), direction ) );
	    }
	}
	if( !success && !walkingPath )
	{
	    // Was the move blocked by a monster? if so, try to attack
	    Point target = game.getMap().getNeighbor( character.getLocation(), direction );
	    Monster monster = null;
	    boolean found = false;
	    for( int i=game.getMonsters().size()-1; (i >= 0) && (!found); i-- )
	    {
		monster = game.getMonster( i );
		if( monster.isAlive() && monster.getLocation().equals( target ) && monster.isHostile( character ) )
		{
		    found = true;
		    System.out.println( "Attacking " + monster.getClass().getName() );
		}
	    }
	    if( found && monster != null )
	    {
		success = game.handleMelee( character, monster );
	    }
	}
	if( success )
	{
            //mapPanel.scrollRectToVisible( new Rectangle( (game.getPlayerCharacter(activeCharacter).getLocation().x-4)*48, (game.getPlayerCharacter(activeCharacter).getLocation().y-4)*48, 9*48, 9*48 ) );
	    mapPanel.update( game, new SyncEvent() );
            RepaintManager.currentManager( mapPanel ).paintDirtyRegions();
	    stats.repaint();
            marineDialog.repaint();
	}
	else
	{
	    game.handleOutOfAP( character );
	}
	return success;
    }

    public void regroup()
    {
	Character leader = null;
	int maxAP = 0;
	for( int i=0; i < game.getPlayerCharacters().size(); i++ )
	{
	    Character check = game.getPlayerCharacter( i );
	    if( check.isAlive() && ( leader == null || ((Marine)check).getRank() > ((Marine)leader).getRank() ) )
	    {
		leader = check;
	    }
	    if( check.isAlive() && check.getCurrentActionPoints() > maxAP )
	    {
		maxAP = check.getCurrentActionPoints();
	    }
	}
	if( leader == null )
	{
	    return;
	}
	java.util.Vector[] paths = new java.util.Vector[ game.getPlayerCharacters().size() ];
	for( int i=0; i < game.getPlayerCharacters().size(); i++ )
	{
	    Character character = game.getPlayerCharacter( i );
	    if( character != leader )
	    {
		Vector thePath;
		Vector mainPath = game.getMap().findShortestPath( character, leader.getLocation(), character.getCurrentActionPoints(), true );
		Vector naivePath = game.getMap().findShortestPathFromPoint( character.getLocation(), leader.getLocation(), true );
		thePath = mainPath;
		if( mainPath.size() > naivePath.size()*2 || ( mainPath.size() == 0 && naivePath.size() > 0 ) )
		{
		    //System.out.println( "Using naive path for " + character.getName() );
		    thePath = naivePath;
		}
		paths[i] = thePath;//game.getMap().findShortestPath( character, leader.getLocation(), character.getCurrentActionPoints(), true );
	    }
	}
	for( int a=0; a < maxAP*2; a++ )
	{
	    for( int i=0; i < game.getPlayerCharacters().size(); i++ )
	    {
		Character character = game.getPlayerCharacter( i );
		if( character.isAlive() && character != leader )
		{
		    java.util.Vector path = paths[i];
		    if( path.size() == 0 )
		    {
			//paths[i] = game.getMap().findShortestPath( character, leader.getLocation(), character.getCurrentActionPoints(), true );
			Vector thePath;
			Vector mainPath = game.getMap().findShortestPath( character, leader.getLocation(), character.getCurrentActionPoints(), true );
			Vector naivePath = game.getMap().findShortestPathFromPoint( character.getLocation(), leader.getLocation(), true );
			thePath = mainPath;
			if( mainPath.size() > naivePath.size()*2 || ( mainPath.size() == 0 && naivePath.size() > 0 ) )
			{
			    //System.out.println( "Using naive path for " + character.getName() );
			    thePath = naivePath;
			}
			paths[i] = thePath;
			path = paths[i];
		    }
		    if( path.size() > 1 )
		    {
			walkingPath = true;
	    
			Point cell = ((MapNode)path.remove( 1 )).getPoint();
			if( game.getMap().isClosedDoor( cell ) )
			{
			    game.handleUse( character, cell );
			}
			//System.out.println( "Walking path: " + cell.toString() );
			int direction = game.getMap().getDirection( character.getLocation(), cell, character.getDirection() );
			if( game.canCharacterMove( character, direction, true ) )
			{
			    game.movePlayer( i, direction );
			}
			else
			{
			    paths[i].clear();
			}
		    }
		}
	    }
	    mapPanel.update( game, new SyncEvent() );
            RepaintManager.currentManager( mapPanel ).paintDirtyRegions();
	}
    }
    
    public void setActiveCharacter( int index, boolean center )
    {
        game.setActiveCharacter( index );
        playerButtons[ index ].setSelected( true );
        mapPanel.setDrawPlayerHint( true );
	mapPanel.setTargetReticleLocation( new Point( -1, -1 ) );
	if( center )
        {
            mapPanel.centerOnCharacter( game.getActiveCharacter() );
        }
	walkingPath = false;
        mapPanel.repaint();
        stats.setMarine( (Marine)game.getActiveCharacter() );
        marineDialog.setMarine( (Marine)game.getActiveCharacter() );
        System.out.println( "Switched to " + ((Marine)game.getActiveCharacter()).getName() );

        weaponModes.removeActionListener( this ); // Needed so we don't try to set the weapon modes while updating the list
        weaponModes.removeAllItems();
        String [] modes = ((Marine)game.getActiveCharacter()).getWeapon().getFiringModeNames();
        for( int i = 0; i < modes.length; i++ )
        {
            weaponModes.addItem( modes[ i ] );
        }
        weaponModes.setSelectedIndex( ((Marine)game.getActiveCharacter()).getWeapon().getFiringMode() );
        weaponModes.addActionListener( this );
    }
     
    public void actionPerformed( ActionEvent e )
    {
    	if( e.getSource() == shootButton )
	{
            setGameMode( Game.ATTACK_RANGED );
	}
	if( e.getSource() == walkButton )
	{
            setGameMode( Game.MOVE );
	}
        if( e.getSource() == useButton )
        {
            setGameMode( Game.USE );
        }
	if( e.getSource() == nextRoundButton )
	{
	    if( !hasMoved )
	    {
		int answer = JOptionPane.showConfirmDialog( this, "You have not moved. Are you sure you want to end the turn?", "Not moved", JOptionPane.YES_NO_OPTION );
		if( answer != JOptionPane.OK_OPTION && answer != JOptionPane.YES_OPTION )
		{
		    return;
		}
	    }
	    nextRoundButton.setEnabled( false );
	    endTurn();
	    mapPanel.requestFocus();
	    
	}
        if( e.getSource() == weaponModes )
        {
            ((Marine)game.getActiveCharacter()).getWeapon().setFiringMode( weaponModes.getSelectedIndex() );
	    stats.repaint();
            mapPanel.grabFocus();	// Workaround for the combobox hogging focus
        }
	for( int i = 0; i < playerButtons.length; i++ )
	{
	    if( e.getSource() == playerButtons[ i ] )
	    {
 		for( int j = 0; j < playerButtons.length; j++ )
		{
		    playerButtons[ j ].setSelected( false );
		}
		mapPanel.clearMessageString();
                setActiveCharacter( i, ( !( ( e.getModifiers() & ActionEvent.SHIFT_MASK ) == ActionEvent.SHIFT_MASK ) ) );
	    }
	}
    }
    
    public void endTurn()
    {
	mapPanel.clearMessageString();
	// Activate the critters
	game.doMonsterActions();
	
        // Check whether the game is completed
	boolean complete = game.isCompleted();
	
        // Check the dead players... 
	for( int i = 0; i < game.getPlayerCharacters().size(); i++ )
	{
            playerButtons[ i ].setSelected( false );
	    if( ! game.getPlayerCharacter( i ).isAlive() )
	    {
                playerButtons[ i ].setDisabledIcon( skullIcon );
		playerButtons[ i ].setEnabled( false );
	    }
	}
        // ... and set the first living one as active
	boolean allPlayersDead = true;
	for( int i = 0; i < game.getPlayerCharacters().size(); i++ )
	{
	    if( game.getPlayerCharacter( i ).isAlive() )
	    {
		game.setActiveCharacter( i );
		playerButtons[ i ].setSelected( true );
		allPlayersDead = false;
		break;
	    }
 	}
	if( allPlayersDead )
	{
            String message = "All soldiers are dead\n" + getMissionStatistics();
	    JOptionPane.showMessageDialog( this, message, "Mission failed", JOptionPane.WARNING_MESSAGE );
            teamWindow.savePlayers();
            savePreferences();
            setVisible( false );
            teamWindow.cleanUpMission();
            teamWindow.setVisible( true );
            return;
        }
        if( complete )
        {
	    endMission();
            return;
        }
        else
        {
            setActiveCharacter( game.getActiveCharacterIndex(), true );
	    // Reset action points
	    game.setupNewTurn();
	    mapPanel.setupNewTurn();
	    
	    // Check abort
            if( game.getElapsedTurns() > 4 && game.isAbortable() && ( game.getElapsedTurns() % 5 == 0 ) )
            {
                int selection = JOptionPane.showConfirmDialog( this, "Mission is not complete. Do you want to abort", "Abort mission?", JOptionPane.YES_NO_OPTION );
                if( selection == JOptionPane.YES_OPTION )
                {
                    endMission();
                    return;
                }
            }
        }
	nextRoundButton.setEnabled( true );
	hasMoved = false;
    }

    public void endMission()
    {
        String message = "Congratulations";
        String title;
	boolean success = true;
        boolean allPlayersAlive = true;
	int missionStatus = game.getMissionStatus();
	if( missionStatus == Mission.VICTORY || missionStatus == Mission.COMPLETE_VICTORY )
	{
	    title = "Mission complete";
	    success = true;
	}
	else
	{
	    title = "Mission failed";
	    success = false;
	}
        for( int i = 0; i < game.getPlayerCharacters().size(); i++ )
        {
            if( !game.getPlayerCharacter( i ).isAlive() )
            {
                allPlayersAlive = false;
            }
        }
	message = game.getDebriefMessage()+"\n\n"+getMissionStatistics();
	JTextArea theText = new JTextArea( message );
        theText.setFont( new Font( "Serif", Font.PLAIN, 12 ) );
        JOptionPane.showMessageDialog( this, theText, title, JOptionPane.INFORMATION_MESSAGE );
        if( allPlayersAlive && success )
        {
            // Grant extra leadership experience to the leader
            ((Marine)game.getPlayerCharacter(0)).grantExperienceTo( Marine.LEADERSHIP, (int)(Math.random()*3 ) );
        }
        for( int i = 0; i < game.getPlayerCharacters().size(); i++ )
        {
            // Reward the whole team
            ((Marine)game.getPlayerCharacter(i)).endMission( success, game.getMissionBonus() );
	    if( game.canChangeMissionTeamAfter() )
	    {
		((Marine)game.getPlayerCharacter(i)).checkExperience();
	    }
        }
        teamWindow.savePlayers();
        savePreferences();
	finished = true;
        setVisible( false );
        teamWindow.cleanUpMission();
        teamWindow.setVisible( true );
    }
    
    public void mouseClicked( MouseEvent e )
    {
	walkingPath = false;
	if( e.getSource() == mapPanel )
	{
	    // Check whether the game panel is still animating previous action
	    if( mapPanel.isAnimating() )
	    {
		System.out.println( "Clicked while animating. Ignoring" );
		e.consume();
		return;
	    }
	    
	    // Otherwise, process the action
            int actionType = -1;

            if( SwingUtilities.isRightMouseButton( e ) )
            {
                actionType = Game.ATTACK_RANGED;
            }
            else if( SwingUtilities.isMiddleMouseButton( e ) || e.isAltDown() )
            {
                actionType = Game.USE;
            }
            else if( SwingUtilities.isLeftMouseButton( e ) )
            {
                actionType = mode;
            }

            if( actionType == Game.ATTACK_RANGED )
            {
		hasMoved = true;
                if( ! game.getActiveCharacter().isAlive() )
                {
                    // Dead people don't shoot...
                    return;
                }
                mapPanel.setDrawPlayerHint( false );


                int temp = ((Marine)game.getActiveCharacter()).getWeapon().getFiringMode();
                if( SwingUtilities.isRightMouseButton( e ) )
                {
                    ((Marine)game.getActiveCharacter()).getWeapon().setFiringMode( 1 );
                    System.out.println( "Second trigger" );
                }
                Point cell = null;

                // Area effect weapons fire just where we want them to fire
                if( ((Marine)game.getActiveCharacter()).getWeapon().hasAreaEffect() )
                {
                    cell = mapPanel.getCell( e.getPoint() );
                }
                else
                {
                    cell = mapPanel.getCellForAttack( e.getPoint() );
                }

                boolean valid = game.handleRangedAttack( (Marine)game.getActiveCharacter(), cell );

                if( valid )
                {
                    //System.out.println( "Valid attack" );
		    mapPanel.update( game, new SyncEvent() );
                    for( int i=0; i < game.getPlayerCharacters().size(); i++ )
                    {
                        if( !game.getPlayerCharacter( i ).isAlive() )
                        {
                            playerButtons[i].setEnabled( false );
                        }
                    }
                    overview.repaint();
                    stats.repaint();
                    marineDialog.repaint();
                }
                else
		{
		    //mapPanel.animateString( "Not a \nvalid attack" );
                    System.out.println( "Not a valid attack" );
		}
                ((Marine)game.getActiveCharacter()).getWeapon().setFiringMode( temp );
                e.consume();
            }
            if( actionType == game.MOVE )
            {
                if( ! game.getActiveCharacter().isAlive() )
                {
                    // Dead people don't move either...
                    return;
                }
		
                mapPanel.setDrawPlayerHint( false );
		hasMoved = true;
                Point cell = mapPanel.getCell( e.getPoint() );
                Character character = game.getActiveCharacter();
//                java.util.Vector path = game.getMap().findShortestPath( character, cell, true );
		System.out.println( "Alternative:" );
		java.util.Vector path = game.getMap().findShortestPath( character, cell, character.getCurrentActionPoints(), true );
                if( path.size() > 1 )
                {
		    boolean success = true;
		    walkingPath = true;
		    
                    for( int i = 1; ( i < path.size() ) && ( character.getCurrentActionPoints() > 0); i++ )
                    {
                        cell = ((MapNode)path.elementAt( i )).getPoint();
                        if( game.getMap().isClosedDoor( cell ) )
			{
			    game.handleUse( character, cell );
			}
                        //System.out.println( "Walking path: " + cell.toString() );
                        success = moveCharacter( true, game.getActiveCharacterIndex(), game.getMap().getDirection( character.getLocation(), cell, character.getDirection() ) );
			if( !success )
			{
			    return;
			}
                    }
		    
                }
		else
		{
		    game.handleOutOfAP( character );
		}
            }
            if( actionType == Game.USE )
            {
		Point cell = mapPanel.getCell( e.getPoint() );
		handleUse( game.getActiveCharacter(), cell );
            }
        }
	if( e.getSource() == overview )
	{
	    Point point = e.getPoint();
	    int div = overview.getCellSize();
	    Point cell = new Point( point.x/div, point.y/div );
	    mapPanel.scrollRectToVisible( new Rectangle( (cell.x-6)*48, (cell.y-6)*48, 13*48, 13*48 ) );	    
	}
        if( e.getSource() == stats )
        {
            marineDialog.setMarine( (Marine)game.getActiveCharacter() );
            marineDialog.setVisible( true );
        }
    }
    
    public String getMissionStatistics()
    {
        int totalKills = 0;
        int totalMarineKills = 0;
        int kills;
        int monsters = game.getMonsters().size();
        int turns = game.getElapsedTurns();
        double percentage;
        double average;
        StringBuffer message = new StringBuffer();
        message.append( "Mission statistics:\n" );
	for( int i=0; i < game.getPlayerCharacters().size(); i++ )
	{
	    message.append( ((Marine)game.getPlayerCharacter( i )).getName() );
	    if( ! game.getPlayerCharacter( i ).isAlive() )
		message.append( " (DECEASED) " );
            kills = ((Marine)game.getPlayerCharacter( i )).getLastMissionKills();
            totalMarineKills += kills;
	    message.append( ": " + kills + " kills\n" );
	}
        for( int i=0; i < monsters; i++ )
        {
            if( ! game.getMonster( i ).isAlive() )
            {
                totalKills++;
            }
        }
	NumberFormat nf = NumberFormat.getInstance();
	nf.setMaximumFractionDigits( 2 );
	nf.setMinimumFractionDigits( 2 );
//        percentage = StrictMath.round( 10000*(double)totalKills/(double)monsters)/100; // "Round" to two decimals
//        average = StrictMath.round( 100*(double)totalKills/(double)turns )/100;
	percentage = 100.0*(double)totalKills/(double)monsters; 
        average = (double)totalKills/(double)turns;

        message.append( "Total kills: " + totalMarineKills + " (" + totalKills +")" + " of " + monsters + " (" + nf.format( percentage ) + "%)\n");
        message.append( "Average kills per turn: " + nf.format( average ) + "\n");
        message.append( "Turns taken: " + turns + "\n");
        System.out.println( message.toString() );
        return message.toString();       
    }
    
    public void handleUse( Character character, int direction )
    {
	Point point = Map.getNeighbor( character.getLocation(), direction );
	handleUse( character, point );
    }
    
    public void handleUse( Character character, Point target )
    {
	mapPanel.setDrawPlayerHint( false );
	game.handleUse( game.getActiveCharacter(), target );
	SyncEvent sync = new SyncEvent();
	mapPanel.update( game, sync );
	stats.repaint();
	marineDialog.repaint();
	mapPanel.repaint();
	overview.repaint();		
	// Check for possible player deaths caused by the action
	for( int i=0; i < game.getPlayerCharacters().size(); i++ )
	{
	    if( !game.getPlayerCharacter( i ).isAlive() )
	    {
		playerButtons[ i ].setEnabled( false );
	    }
	}
    }

    private void loadPreferences()
    {
	preferences = teamWindow.getPreferences();
	if( preferences != null )
	{
	    System.out.println( "Loaded preferences" );
            Point point;
            String marineInfoLocationString = preferences.getProperty( MARINE_INFO_DIALOG_LOCATION_KEY );
            if( marineInfoLocationString != null )
            {
                point = Objective.pointFromString( marineInfoLocationString.trim() );
                marineDialog.setLocation( point );
            }
            String mainWindowLocationString = preferences.getProperty( MAIN_WINDOW_LOCATION_KEY );
            if( mainWindowLocationString != null )
            {
                point = Objective.pointFromString( mainWindowLocationString );
                this.setLocation( point );
            }
            String mainWindowSizeString = preferences.getProperty( MAIN_WINDOW_SIZE_KEY );
            if( mainWindowSizeString != null )
            {
                point = Objective.pointFromString( mainWindowSizeString );
                this.setSize( point.x, point.y );
            }
            String showMarineDialogString = preferences.getProperty( SHOW_MARINE_INFO_DIALOG_KEY );
            if( showMarineDialogString != null )
            {
                if( showMarineDialogString.trim().equalsIgnoreCase( "True" ) )
                {
                    marineDialog.setVisible( true );
                }
            }
        }
    }
    
    private void savePreferences() 
    {
        Point point;
        point = marineDialog.getLocation();
        preferences.setProperty( MARINE_INFO_DIALOG_LOCATION_KEY, Objective.pointToString( point ) );
        point = getLocation();
        preferences.setProperty( MAIN_WINDOW_LOCATION_KEY, Objective.pointToString( point ) );
        point = new Point( getWidth(), getHeight() );
        preferences.setProperty( MAIN_WINDOW_SIZE_KEY, Objective.pointToString( point ) );
        String showMarineDialogString = "False";
        if( marineDialog.isShowing() )
        {
            showMarineDialogString = "True";
        }
        preferences.setProperty( SHOW_MARINE_INFO_DIALOG_KEY, showMarineDialogString );
        String directory = System.getProperty( "user.home" );
        File prefsFile = new File( directory, ".theGame.properties" );
        if( preferences != null )
        {
            if( ! prefsFile.exists() )
            {
                try
                {
                    prefsFile.createNewFile();
                }
                catch( Exception e )
                {
                    System.out.println( "Couldn't save the preferences!" );
                    return;
                }
            }
            if( prefsFile.canWrite() )
            {
                try
                {
                    FileOutputStream fos = new FileOutputStream( prefsFile );
                    preferences.store( fos, "TheGame Preferences" );
                    fos.close();
                }
                catch( Exception e )
                {
                    System.out.println( "Couldn't save the preferences!" );
                }
            }
	    System.out.println( "Saved mission window preferences" );
        }
        
    }
    
    public void mousePressed( MouseEvent e )
    {}
    
    public void mouseReleased( MouseEvent e )
    {}
    
    public void mouseEntered( MouseEvent e )
    {}
    
    public void mouseExited( MouseEvent e )
    {}

    public void windowClosing( WindowEvent e )
    {
	if( !finished )
	{
	    int answer = JOptionPane.showConfirmDialog( this, "Are you sure you want to end the game?", "End game", JOptionPane.YES_NO_OPTION );
	    if( answer == JOptionPane.YES_OPTION || answer == JOptionPane.OK_OPTION )
	    {
		System.exit( 0 );
	    }
	    else
	    {
		setVisible( true );
	    }
	}
    }
    
    public void windowClosed( WindowEvent e )
    {
	if( !finished )
	{
	    int answer = JOptionPane.showConfirmDialog( this, "Are you sure you want to end the game?", "End game", JOptionPane.YES_NO_OPTION );
	    if( answer == JOptionPane.YES_OPTION || answer == JOptionPane.OK_OPTION )
	    {
		System.exit( 0 );
	    }
	    else
	    {
		setVisible( true );
	    }
	}
    }
    
    public void windowIconified( WindowEvent e )
    {}
    
    public void windowDeiconified( WindowEvent e )
    {}
    
    public void windowOpened( WindowEvent e )
    {}
    
    public void windowActivated( WindowEvent e )
    {
	requestFocus();
    }
    
    public void windowDeactivated( WindowEvent e )
    {}

}
