// GenericMonster.java
// Started on July 30th, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;
import java.net.URL;
import java.io.*;

public class GenericMonster extends Monster implements Armed
{
    public static final String GENERIC_MONSTER_KEY = "(Generic)";
    
    public static final int ROAMING = 0;
    public static final int SEEKING = 1;
    public static final int ATTACKING = 2;
    public static final int SHOOTING = 3;
    public static final int WAITING = 4;
    public static final int ESCAPING = 5;
    public static final int PANICKING = 6;
    
    private static HashMap dataSets = null;
    
    private String type;
    private boolean reactsToSound = false;
    private int maximumShootRange = 16;
    private double regenerationProbability = 0;
    private int weaponSkill = 0;
    private int meleeAbility = 5;
    private int meleeCost = 3;
    private int size = 75;
    private double fanaticism = 1.0;
    private double turnProbability = 0.33;
    private Weapon weapon = null;
    private String [] foeTypes = { "Marine", "Bob" };

    private Character target = null;
    private Point targetPoint = null;
    private boolean hasMoreActions = false;
    private int state = ROAMING;
    private Character [] visibleEnemies = null;

    private GenericMonster()
    {
	if( dataSets == null )
	{
	    dataSets = new HashMap();
	}
	type = "Ripper";
        location = new Point( -1, -1 );
        direction = Map.NORTH;
        alive = true;
        lastAction = new MoveAction( this, location, location );
        maximumActionPoints = 6;
        currentActionPoints = 6;
        maximumHitPoints = 7;
        currentHitPoints = 7;
    }
    
    private GenericMonster( String type, HashMap stats )
    {
	this.type = type;
	String value;
	location = new Point( -1, -1 );
	direction = Map.NORTH;
	alive = true;
	lastAction = new MoveAction( this, location, location );
	       
	value = (String)stats.get( "hitPoints" );
	if( value != null )
	{
	    setCustomHitPoints( value );
	}
	value = (String)stats.get( "actionPoints" );
	if( value != null )
	{
	    setCustomActionPoints( value );
	}
	value = (String)stats.get( "reactsToSound" );
	if( value != null )
	{
	    reactsToSound = new Boolean( value ).booleanValue();
	}
	value = (String)stats.get( "maximumShootRange" );
	if( value != null )
	{
	    maximumShootRange = Objective.intFromString( value );
	}
	value = (String)stats.get( "regenerationProbability" );
	if( value != null )
	{
	    regenerationProbability = Objective.doubleFromString( value );
	}
	value = (String)stats.get( "weaponSkill" );
	if( value != null )
	{
	    weaponSkill = Objective.intFromString( value );
	}
	value = (String)stats.get( "meleeAbility" );
	if( value != null )
	{
	    meleeAbility = Objective.intFromString( value );
	}
	value = (String)stats.get( "meleeCost" );
	if( value != null )
	{
	    meleeCost = Objective.intFromString( value );
	}
	value = (String)stats.get( "size" );
	if( value != null )
	{
	    size = Objective.intFromString( value );
	}
	value = (String)stats.get( "fanaticism" );
	if( value != null )
	{
	    fanaticism = Objective.doubleFromString( value );
	}
	value = (String)stats.get( "turnProbability" );
	if( value != null )
	{
	    turnProbability = Objective.doubleFromString( value );
	}
	value = (String)stats.get( "weapon" );
	if( value != null )
	{
	    if( !value.equalsIgnoreCase( "Null" ) )
	    {
		try
		{
		    String weaponClass = value;
		    if( !weaponClass.startsWith( "TheGame." ) )
		    {
			weaponClass = "TheGame."+weaponClass;
		    }
		    weapon = (Weapon)Class.forName( weaponClass ).newInstance();
		}
		catch( Exception e )
		{
		    System.out.println( "Exception loading weapon for " + type + ": " + value );
		    e.printStackTrace();
		}
	    }
	}
	value = (String)stats.get( "foeTypes" );
	if( value != null )
	{
	    foeTypes = Objective.parseStrings( value );
	}
    }

    public void addKill()
    {}
    
    public void setSpottedPlayer( Character character, boolean canSee )
    {
	setDormant( false );
        if( canSee )
        {
            target = character;
            state = SEEKING;
            //System.out.println( getType() + " spotted player: " + character.getName() );
        }
	else
	{
	    if( reactsToSound )
	    {
		targetPoint = character.getLocation();
		state = SEEKING;
	    }
	}
    }

    public void setState( int state )
    {
        this.state = state;
    }

    public Weapon getWeapon()
    {
        return weapon;
    }

    public void setWeapon( Weapon weapon )
    {
	this.weapon = weapon;
    }    
    
    public void preAction()
    {
	visibleEnemies = activeGame.getVisibleEnemies( this );
	
	if( visibleEnemies.length > 0 )
	{
	    // Check for cowardism
	    if( ( currentHitPoints/(double)maximumHitPoints < fanaticism ) || ( fanaticism == 0 ) )
	    {
		state = ESCAPING;
		targetPoint = SeekerAI.getSafePointForCharacter( this );
		if( targetPoint == null )
		{
		    state = PANICKING;
		}
	    }
	    else
	    {
		if( target == null )
		{
		    target = activeGame.getClosestCharacter( this, visibleEnemies );
		    state = SEEKING;
		}
	    }
	}
    }
    
    public GameAction playAction()
    {
        int x;
        int y;

        Point cell;
        Character check;

        hasMoreActions = true;

	visibleEnemies = activeGame.getVisibleEnemies( this );
        if( alive && !dormant )
        {
	    //System.out.println( getType() + ": playAction" );
	    //System.out.println( getType() + " loitering at " + Objective.pointToString( location ) );
            if( target != null )
	    {
                if( !target.isAlive() )
                {
                    target = null;
                    state = ROAMING;
                }
                else
                {
                    state = SEEKING;
                }
	    }
            if( currentActionPoints <= 0 )
            {
                lastAction = new WaitAction( this, new Point( location ) );
                return lastAction;
            }
            // If next to an enemy, attack
            //System.out.println( getType() + " checking neighbours" );
            for( int i=0; i < visibleEnemies.length; i++ )
            {
                check = visibleEnemies[i];
                if( check.isAlive() )
                {
                    cell = check.getLocation();
                    if( activeGame.getMap().isNeighbor( location, cell ) )
                    {
                        if( currentActionPoints >= getMeleeCost() )
                        {
                            state = ATTACKING;
                            direction = Map.getDirection( location, cell, direction );
                            lastAction = new MeleeAction( this, check );
                            return lastAction;
                        }
                        else
                        {
                            state = WAITING;
                            lastAction = new WaitAction( this, new Point( location ) );
                            hasMoreActions = false;
                            return lastAction;
                        }
                    }
                }
            }
            // If not next to an enemy, but can see one, shoot it
            if( visibleEnemies.length > 0 )
            {
                target = activeGame.getClosestCharacter( this, visibleEnemies );
                if( weapon != null && activeGame.getMap().getLineOfFire( location, target.getLocation(), weapon.getBlockingSize() ).equals( target.getLocation() ) && ( location.distance( target.getLocation() ) <= maximumShootRange ) && ( currentActionPoints >= weapon.getActionsToFire() ) )
                {
                    state = SHOOTING;
                    direction = Map.getDirection( location, target.getLocation(), direction );
                    lastAction = new ShootAction( this, weapon, target.getLocation(), true );
                    return lastAction;
                }
                else
                {
                    state = SEEKING;
                }
            }
            if( state == SEEKING )
            {
                if( target != null || targetPoint != null )
                {
		    if( target != null )
		    {
			targetPoint = target.getLocation();
		    }
                    GameAction action = SeekerAI.seek( this, targetPoint );
                    if( action != null )
                    {
                        lastAction = action;
                        return lastAction;
                    }
                    else
                    {
                        state = ROAMING;
                        target = null;
			targetPoint = null;
                    }
                }
            }
	    if( state == ESCAPING )
	    {
		GameAction action = SeekerAI.seek( this, targetPoint );
		if( action != null )
		{
		    lastAction = action;
		    return lastAction;
		}
		else
		{
		    state = PANICKING;
		    target = null;
		    targetPoint = null;
		}
	    }
	    if( state == PANICKING )
            {
                // Run away from the closest enemy
                Character target = activeGame.getClosestCharacter( this, visibleEnemies );
		int tempDirection = Map.getDirection( location, target.getLocation(), direction );
                // Check the opposite direction
                int newDirection = (tempDirection+4)%8;
                if( activeGame.getMap().isPassable( location, newDirection ) )
                {
                    // This will do
                    direction = newDirection;
                }
                else
                {
                    newDirection = (tempDirection+3)%8;
                    if( activeGame.getMap().isPassable( location, newDirection ) )
                    {
                        direction = newDirection;
                    }
                    else
                    {
                        newDirection = (tempDirection+5)%8;
                        if( activeGame.getMap().isPassable( location, newDirection ) )
                        {
                            direction = newDirection;
                        }
                        else
                        {
                            newDirection = -1;
                        }
                    }
                }
                // If found suitable escape direction, run there
                if( newDirection > -1 )
                {
                    //System.out.println( getType() + " panicking from (" + location.x +","+location.y +") towards " +newDirection );
                    lastAction = new MoveAction( this, new Point( location ), activeGame.getMap().getNeighbor( location, direction ));
                    return lastAction;
                }
                else
                {
                    state = ROAMING;
                }
            }
            if( state == ROAMING )
            {
                // Roam around mindlessly like a common Oid
		lastAction = RoamerAI.roam( this, turnProbability );
		//System.out.println( getType() + " roaming" );
            }
        }
        return lastAction;
    }
    
    public void postAction()
    {
	visibleEnemies = activeGame.getVisibleEnemies( this );
	if( visibleEnemies.length == 0 )
	{
	    if( state == ESCAPING || state == PANICKING )
	    {
		state = ROAMING;
	    }
	}
	if( currentHitPoints < maximumHitPoints )
	{
	    // Try to regenerate
	    double probability = regenerationProbability;
	    while( Math.random() < regenerationProbability )
	    {
		currentHitPoints++;
		probability-=1.0;
	    }
	}
    }
	
    public String getClassName()
    {
	return type + GENERIC_MONSTER_KEY;
    }
    
    public String getType()
    {
	return type;
    }

    public GameAction getLastAction()
    {
	return lastAction;
    }

    public void registerHit()
    {}
    
    public void resetActionPoints()
    {
	currentActionPoints = maximumActionPoints;
    }
    
    public boolean hasMoreActions()
    {
	if( currentActionPoints > 0 && hasMoreActions )
	{
	    return true;
	}
	return false;
    }
    
    public int getMeleeAbility()
    {
	return meleeAbility;
    }
    
    public int getMeleeCost()
    {
	return meleeCost;
    }
    
    public int getWeaponSkill()
    {
	return weaponSkill;
    }
    
    public boolean isAlive()
    {
	return alive;
    }
    
    public boolean isFoe( Character character )
    {
        for( int i = 0; i < foeTypes.length; i++ )
        {
            if( character.getType().equals( foeTypes[ i ] ) )
                return true;
        }
        return false;
    }
    
    
    public boolean applyDamage( int damage )
    {
	currentHitPoints = currentHitPoints-damage;
	lastAction = new WoundAction( this, location );
	if( currentHitPoints <= 0 )
	{
	    alive = false;
	    lastAction = new DeadAction( this, new Point( location ) );
	    maximumActionPoints = 0;
	    currentActionPoints = 0;
	}
	return alive;
    }
    
    public int getSize()
    {
	return size;
    }
    
    public static HashMap getStatsFor( String type )
    {
	if( dataSets != null )
	{
	    return (HashMap)dataSets.get( type );
	}
	return null;
    }
    
    public static GenericMonster create( String type )
    {
	HashMap stats;
	if( dataSets == null )
	{
	    dataSets = new HashMap();
	}
	if( dataSets.containsKey( type ) )
	{
	    stats = (HashMap)dataSets.get( type );
	    GenericMonster monster = new GenericMonster( type, stats );
	    return monster;
	}
	else
	{
	    // Try to load the data set
	    try
	    {
		URL fileURL = ResourceManager.getResourceURL( type+".xml" );
		if( fileURL != null )
		{
		    //File monsterFile = new File( fileURL.toURI() );
		    //if( monsterFile.exists() && monsterFile.canRead() )
		    {
			System.out.println( "Parsing monster file for: " + type );
		    
			DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
			DOMImplementation builder = documentBuilder.getDOMImplementation();
		    
			Document document = documentBuilder.parse( fileURL.openStream() );
	    	    
			Element root = document.getDocumentElement();
			if( root.getNodeName().equals( "genericmonster" ) )
			{
			    stats = new HashMap();
			    Element [] attrs = XMLUtils.getNamedChildren( root, "attribute" );
			    for( int i=0; i < attrs.length; i++ )
			    {
				String key = attrs[i].getAttribute( "name" );
				String value = attrs[i].getAttribute( "value" );
				stats.put( key, value );
			    }
			    Element spritesElement = XMLUtils.getNamedChild( root, "sprites" );
			    stats.put( "Sprites", spritesElement );
			    System.out.println( "Adding dataset for " + type );
			    dataSets.put( type, stats );
			    GenericMonster monster = new GenericMonster( type, stats );
			    return monster;
			}
		    }
		}
	    }
	    catch( Exception e )
	    {
		System.out.println( "Exception loading data set for: " + type );
		e.printStackTrace();
	    }
	}
	System.out.println( "Data for type not found: " + type );
	return null;
    }
}