// GenericMonsterSprite.java
// Started on July 30th, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import java.util.*;
import org.w3c.dom.*;

public class GenericMonsterSprite extends CharacterSprite
{
    private static HashMap imageSets = null;
    public static final int WALK_SET_KEY = 0;
    public static final int MELEE_SET_KEY = 1;
    public static final int SHOOT_SET_KEY = 2;
    public static final int USE_SET_KEY = 3;
    public static final int WOUND_SET_KEY = 4;
    public static final int DEAD_SET_KEY = 5;
    
    public static final String [] directionCodes = { "n", "ne", "e", "se", "s", "sw", "w", "nw" };
    
    private int deadDir = -1;
    private int size = 96;
    private int xOffset = 0;
    private int yOffset = 0;
    private String type = "";
    private int visualBoundsSize = 48;
    
    public GenericMonsterSprite()
    {
	if( imageSets == null )
	{
	    imageSets = new HashMap();
	}
	location = new Point( 0, 0 );
        bounds = new Rectangle( 0, 0, size, size );
        direction = Map.NORTH;
    }
    
    public void setType( String type )
    {
	this.type = type;
	if( !imageSets.containsKey( type ) )
	{
	    String value = null;
	    HashMap stats = GenericMonster.getStatsFor( type );
	    if( stats != null )
	    {
		System.out.println( "Creating sprite set for " + type );
		try
		{
		    Element spritesElement = (Element)stats.get( "Sprites" );
		    if( spritesElement != null )
		    {
			size = Objective.intFromString( spritesElement.getAttribute( "size" ) );
			int x = Objective.intFromString( spritesElement.getAttribute( "x" ) );
			int y = Objective.intFromString( spritesElement.getAttribute( "y" ) );
			xOffset = Objective.intFromString( spritesElement.getAttribute( "xoffset" ) );
			yOffset = Objective.intFromString( spritesElement.getAttribute( "yoffset" ) );
			visualBoundsSize = Objective.intFromString( spritesElement.getAttribute( "visualsize" ) );

			String spriteFileName = spritesElement.getAttribute( "file" );
			Vector images = new Vector();
			ResourceManager.cropImage( spriteFileName, images, size );
			
			SpriteBundle spriteBundle = new SpriteBundle( size );
			
			Element [] frames;
			Element walkElement = XMLUtils.getNamedChild( spritesElement, "walk" );
			if( walkElement != null )
			{
			    frames = XMLUtils.getNamedChildren( walkElement, "frame" );
			    spriteBundle.createWalkImages( frames.length );
			    for( int i=0; i < frames.length; i++ )
			    {
				int index = Objective.intFromString( frames[i].getAttribute( "index" ) );
				for( int j=0; j < directionCodes.length; j++ )
				{
				    int spriteIndex = Objective.intFromString( frames[i].getAttribute( directionCodes[ j ] ) );
				    Image image = (Image)images.elementAt( spriteIndex );
				    spriteBundle.setImage( WALK_SET_KEY, j, index, image );
				}
			    }
			}
			Element meleeElement = XMLUtils.getNamedChild( spritesElement, "melee" );
			if( meleeElement != null )
			{
			    frames = XMLUtils.getNamedChildren( meleeElement, "frame" );
			    spriteBundle.createMeleeImages( frames.length );
			    for( int i=0; i < frames.length; i++ )
			    {
				int index = Objective.intFromString( frames[i].getAttribute( "index" ) );
				for( int j=0; j < directionCodes.length; j++ )
				{
				    int spriteIndex = Objective.intFromString( frames[i].getAttribute( directionCodes[ j ] ) );
				    Image image = (Image)images.elementAt( spriteIndex );
				    spriteBundle.setImage( MELEE_SET_KEY, j, index, image );
				}
			    }
			}
			Element shootElement = XMLUtils.getNamedChild( spritesElement, "shoot" );
			if( shootElement != null )
			{
			    frames = XMLUtils.getNamedChildren( shootElement, "frame" );
			    spriteBundle.createShootImages( frames.length );
			    for( int i=0; i < frames.length; i++ )
			    {
				int index = Objective.intFromString( frames[i].getAttribute( "index" ) );
				for( int j=0; j < directionCodes.length; j++ )
				{
				    int spriteIndex = Objective.intFromString( frames[i].getAttribute( directionCodes[ j ] ) );
				    Image image = (Image)images.elementAt( spriteIndex );
				    spriteBundle.setImage( SHOOT_SET_KEY, j, index, image );
				}
			    }
			}
			Element useElement = XMLUtils.getNamedChild( spritesElement, "use" );
			if( useElement != null )
			{
			    frames = XMLUtils.getNamedChildren( useElement, "frame" );
			    spriteBundle.createUseImages( frames.length );
			    for( int i=0; i < frames.length; i++ )
			    {
				int index = Objective.intFromString( frames[i].getAttribute( "index" ) );
				for( int j=0; j < directionCodes.length; j++ )
				{
				    int spriteIndex = Objective.intFromString( frames[i].getAttribute( directionCodes[ j ] ) );
				    Image image = (Image)images.elementAt( spriteIndex );
				    spriteBundle.setImage( USE_SET_KEY, j, index, image );
				}
			    }
			}
			Element woundElement = XMLUtils.getNamedChild( spritesElement, "wound" );
			if( woundElement != null )
			{
			    frames = XMLUtils.getNamedChildren( woundElement, "frame" );
			    spriteBundle.createWoundImages( frames.length );
			    for( int i=0; i < frames.length; i++ )
			    {
				int index = Objective.intFromString( frames[i].getAttribute( "index" ) );
				for( int j=0; j < directionCodes.length; j++ )
				{
				    int spriteIndex = Objective.intFromString( frames[i].getAttribute( directionCodes[ j ] ) );
				    Image image = (Image)images.elementAt( spriteIndex );
				    spriteBundle.setImage( WOUND_SET_KEY, j, index, image );
				}
			    }
			}
			Element deadElement = XMLUtils.getNamedChild( spritesElement, "dead" );
			if( deadElement != null )
			{
			    frames = XMLUtils.getNamedChildren( deadElement, "frame" );
			    spriteBundle.createDeadImages( frames.length );
			    for( int i=0; i < frames.length; i++ )
			    {
				int index = Objective.intFromString( frames[i].getAttribute( "index" ) );
				int spriteIndex = Objective.intFromString( frames[i].getAttribute( "sprite" ) );
				Image image = (Image)images.elementAt( spriteIndex );
				spriteBundle.setImage( DEAD_SET_KEY, i, index, image );
			    }
			}
			// Okay, we should have all the images now...
			System.out.println( "Adding sprite bundle for " + type );
			imageSets.put( type, spriteBundle );
		    }
		}
		catch( Exception e )
		{
		    System.out.println( "Exception loading sprite data for " + type );
		    e.printStackTrace();
		}
	    }
	}
	else
	{
	    SpriteBundle spriteBundle = (SpriteBundle)imageSets.get( type );
	    this.size = spriteBundle.getSize();
	}
	xOffset = (int)(48*48/(double)size);
	yOffset = (int)(48*48/(double)size);
    }
    
    public void setCharacter( Character character )
    {
	this.character = character;
    }
    
    public Image getSprite( GameAction action )
    {
	SpriteBundle spriteBundle = (SpriteBundle)imageSets.get( type );
	if( spriteBundle != null )
	{
	    if( action instanceof DeadAction )
	    {
		return spriteBundle.getImage( DEAD_SET_KEY, -1, frame );
	    }
	    if( action instanceof MoveAction )
	    {
		return spriteBundle.getImage( WALK_SET_KEY, direction, frame );
	    }
	    if( action instanceof MeleeAction )
	    {
		return spriteBundle.getImage( MELEE_SET_KEY, direction, frame );
	    }
	    if( action instanceof WoundAction )
	    {
		return spriteBundle.getImage( WOUND_SET_KEY, direction, frame );
	    }
	    if( action instanceof ShootAction )
	    {
		return spriteBundle.getImage( SHOOT_SET_KEY, direction, frame );
	    }
	    if( action instanceof UseAction )
	    {
		return spriteBundle.getImage( USE_SET_KEY, direction, frame );
	    }
	    if( action instanceof WaitAction )
	    {
		return spriteBundle.getImage( WALK_SET_KEY, direction, 0 );
	    }
	}
	return null;
    }
    
    public Image getSprite()
    {
	SpriteBundle spriteBundle = (SpriteBundle)imageSets.get( type );
	if( spriteBundle != null )
	{
	    if( deadDir == -1 )
	    {
		deadDir = (int)(Math.random()*spriteBundle.getSetSize( DEAD_SET_KEY ) );
	    }
	    return spriteBundle.getImage( DEAD_SET_KEY, -1, deadDir );
	}
        return null;
    }
    
    public int getNumberOfRangedAttackFrames()
    {
	SpriteBundle spriteBundle = (SpriteBundle)imageSets.get( type );
	if( spriteBundle != null )
	{
	    return spriteBundle.getSetSize( SHOOT_SET_KEY );
	}
        return 0;
    }

    
    public int getNumberOfWalkFrames()
    {
	SpriteBundle spriteBundle = (SpriteBundle)imageSets.get( type );
	if( spriteBundle != null )
	{
	    return spriteBundle.getSetSize( WALK_SET_KEY );
	}
        return 10;
    }
    
    public int getNumberOfMeleeAttackFrames()
    {
	SpriteBundle spriteBundle = (SpriteBundle)imageSets.get( type );
	if( spriteBundle != null )
	{
	    return spriteBundle.getSetSize( MELEE_SET_KEY );
	}
        return 10;
    }
    
    public Rectangle getVisualBounds()
    {
	Rectangle rect = new Rectangle( location.x+xOffset, location.y+yOffset, visualBoundsSize, visualBoundsSize );
	return rect;
    }
    
    public void setLocation( Point location )
    {
        this.location.x = location.x-xOffset;
        this.location.y = location.y-yOffset;
        bounds.x = location.x;
        bounds.y = location.y;
    }
    
    public void setLocation( int x, int y )
    {
        location.x = x-xOffset;
        location.y = y-yOffset;
        bounds.x = x-xOffset;
        bounds.y = y-yOffset;
    }
}

class SpriteBundle
{
    private Image [][] walkImages;
    private Image [][] meleeImages;
    private Image [][] shootImages;
    private Image [][] useImages;
    private Image [][] woundImages;
    private Image [] deadImages;
    private int size = 48;
    private int walkFrames = 1;
    private int meleeFrames = 1;
    private int shootFrames = 1;
    private int useFrames = 1;
    private int woundFrames = 1;
    private int deadFrames = 1;
    
    public SpriteBundle( int size )
    {
	this.size = size;
    }
    
    public void createWalkImages( int sets )
    {
	walkImages = new Image[ sets ][8];
	walkFrames = sets;
	//System.out.println( "SpriteBundle: walk set size " + sets );
    }
    
    public void createMeleeImages( int sets )
    {
	meleeImages = new Image[ sets ][8];
	meleeFrames = sets;
	//System.out.println( "SpriteBundle: melee set size " + sets );
    }    
    
    public void createShootImages( int sets )
    {
	shootImages = new Image[ sets ][8];
	shootFrames = sets;
	//System.out.println( "SpriteBundle: shoot set size " + sets );
    }
    
    public void createUseImages( int sets )
    {
	useImages = new Image[ sets ][8];
	useFrames = sets;
	//System.out.println( "SpriteBundle: use set size " + sets );
    }
    
    public void createWoundImages( int sets )
    {
	woundImages = new Image[ sets ][8];
	woundFrames = sets;
	//System.out.println( "SpriteBundle: wound set size " + sets );
    }
    
    public void createDeadImages( int sprites )
    {
	deadImages = new Image[ sprites ];
	deadFrames = sprites;
	//System.out.println( "SpriteBundle: dead set size " + sprites );
    }
    
    public void setImage( int setKey, int direction, int frame, Image image )
    {
	//System.out.println( "SpriteBundle setting image" );
	switch( setKey )
	{
	    case GenericMonsterSprite.WALK_SET_KEY:
		walkImages[ frame ][ direction ] = image;
		break;
	    case GenericMonsterSprite.MELEE_SET_KEY:
		meleeImages[ frame ][ direction ] = image;
		break;
	    case GenericMonsterSprite.SHOOT_SET_KEY:
		shootImages[ frame ][ direction ] = image;
		break;
	    case GenericMonsterSprite.USE_SET_KEY:
		useImages[ frame ][ direction ] = image;
		break;
	    case GenericMonsterSprite.WOUND_SET_KEY:
		woundImages[ frame ][ direction ] = image;
		break;
	    case GenericMonsterSprite.DEAD_SET_KEY:
		deadImages[ frame ] = image;
		break;
	}
    }
    
    public Image getImage( int setKey, int direction, int frame )
    {
	Image image = null;
	int theFrame = frame;
	switch( setKey )
	{
	    case GenericMonsterSprite.WALK_SET_KEY:
		if( theFrame >= walkFrames )
		{
		    theFrame = theFrame % walkFrames;
		}
		//System.out.println( "Getting walk frame: " + frame + "(got " + theFrame + ")" );
		image = walkImages[ theFrame ][ direction ];
		break;
	    case GenericMonsterSprite.MELEE_SET_KEY:
		if( theFrame >= meleeFrames )
		{
		    theFrame = theFrame % meleeFrames;
		}
		image = meleeImages[ theFrame ][ direction ];
		break;
	    case GenericMonsterSprite.SHOOT_SET_KEY:
		if( theFrame >= shootFrames )
		{
		    theFrame = 0;
		}
		image = shootImages[ theFrame ][ direction ];
		break;
	    case GenericMonsterSprite.USE_SET_KEY:
		if( theFrame >= useFrames )
		{
		    theFrame = 0;   
		}
		image = useImages[ theFrame ][ direction ];
		break;
	    case GenericMonsterSprite.WOUND_SET_KEY:
		if( theFrame >= woundFrames )
		{
		    theFrame = 0;
		}
		image = woundImages[ theFrame ][ direction ];
		break;
	    case GenericMonsterSprite.DEAD_SET_KEY:
		if( theFrame >= deadFrames )
		{
		    theFrame = 0;
		}
		image = deadImages[ theFrame ];
		break;
	}
	return image;
    }
    
    public int getSetSize( int setKey )
    {
	int setSize = 1;
	switch( setKey )
	{
	    case GenericMonsterSprite.WALK_SET_KEY:
		setSize = walkFrames;
		break;
	    case GenericMonsterSprite.MELEE_SET_KEY:
		setSize = meleeFrames;
		break;
	    case GenericMonsterSprite.SHOOT_SET_KEY:
		setSize = shootFrames;
		break;
	    case GenericMonsterSprite.USE_SET_KEY:
		setSize = useFrames;
		break;
	    case GenericMonsterSprite.WOUND_SET_KEY:
		setSize = woundFrames;
		break;
	    case GenericMonsterSprite.DEAD_SET_KEY:
		setSize = deadFrames;
		break;
	}
	return setSize;
    }
    
    public int getSize()
    {
	return size;
    }
    
}