// ImageStain.java
// Started on February 25th, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Image;

public class ImageStain extends Sprite
{
    private Image image;
    private Sprite sprite = null;
    private boolean visible = false;
    private int xOffset = 0;
    private int yOffset = 0;

    public ImageStain( Image image )
    {
        this.image = image;
        location = new Point( -1, -1 );
        bounds = new Rectangle( -1, -1, 1, 1 );
    }

    public ImageStain( Sprite sprite )
    {
        if( sprite.isAnimated() )
        {
            this.sprite = sprite;
        }
        this.image = sprite.getSprite();
        location = new Point( -1, -1 );
        xOffset = sprite.getXOffset();
        yOffset = sprite.getYOffset();
        bounds = new Rectangle( sprite.getBounds() );
    }
    
    public Image getSprite()
    {
        if( sprite != null )
        {
            return sprite.getSprite();
        }
        return image;
    }

    public Image getSprite( GameAction action )
    {
        return image;
    }

    public boolean isVisible()
    {
        return visible;
    }

    public void setVisible( boolean visible )
    {
        this.visible = visible;
    }

    /**
     * Set the bounding box of the stain. The stains are drawn according
     * to their bounding box location on the GameMapPanel instead of
     * their locaction, which represents the stain object's location
     * on the map.
     *
     * @param x		the x coordinate of the stain sprite
     * @param y		the y coordinate of the stain sprite
     * @param width	the width of the stain sprite
     * @param height	the height of the stain sprite
     */
    public void setBounds( int x, int y, int width, int height )
    {
        bounds.x = x;
        bounds.y = y;
        bounds.width = width;
        bounds.height = height;
    }

    public int getXOffset()
    {
        return xOffset;
    }

    public int getYOffset()
    {
        return yOffset;
    }

}