// MapObject.java
// Started on July 20th, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.util.Vector;
import java.awt.Point;

public class MapObject
{
    private static int lastId = -1;
    
    private String name ="";
    private String id;
    private Vector cells;
    private int minX=-1;
    private int minY=-1;
    private int maxX=-1;
    private int maxY=-1;
    private int strength = -1;
    private int costToMove = -1;
    
    public MapObject()
    {
	id = "Object" + (++lastId);
	cells = new Vector();
    }
    
    public MapObject( String id, String name, int strength, int costToMove )
    {
	this.id = id;
	this.name = name;
	this.strength = strength;
	this.costToMove = costToMove;
	cells = new Vector();
	
	int idNumber = Integer.parseInt( id.substring( 6 ) );
	if( lastId <= idNumber )
	{
	    lastId = idNumber;
	}
    }
    
    public void addObjectCell( MapObjectCell cell )
    {
	if( !cells.contains( cell ) )
	{
	    cells.add( cell );
	    if( minX < 0 )
	    {
		minX = cell.getX();
	    }
	    else
	    {
		minX = Math.min( minX, cell.getX() );
	    }
	    if( maxX < 0 )
	    {
		maxX = cell.getX();
	    }
	    else
	    {
		maxX = Math.max( maxX, cell.getX() );
	    }
	    if( minY < 0 )
	    {
		minY = cell.getY();
	    }
	    else
	    {
		minY = Math.min( minY, cell.getY() );
	    }
	    if( maxY < 0 )
	    {
		maxY = cell.getY();
	    }
	    else
	    {
		maxY = Math.max( maxY, cell.getY() );
	    }
	    if( strength < cell.getHitPoints() )
	    {
		strength = cell.getHitPoints();
	    }
	}
    }
    
    public void calculateSize()
    {
	minX = Integer.MAX_VALUE;
	minY = Integer.MAX_VALUE;
	maxX = Integer.MIN_VALUE;
	maxY = Integer.MIN_VALUE;
	MapObjectCell [] theCells = getCells();
	for( int i=0; i < theCells.length; i++ )
	{
	    minX = Math.min( minX, theCells[i].getX() );
	    minY = Math.min( minY, theCells[i].getY() );
	    maxX = Math.max( maxX, theCells[i].getX() );
	    maxY = Math.max( maxY, theCells[i].getY() );
	}
    }
    
    public int getCostToMove()
    {
	return costToMove;
    }
    
    public void setCostToMove( int costToMove )
    {
	this.costToMove = costToMove;
    }
    
    public String getId()
    {
	return id;
    }
    
    public String getName()
    {
	return name;
    }
    
    public void setName( String name )
    {
	this.name = name;
    }
    
    public Point getTopLeft()
    {
	return new Point( minX, minY );
    }
    
    public Point getBottomRight()
    {
	return new Point( maxX, maxY );
    }
    
    public int getWidth()
    {
	return (maxX-minX)+1;
    }
    
    public int getHeight()
    {
	return (maxY-minY)+1;
    }
    
    public int getStrength()
    {
	return strength;
    }
    
    public void setStrength( int strength )
    {
	this.strength = strength;
    }
    
    public boolean isBlockingMovement( Point point )
    {
	if( isLocatedIn( point ) )
	{
	    MapObjectCell [] theCells = getCells( point.x, point.y );
	    for( int i=0; i < theCells.length; i++ )
	    {
		if( theCells[i].isBlockingMovement() )
		{
		    return true;
		}
	    }
	}
	return false;
    }
    
    public boolean isBlockingSight( Point point )
    {
	if( isLocatedIn( point ) )
	{
	    MapObjectCell [] theCells = getCells( point.x, point.y );
	    for( int i=0; i < theCells.length; i++ )
	    {
		if( theCells[i].isBlockingSight() )
		{
		    return true;
		}
	    }
	}
	return false;
    }
    
    public boolean isLocatedIn( Point point )
    {
	return isLocatedIn( point.x, point.y );
    }
    
    public boolean isDestroyed()
    {
	return (strength <= 0);
    }
    
    public boolean isLocatedIn( int x, int y )
    {
	for( int i=0; i < cells.size(); i++ )
	{
	    MapObjectCell cell = (MapObjectCell)cells.elementAt( i );
	    if( cell.isLocatedIn( x, y ) )
	    {
		return true;
	    }
	}
	return false;
    }
    
    public MapObjectCell [] getCells()
    {
	MapObjectCell [] theCells = new MapObjectCell[ cells.size() ];
	for( int i=0; i < cells.size(); i++ )
	{
	    theCells[i] = (MapObjectCell)cells.elementAt( i );
	}
	return theCells;
    }
    
    public MapObjectCell [] getCells( int x, int y )
    {
	MapObjectCell [] theCells = null;
	Vector help = new Vector();
	for( int i=0; i < cells.size(); i++ )
	{
	    MapObjectCell cell = (MapObjectCell)cells.elementAt( i );
	    if( cell.isLocatedIn( x, y ) )
	    {
		help.add( cell );
	    }
	}
	if( help.size() > 0 )
	{
	    theCells = new MapObjectCell[ help.size() ];
	    for( int i=0; i < help.size(); i++ )
	    {
		theCells[i] = (MapObjectCell)help.elementAt( i );
	    }
	}
	return theCells;
    }
    
    // returns true if destroyed
    public boolean applyDamage( int damage )
    {
        //System.out.println( name + " takes " + damage + " points of damage against " + strength + " hit points" );
        if( ( damage > strength/2 ) && ( strength > 0 ) )
        {
            strength-=(damage-strength/2);
        }
        if( strength <= 0 )
	{
	    for( int i=0; i < cells.size(); i++ )
	    {
		((MapObjectCell)cells.elementAt(i)).setDestroyed( true );
	    }
            return true;
	}
        return false;
    }
    
    public String toString()
    {
	return name + " (" + id + "), " + cells.size() + " components";
    }
}