// MarineSprite.java
// Started on June 20th, 2001
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import java.util.Vector;

public class MarineSprite extends CharacterSprite
{
    // Smartgun, flamer and autocannon share the sprites
    private static final int PISTOL = 0;
    private static final int ASSAULT_RIFLE = 1;
    private static final int SMART_GUN = 2;
    private static final int MINIGUN = 3;
    private static final int FLAMER = 4;
    private static final int AUTO_CANNON = 5;
    private static final int SUBMACHINE_GUN = 6;
    private static Image [][][] images = null;
    
    private int weaponIndex = ASSAULT_RIFLE;
    
    private void checkWeapon()
    {
	weaponIndex = ASSAULT_RIFLE;
	if( character instanceof Armed )
	{
	    Weapon weapon = ((Armed)character).getWeapon();
	    if( weapon instanceof Pistol )
	    {
		weaponIndex = PISTOL;
	    }
	    if( weapon instanceof Flamer )
	    {
		weaponIndex = FLAMER;
	    }
	    if( weapon instanceof SmartGun )
	    {
		weaponIndex = SMART_GUN;   
	    }
	    if( weapon instanceof MiniGun )
	    {
		weaponIndex = MINIGUN;
	    }
	    if( weapon instanceof AutoCannon )
	    {
		weaponIndex = AUTO_CANNON;
	    }
	    if( weapon instanceof SubmachineGun )
	    {
		weaponIndex = SUBMACHINE_GUN;
	    }
	    if( weapon instanceof PlasmaBlaster )
	    {
		weaponIndex = FLAMER;
	    }
	}
    }
    
    public MarineSprite()
    {
	if( images == null )
	{
	    images = new Image[4][7][8];
	    Vector temp = new Vector();
	    // Load all the images and hope they end up in the correct slots
	    ResourceManager.cropImage( "MarinePistol.png", temp, 96 );
	    int index = 0;
	    for( int row=0; row < 7; row++ )
	    {
		for( int dir=0; dir < 8; dir++ )
		{
		    images[ PISTOL ][ row ][ dir ] = (Image)temp.elementAt( index++ );
		    //System.out.println( "Row: " + row + " Dir: " + dir + " Index: " + (index-1) );
		}
	    }
	    temp = new Vector();
	    ResourceManager.cropImage( "MarineAssaultRifle.png", temp, 96 );
	    index = 0;
	    for( int row=0; row < 7; row++ )
	    {
		for( int dir=0; dir < 8; dir++ )
		{
		    images[ ASSAULT_RIFLE ][ row ][ dir ] = (Image)temp.elementAt( index++ );
		    //System.out.println( "Row: " + row + " Dir: " + dir + " Index: " + (index-1) );
		}
	    }
	    temp = new Vector();
	    ResourceManager.cropImage( "MarineBigGun.png", temp, 96 );
	    index = 0;
	    for( int row=0; row < 7; row++ )
	    {
		for( int dir=0; dir < 8; dir++ )
		{
		    images[ SMART_GUN ][ row ][ dir ] = (Image)temp.elementAt( index++ );
		    //System.out.println( "Row: " + row + " Dir: " + dir + " Index: " + (index-1) );
		}
	    }
	    temp = new Vector();
	    ResourceManager.cropImage( "MarineMiniGun.png", temp, 96 );
	    index = 0;
	    for( int row=0; row < 7; row++ )
	    {
		for( int dir=0; dir < 8; dir++ )
		{
		    images[ MINIGUN ][ row ][ dir ] = (Image)temp.elementAt( index++ );
		    //System.out.println( "Row: " + row + " Dir: " + dir + " Index: " + (index-1) );
		}
	    }
	    
	}
 	location = new Point( 0, 0 );
	bounds = new Rectangle( 0, 0, 96, 96 );
	direction = Map.NORTH;
    }
    
    public Image getSprite( GameAction action )
    {
	checkWeapon();
	int theIndex = weaponIndex;
	if( weaponIndex == FLAMER || weaponIndex == AUTO_CANNON )
	{
	    theIndex = SMART_GUN;
	}
	if( weaponIndex == SUBMACHINE_GUN )
	{
	    theIndex = PISTOL;
	}
	if( action instanceof MoveAction )
	{
	    int theFrame = 0;
	    switch( frame )
	    {
		case 1:
		    theFrame = 1;
		    break;
		case 3:
		    theFrame = 2;
		    break;
	    }
	    return images[ theIndex ][ theFrame ][ direction ];
	}
	if( action instanceof ShootAction )
	{
	    int theFrame = 3;
	    if( weaponIndex == ASSAULT_RIFLE || weaponIndex == PISTOL || weaponIndex == AUTO_CANNON )
	    {
		if( frame == 1 )
		{
		    theFrame = 4;
		}
	    }
	    if( weaponIndex == SMART_GUN || weaponIndex == MINIGUN || weaponIndex == SUBMACHINE_GUN )
	    {
		theFrame = 3+(frame%2);
	    }
	   
	    return images[ theIndex ][ theFrame ][ direction ];
	}
	if( action instanceof MeleeAction )
	{
	    return images[ theIndex ][ 5 ][ direction ];
	}
	if( action instanceof WoundAction )
	{
	    return images[ theIndex ][ 6 ][ direction ];
	}
        return images[ theIndex ][ 0 ][ direction ];
	
    }

    public int getNumberOfWalkFrames()
    {
	return 5;
    }
    
    public int getNumberOfMeleeAttackFrames()
    {
	return 10;
    }

    public int getNumberOfRangedAttackFrames()
    {
	checkWeapon();
	int theFrames = 3;
	if( weaponIndex == FLAMER )
	{
	    theFrames = 1;   
	}
	if( weaponIndex == SMART_GUN )
	{
	    theFrames = 9;
	}
	if( weaponIndex == MINIGUN )
	{
	    theFrames = 11;
	}
	if( weaponIndex == SUBMACHINE_GUN )
	{
	    theFrames = 9;
	}
	return theFrames;
    }
    
    public Rectangle getVisualBounds()
    {
	return new Rectangle( location.x+24, location.y+24, 48, 48 );
    }
    
    public void setLocation( Point location)
    {
        this.location.x = location.x-24;
        this.location.y = location.y-24;
        bounds.x = location.x;
        bounds.y = location.y;
    }
    
    public void setLocation( int x, int y )
    {
        location.x = x-24;
        location.y = y-24;
        bounds.x = x-24;
        bounds.y = y-24;
    }
    
}