// MarineStatisticsPanel.java
// Started on June 29th, 2001
// Copyright (c) Virtu Halttunen

package TheGame;
import java.awt.*;
import javax.swing.*;
import java.util.Vector;

public class MarineStatisticsPanel extends JPanel
{
    public static final int HEADS = 7;
    public static final int MOUTHS = 11;
    public static final int NOSES = 6;
    public static final int EYES = 8;
    public static final int BROWS = 7;
    public static final int EXTRAS = 6;
    public static final int HAIRS = 12;
    public static final int EARS = 4;
    public static final int HELMETS = 2;

    private Marine marine = null;
    private static boolean imagesLoaded = false;
    private static Vector headImages;
    private static Vector mouthImages;
    private static Vector noseImages;
    private static Vector eyeImages;
    private static Vector browsImages;
    private static Vector extrasImages;
    private static Vector hairImages;
    private static Image backImage;
    private static Image neckImage;
    private static Vector helmetImages;
    private static Vector earsImages;
    private boolean enabled = true;

    private boolean faceOnly = false;
    private boolean showWeapon = false;
    private boolean showWeaponSelectionHint = false;
    private Polygon weaponHint;

    public MarineStatisticsPanel()
    {
	super();
        setSize();
        if( !imagesLoaded )
        {
            // Load images
            Image image;
            earsImages = new Vector();
            for( int i = 1; i <= EARS; i++ )
            {
                image = ResourceManager.loadImage( "SoldierPNGs/Ears"+i+".png" );
                earsImages.add( image );
            }
            headImages = new Vector();
            for( int i = 1; i <= HEADS; i++ )
            {
                image = ResourceManager.loadImage( "SoldierPNGs/Head"+i+".png" );
                headImages.add( image );
            }
            mouthImages = new Vector();
            for( int i = 1; i <= MOUTHS; i++ )
            {
                image = ResourceManager.loadImage( "SoldierPNGs/Mouth"+i+".png" );
                mouthImages.add( image );
            }
            noseImages = new Vector();
            for( int i = 1; i <= NOSES; i++ )
            {
                image = ResourceManager.loadImage( "SoldierPNGs/Nose"+i+".png" );
                noseImages.add( image );
            }
            eyeImages = new Vector();
            for( int i = 1; i <= EYES; i++ )
            {
                image = ResourceManager.loadImage( "SoldierPNGs/Eyes"+i+".png" );
                eyeImages.add( image );
            }
            browsImages = new Vector();
            for( int i = 1; i <= BROWS; i++ )
            {
                image = ResourceManager.loadImage( "SoldierPNGs/Brows"+i+".png" );
                browsImages.add( image );
            }	
            extrasImages = new Vector();
            for( int i = 1; i <= EXTRAS; i++ )
            {
                image = ResourceManager.loadImage( "SoldierPNGs/Extras"+i+".png" );
                extrasImages.add( image );
            }
            hairImages = new Vector();
            for( int i = 1; i <= HAIRS; i++ )
            {
                image = ResourceManager.loadImage( "SoldierPNGs/Hair"+i+".png" );
                hairImages.add( image );
            }
            helmetImages = new Vector();
            for( int i = 1; i <= HELMETS; i++ )
            {
                image = ResourceManager.loadImage( "SoldierPNGs/Helmet"+i+".png" );
                helmetImages.add( image );
            }	
            neckImage = ResourceManager.loadImage( "SoldierPNGs/Neck.png" );
            backImage = ResourceManager.loadImage( "SoldierPNGs/Back.png" );
            imagesLoaded = true;
        }
        weaponHint = new Polygon();
        weaponHint.addPoint( 125, 150 );
        weaponHint.addPoint( 135, 150 );
        weaponHint.addPoint( 130, 160 );
    }
    
    public void paintComponent( Graphics g )
    {
	g.setColor( Color.darkGray );
        int width = 140;
        int faceCoord = 20;
        if( faceOnly )
        {
            width = 120;
            faceCoord = 10;
        }
        int height = 165;
        if( showWeapon )
        {
            height = 200;
        }
	g.fillRect( 0, 0, width, height );
        if( marine != null )
        {
            g.drawImage( neckImage, faceCoord, 0, this );
            g.drawImage( backImage, faceCoord, 0, this );
            g.drawImage( (Image)earsImages.elementAt( marine.getEarsIndex() ), faceCoord, 0, this );
            g.drawImage( (Image)headImages.elementAt( marine.getHeadIndex() ), faceCoord, 0, this );
            g.drawImage( (Image)mouthImages.elementAt( marine.getMouthIndex() ), faceCoord+marine.getMouthXOffset(), 0+marine.getMouthYOffset(), this );
            g.drawImage( (Image)eyeImages.elementAt( marine.getEyeIndex() ), faceCoord, 0+marine.getEyeOffset(), this );
            g.drawImage( (Image)noseImages.elementAt( marine.getNoseIndex() ), faceCoord, 0, this );
            if( marine.getBrowsIndex() > -1 )
                g.drawImage( (Image)browsImages.elementAt( marine.getBrowsIndex() ), faceCoord, 0+marine.getBrowsOffset(), this );
            if( marine.getExtrasIndex() > -1 )
                g.drawImage( (Image)extrasImages.elementAt( marine.getExtrasIndex() ), faceCoord, 0, this );
            g.drawImage( (Image)hairImages.elementAt( marine.getHairIndex() ), faceCoord, 0, this );
            if( marine.getHelmetIndex() > -1 )
                g.drawImage( (Image)helmetImages.elementAt( marine.getHelmetIndex() ), faceCoord, 0, this );

            g.setColor( Color.white );
            g.drawString( marine.getName(), 5, 162 );
            if( !faceOnly )
            {
                g.setColor( Color.green );
                for( int i=0; i < marine.getCurrentActionPoints(); i++ )
                {
                    g.fill3DRect( 5, width-i*15, 10, 10, true );
                }
                if( marine.getCurrentHitPoints() > 10 )
                {
                    g.setColor( Color.green.darker() );
                }
                if( marine.getCurrentHitPoints() <= 6 )
                {
                    g.setColor( Color.yellow );
                }
                if( marine.getCurrentHitPoints() <= 3 )
                {
                    g.setColor( Color.red );
                }
                if( marine.getCurrentHitPoints() > 0 )
                {
                    int step = (int)(100.0/(double)marine.getMaximumHitPoints());	// How many pixels per hit point
                    int range = step*marine.getCurrentHitPoints();			// How many pixels it all takes
                    int diff = 100-step*marine.getMaximumHitPoints();		// The remainder
                    if( marine.getCurrentHitPoints() < 1 )
                        diff = 0;
                    g.fill3DRect( 125, 150-range-diff, 10, range+diff, true );
                }
            }
            if( showWeapon )
            {
                g.setColor( Color.white );
                Weapon weapon = marine.getWeapon();
                if( weapon != null )
                {
                    Image icon = weapon.getIcon();
                    if( icon != null )
                    {
                        g.drawImage( icon, 5, 170, this );
                    }
                    else
                    {
                        g.drawString( weapon.getName(), 5, 195 );
                    }
                }
            }
            if( showWeaponSelectionHint && enabled )
            {
                g.setColor( Color.white );
                g.fillPolygon( weaponHint );
            }
	    if( !enabled )
	    {
		g.setColor( Color.red );
		g.drawString( "Unavailable", 10, getHeight()/2-10 );
	    }
        }
    }
    
    public void setMarine( Marine marine )
    {
	this.marine = marine;
	repaint();
    }

    public Marine getMarine()
    {
        return marine;
    }

    public void setDrawFaceOnly( boolean faceOnly )
    {
        this.faceOnly = faceOnly;
        setSize();
    }

    public void setShowWeapon( boolean showWeapon )
    {
        this.showWeapon = showWeapon;
        setSize();
        repaint();
    }

    public void setShowWeaponSelectionHint( boolean hint )
    {
        showWeaponSelectionHint = hint;
    }

    public void setEnabled( boolean enabled )
    {
	this.enabled = enabled;
    }
    
    public void setSize()
    {
        int width = 140;
        int height = 165;
        if( faceOnly )
        {
            width = 120;
        }
        if( showWeapon )
        {
            height = 200;
        }
        Dimension mySize = new Dimension( width, height );
        setPreferredSize( mySize );
        setMinimumSize( mySize );
        setMaximumSize( mySize );
        weaponHint = new Polygon();
        weaponHint.addPoint( width-15, height-15 );
        weaponHint.addPoint( width-5, height-15 );
        weaponHint.addPoint( width-10, height-5 );        
        invalidate();
    }
}