// MarineTeamPanel.java
// Started on June 15th, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import javax.swing.*;

public class MarineTeamPanel extends JPanel implements Scrollable
{
    public static final int MARINE_ADDED_TO_TEAM = 0;
    public static final int MARINE_ALREADY_IN_TEAM = -1;
    public static final int TEAM_FULL = -2;
    public static final int DEAD_MARINE = -3;
    
    private Vector marines;
    private Vector marinePanels;
    private int slots = 0;
    private boolean allowGrow = true;
    private boolean allowWeaponSelection = false;
    private boolean enabled = true;

    public MarineTeamPanel()
    {
        marines = new Vector();
        marinePanels = new Vector();
        FlowLayout myLayout = new FlowLayout( FlowLayout.CENTER, 10, 5 );
        this.setLayout( myLayout );
    }

    public void setMarines( Vector newMarines )
    {
        this.marines = new Vector();
        for( int i = 0; i < marinePanels.size(); i++ )
        {
            MarineStatisticsPanel marinePanel = (MarineStatisticsPanel)marinePanels.elementAt( i );
            remove( marinePanel );
        }
        for( int i=0; i < newMarines.size(); i++ )
        {
            marines.add( newMarines.elementAt( i ) );
        }
        marinePanels = new Vector();
        for( int i = 0; i < marines.size(); i++ )
        {
            MarineStatisticsPanel marinePanel = new MarineStatisticsPanel();
            if( allowWeaponSelection )
            {
                marinePanel.setShowWeapon( true );
                marinePanel.setShowWeaponSelectionHint( true );
            }
            marinePanel.setMarine( (Marine)marines.elementAt( i ) );
            marinePanel.setDrawFaceOnly( true );
            marinePanels.add( marinePanel );
            add( marinePanel );
        }
        slots = marines.size();
        invalidate();
        repaint();
    }

    public Vector getMarines()
    {
        return marines;
    }

    public int addMarine( Marine marine )
    {
        if( marines.contains( marine ) )
        {
            // Don't add marines that are already there
            System.out.println( "Marine already in team" );
            return MARINE_ALREADY_IN_TEAM;
        }
	if( !marine.isAlive() )
	{
	    return DEAD_MARINE;
	}
        // Check for empty slots
        boolean full = true;
        int emptyIndex = -1;
        for( int i = 0; i < marinePanels.size(); i++ )
        {
            if( ((MarineStatisticsPanel)marinePanels.elementAt( i )).getMarine() == null )
            {
                full = false;
                emptyIndex = i;
                System.out.println( "Not full, adding the marine to slot " + emptyIndex );
                break;
            }
        }
        if( full )
        {
            if( allowGrow )
            {
                slots++;
                MarineStatisticsPanel marinePanel = new MarineStatisticsPanel();
                if( allowWeaponSelection )
                {
                    marinePanel.setShowWeapon( true );
                    marinePanel.setShowWeaponSelectionHint( true );
                }
                marinePanel.setMarine( marine );
                marinePanel.setDrawFaceOnly( true );
                marinePanels.add( marinePanel );
                marines.add( marine );
                add( marinePanel );
                repaint();
            }
            else
            {
                System.out.println( "Team full" );
                return TEAM_FULL;
            }
        }
        else
        {
            setMarineAt( marine, emptyIndex );
        }
        return MARINE_ADDED_TO_TEAM;
    }
    
    public void setAllowGrow( boolean allowGrow )
    {
        this.allowGrow = allowGrow;
    }

    public int getSlots()
    {
        return slots;
    }
    
    public void setSlots( int slots )
    {
        this.slots = slots;
        for( int i = 0; i < marinePanels.size(); i++ )
        {
            MarineStatisticsPanel marinePanel = (MarineStatisticsPanel)marinePanels.elementAt( i );
            remove( marinePanel );
        }
        marinePanels = new Vector();
        marines = new Vector( slots );
        for( int i=0; i < slots; i++ )
        {
            MarineStatisticsPanel marinePanel = new MarineStatisticsPanel();
            marinePanel.setDrawFaceOnly( true );
            if( allowWeaponSelection )
            {
                marinePanel.setShowWeapon( true );
                marinePanel.setShowWeaponSelectionHint( true );
            }
            marinePanels.add( marinePanel );
            add( marinePanel );
        }
        invalidate();
        repaint();
    }

    public void removeMarineAt( int index )
    {
        Marine marine = ((MarineStatisticsPanel)marinePanels.elementAt( index )).getMarine();
        if( marine != null )
        {
            marines.remove( marine );
            for( int i=0; i < marinePanels.size(); i++ )
            {
                ((MarineStatisticsPanel)marinePanels.elementAt( i )).setMarine( null );
            }
            for( int i=0; i < marines.size(); i++ )
            {
                ((MarineStatisticsPanel)marinePanels.elementAt( i )).setMarine( (Marine)marines.elementAt( i ) );
            }
        }
        repaint();
    }

    public void removeMarine( Marine marine )
    {
        if( marines.contains( marine ) )
        {
            int slot = marines.indexOf( marine );
            marines.setElementAt( null, slot );
            ((MarineStatisticsPanel)marinePanels.elementAt( slot )).setMarine( null );
        }
        repaint();
    }
    
    public void removeMarineAt( Point point )
    {
        int marineIndex = point.x/130;
        removeMarineAt( marineIndex );
    }

    public void setMarineAt( Marine marine, int slot )
    {
        if( marines.contains( marine ) )
        {
            int oldSlot = marines.indexOf( marine );
            marines.setElementAt( null, slot );
            ((MarineStatisticsPanel)marinePanels.elementAt( oldSlot )).setMarine( null );
        }
        ((MarineStatisticsPanel)marinePanels.elementAt( slot )).setMarine( marine );
        ((MarineStatisticsPanel)marinePanels.elementAt( slot )).repaint();
        if( slot < marines.size() )
        {
            marines.setElementAt( marine, slot );
        }
        else
        {
            marines.add( marine );
        }
        repaint();
    }

    public Marine getMarineAt( int index )
    {
        Marine marine = null;
        if( index >= 0 && index < marines.size() )
        {
            marine = (Marine)marines.elementAt( index );
        }
        return marine;
        
    }
    
    public Marine getMarineAt( Point point )
    {
        int marineIndex = point.x/130;
        return getMarineAt( marineIndex );
    }

    public void setAllowWeaponSelection( boolean allowWeaponSelection )
    {
        this.allowWeaponSelection = allowWeaponSelection;
        invalidate();
        repaint();
    }
    
    public void setEnabled( boolean enabled )
    {
	this.enabled = enabled;
	for( int i=0; i < marinePanels.size(); i++ )
	{
	    ((MarineStatisticsPanel)marinePanels.elementAt( i )).setEnabled( enabled );
	}
    }
    
    public Dimension getPreferredSize()
    {
        int height = 185;
        if( allowWeaponSelection )
        {
            height = 205;
        }
        int width = 130 * slots;
        if( width < 130 )
        {
            width = 130;
        }
        Dimension mySize = new Dimension( width, height );
        return mySize;
    }

    public Dimension getMinimumSize()
    {
        return getPreferredSize();
    }

    public Dimension getMaximumSize()
    {
        return getPreferredSize();
    }

    public Dimension getPreferredScrollableViewportSize()
    {
        Dimension myViewPortSize = new Dimension( 5*130, 185 );
        if( allowWeaponSelection )
        {
            myViewPortSize.height=205;
        }
        return myViewPortSize;
    }
    
    public int getScrollableUnitIncrement( Rectangle visibleRect, int orientation, int direction )
    {
        //get the current position
        int currentPosition = 0;
        if( orientation == SwingConstants.HORIZONTAL )
            currentPosition = visibleRect.x;
        else
            currentPosition = visibleRect.y;

        //return the number of pixels between currentPosition
        //and the nearest tick mark in the indicated direction
        if ( direction < 0 )
        {
            int newPosition = currentPosition - (currentPosition / 130) * 130;
            return (newPosition == 0) ? 130 : newPosition;
        }
        else
        {
            return ((currentPosition / 130) + 1) * 130 - currentPosition;
        }
    }

    public int getScrollableBlockIncrement( Rectangle visibleRect, int orientation, int direction )
    {
        int amount = 0;
        if( orientation == SwingConstants.HORIZONTAL )
        {
            amount = getScrollableUnitIncrement( visibleRect, orientation, direction );
            if( amount < 130 )
                amount = amount+130;
            return visibleRect.width - amount;
        }
        else
        {
            amount = getScrollableUnitIncrement( visibleRect, orientation, direction );
            if( amount < 130 )
                amount = amount+130;
            return visibleRect.height - amount;
        }
    }

    public boolean getScrollableTracksViewportWidth()
    {
        return false;
    }

    public boolean getScrollableTracksViewportHeight()
    {
        return false;
    }
    
}