// MissionDialog.java
// Started on July 29th, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.io.*;
import java.util.*;

public class MissionDialog extends JDialog implements Runnable, ActionListener, ListSelectionListener, WindowListener
{
    public static final int CAN_CHANGE_FREELY = 0;
    public static final int ONE_SHOTS_ONLY = 1;
    public static final int NEXT_MISSION_ONLY = 2;
    
    private TeamManagerWindow host;
    private Mission selectedMission;
    private int changeState = CAN_CHANGE_FREELY;
    private boolean showOneShotsOnly = false;
    private String nextMissionName = null;
    private String missionDirectoryPath = null;
    
    private JButton randomizeMissionButton;
    private JList missionList;
    private MissionListModel missionListModel;
    private GameOverviewMapPanel mapPanel;
    private JTextField missionNameField;
    private JTextArea missionBriefArea;
    private JButton selectButton;
    
    public MissionDialog( TeamManagerWindow host )
    {
	super( host, "Mission selection", true );
	this.host = host;
	Box mainBox = Box.createHorizontalBox();
	Box left = Box.createVerticalBox();
	Box buttonBox = Box.createHorizontalBox();
	buttonBox.add( Box.createHorizontalGlue() );
	randomizeMissionButton = new JButton( "Random" );
	randomizeMissionButton.addActionListener( this );
	buttonBox.add( randomizeMissionButton );
	buttonBox.add( Box.createHorizontalGlue() );
	left.add( buttonBox );
	missionListModel = new MissionListModel();
	missionList = new JList( missionListModel );
	missionList.setPrototypeCellValue( "This is a rather long mission name, whoo" );
	missionList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
	missionList.addListSelectionListener( this );
	left.add( new JScrollPane( missionList ) );
	left.add( Box.createVerticalGlue() );
	mainBox.add( left );
	mainBox.add( Box.createHorizontalStrut( 10 ) );
	Box right = Box.createVerticalBox();
	JPanel mapPanelPanel = new JPanel(); // Fantastic naming scheme, isn't it... 
	mapPanelPanel.setPreferredSize( new Dimension( 200, 200 ) );
	mapPanel = new GameOverviewMapPanel();
	mapPanelPanel.add( mapPanel );
	right.add( mapPanelPanel );
	missionNameField = new JTextField( 30 );
	missionNameField.setEditable( false );
	right.add( missionNameField );
	right.add( Box.createVerticalStrut( 12 ) );
	missionBriefArea = new JTextArea( 10, 30 );
	missionBriefArea.setLineWrap( true );
	missionBriefArea.setWrapStyleWord( true );
	missionBriefArea.setEditable( false );
	JScrollPane missionBriefScroller = new JScrollPane( missionBriefArea, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
	right.add( missionBriefScroller );
	right.add( Box.createVerticalGlue() );
	Box selectButtonBox = Box.createHorizontalBox();
	selectButtonBox.add( Box.createHorizontalGlue() );
	selectButton = new JButton( "OK" );
	selectButton.addActionListener( this );
	selectButtonBox.add( selectButton );
	right.add( selectButtonBox );
	mainBox.add( right );
	mainBox.setBorder( new EmptyBorder( 10, 20, 20, 20 ) ); 
	getContentPane().add( mainBox );
	addWindowListener( this );
	pack();
	missionDirectoryPath = System.getProperty( "user.dir" );
    }
    
    public void actionPerformed( ActionEvent e )
    {
	if( e.getSource() == selectButton )
	{
	    setVisible( false );
	}
	if( e.getSource() == randomizeMissionButton )
	{
	    if( changeState == CAN_CHANGE_FREELY || changeState == ONE_SHOTS_ONLY )
	    {
		randomizeMission();
	    }
	}
    }
    
    public void setMissionDirectoryPath( String missionDirectoryPath )
    {
	this.missionDirectoryPath = missionDirectoryPath;
	Thread filler = new Thread( this );
	filler.start();
    }
    
    public Mission getMission()
    {
	return selectedMission;
    }
        
    public void setMission( String missionFileName, int changeStatus )
    {
	System.out.println( "MissionDialog.setMission (" + missionFileName + ")" );
	System.out.println( "ChangeStatus: " + changeStatus );
	changeState = changeStatus;
	if( changeStatus != CAN_CHANGE_FREELY )
	{
	    nextMissionName = missionFileName;
	}
	if( missionFileName == null )
	{
	    changeState = CAN_CHANGE_FREELY;
	    selectedMission = null;
	    nextMissionName = null;
	}
	else
	{
	    selectedMission = loadMission( missionFileName );
	}
	if( selectedMission != null )
	{
	    mapPanel.setMap( selectedMission.getMap() );
	    missionNameField.setText( selectedMission.getName() );
	    String description = selectedMission.getMissionDescription();
	    description = description.replaceAll( "\\n\\n", "TEMPNEWLINE" );
	    description = description.replaceAll( "\\n", " " );
	    description = description.replaceAll( "TEMPNEWLINE", "\n\n" );
	    missionBriefArea.setText( description );
	    //missionBriefArea.invalidate();
	    validate();
	    missionBriefArea.setCaretPosition( 0 );
	}
	else
	{
	    mapPanel.setMap( null );
	    showOneShotsOnly = false;
	    missionNameField.setText( "No mission selected" );
	    missionBriefArea.setText( "" );
	}
	boolean canChange = false;
	if( changeState == CAN_CHANGE_FREELY || changeState == ONE_SHOTS_ONLY )
	{
	    canChange = true;
	}
	randomizeMissionButton.setEnabled( canChange );
	missionList.setEnabled( canChange );
    }
    
    public void setVisible( boolean visible )
    {
	// Center on host
	Dimension hostSize = host.getSize();
	int xLoc = (hostSize.width-getWidth())/2;
	int yLoc = (int)((hostSize.height-getHeight())*0.25);
	setLocation( xLoc, yLoc );
	
	
	super.setVisible( visible );
    }
    
    public Mission loadMission( String missionFileName )
    {
	Mission mission = null;
        File missionFile;
	String directory;
        if( missionDirectoryPath == null )
        {
            directory = System.getProperty( "user.dir" );
        }
        else
        {
            directory = missionDirectoryPath;
        }
	missionFile = new File( missionDirectoryPath, missionFileName );
	if( missionFile.exists() && missionFile.canRead() )
	{
	    try
	    {
		// Load the mission data
		mission = Mission.openMissionFromFile( missionFile );
		if( mission.getPrimaryObjective() == null )
		{
		    mission.randomizeObjectives();
		}
	    }
	    catch( FileNotFoundException e )
	    {
		System.out.println( "Map file not found!" );
		return null;
	    }
	    catch( Exception e )
	    {
		e.printStackTrace();
		return null;
	    }
	}
	return mission;
    }
    
    public void randomizeMission()
    {
	MissionData [] availableMissions = missionListModel.getAvailableMissions();
	Vector randomMissions = new Vector();
	for( int i=0; i < availableMissions.length; i++ )
	{
	    if( availableMissions[i].canRandomize() )
	    {
		randomMissions.add( availableMissions[i] );
	    }
	}
	MissionData missionData;
	if( randomMissions.size() > 0 )
	{
	    int index = (int)(Math.random()*randomMissions.size());
	    missionData = (MissionData)randomMissions.elementAt( index );
	    // If we get this far, we can change the mission, anyway, so...
	    setMission( missionData.getMissionFileName(), changeState );
	}
    }
    
    public void valueChanged( ListSelectionEvent e )
    {
	if( e.getSource() == missionList )
	{
	    int index = missionList.getSelectedIndex();
	    if( ( changeState == CAN_CHANGE_FREELY || changeState == ONE_SHOTS_ONLY ) && index > -1 )
	    {
		setMission( missionListModel.getMissionFileNameAt( index ), changeState );
	    }
	}
    }
    
    public void windowClosing( WindowEvent e )
    {
	System.out.println( "MissionDialog closing" );
	setVisible( false );
    }
    
    public void windowClosed( WindowEvent e )
    {
	System.out.println( "MissionDialog closed" );
    }
    
    public void windowActivated( WindowEvent e )
    {}
    
    public void windowDeactivated( WindowEvent e )
    {}
    
    public void windowIconified( WindowEvent e )
    {}
    
    public void windowDeiconified( WindowEvent e )
    {}
    
    public void windowOpened( WindowEvent e )
    {}
    
    public void run()
    {
	missionListModel.clear();
	try
	{
	    File directory = new File( missionDirectoryPath );
	    if( directory.exists() )
	    {
		File [] files = directory.listFiles();
		for( int i=0; i < files.length; i++ )
		{
		    if( files[i].getName().endsWith( ".mission" ) )
		    {
			Mission mission = loadMission( files[i].getName() );
			if( mission != null )
			{
			    if( ( changeState == ONE_SHOTS_ONLY || changeState == NEXT_MISSION_ONLY ) && mission.hasFollowUp() && nextMissionName != null )
			    {
				if( mission.getMissionFileName().equals( nextMissionName ) )
				{
				    // if the mission is the next mission, add it to the list, otherwise don't
				    missionListModel.addMission( mission );
				}
			    }
			    else
			    {
				if( !mission.getHasPrequisite() || mission.getMissionFileName().equals( nextMissionName ) )
				{
				    missionListModel.addMission( mission );
				}
			    }
			}
		    }
		}
	    }
	    
	}
	catch( Exception e )
	{
	    System.out.println( "Exception getting mission list!" );
	    System.out.println( e.toString() );
	}
    }
    
}

class MissionData
{
    private String missionName;
    private String missionFileName;
    private boolean hasPrequisite;
    private boolean canRandomize = false;
    
    public MissionData( Mission mission )
    {
	missionName = mission.getName();
	missionFileName = mission.getMissionFileName();
	hasPrequisite = mission.getHasPrequisite();
	if( !hasPrequisite )
	{
	    String missionCompName = missionFileName.substring( 0, missionFileName.lastIndexOf( "." ) );
	    String mapFileName = mission.getMap().getFileName();
	    String mapCompName = mapFileName.substring( 0, mapFileName.lastIndexOf( "." ) );
	    canRandomize = missionCompName.equals( mapCompName );
	}
    }
    
    public String getMissionName()
    {
	return missionName;
    }

    public String getMissionFileName()
    {
	return missionFileName;
    }
    
    public boolean hasPrequisite()
    {
	return hasPrequisite;
    }
    
    public boolean canRandomize()
    {
	return canRandomize;
    }
}

class MissionListModel extends AbstractListModel
{
    Vector availableMissions;
    
    public MissionListModel()
    {
	availableMissions = new Vector();
    }
    
    public void clear()
    {
	availableMissions = new Vector();
	fireContentsChanged( this, 0, 0 );
    }
    
    public synchronized void addMission( Mission mission )
    {
	//if( !mission.getHasPrequisite() )
	//{
	    System.out.println( "Adding mission to list: " + mission.toString() );
	    MissionData data = new MissionData( mission );
	    availableMissions.add( data );
	    fireContentsChanged( this, 0, availableMissions.size() );
	//}
    }
    
    public synchronized MissionData [] getAvailableMissions()
    {
	MissionData [] missions = new MissionData[ availableMissions.size() ];
	for( int i=0; i < availableMissions.size(); i++ )
	{
	    missions[i] = (MissionData)availableMissions.elementAt( i );
	}
	return missions;
    }
    
    public Object getElementAt( int index )
    {
	MissionData data = (MissionData)availableMissions.elementAt( index );
	return data.getMissionName();
    }
    
    public String getMissionFileNameAt( int index )
    {
	MissionData data = (MissionData)availableMissions.elementAt( index );
	return data.getMissionFileName();
    }
    
    public int getSize()
    {
	return availableMissions.size();
    }
}
