// PlasmaBlaster.java
// Started on September 3rd, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import java.awt.geom.*;
import javax.swing.ImageIcon;
import java.util.Vector;

public class PlasmaBlaster extends Weapon
{
    public static final double OVERHEAT_PROBABILITY = 0.05;
    
    private String [] firingModes = { "Normal" };
    private static Image icon = null;
    public static final int COST = 10;
    
    private int baseRange = 10;
    private Point [] area = null;
    private Point start = null;
    private Point end = null;
    private boolean overheated = false;
    
    public PlasmaBlaster()
    {
        if( icon == null )
        {
            icon = ResourceManager.loadImage( "WeaponGraphics/PlasmaBlasterIcon.png" );
        }
    }
    
    public int getDamage()
    {
        return (int)(Math.random()*8)+4;
    }
    
    public int getBlockingSize()
    {
        return 100;
    }
    
    
    public int getActionsToFire()
    {
        return 2;
    }
    
    public int getCost()
    {
        return COST;
    }    
    
    public int getMovementCost()
    {
        return 3;
    }    
    
    public boolean hasAreaEffect()
    {
        return true;
    }
    
    public String getName()
    {
        return "Plasma blaster";
    }
    
    public Point [] getArea( Game theGame, Point fireFrom, Point fireTo )
    {
        area = null;
	overheated = false;
	Vector points = new Vector();
	start = new Point( fireFrom );
	end = new Point( fireTo );

	if( Math.random() < OVERHEAT_PROBABILITY )
	{
	    overheated = true;
	    points.add( new Point( fireFrom ) );
	    for( int i=0; i < 8; i++ )
	    {
		points.add( new Point( Map.getNeighbor( fireFrom, i ) ) );
	    }
	}
	if( !overheated )
	{
	    boolean blocked = false;
	    
	    int dX = Math.abs( fireTo.x-fireFrom.x );	// Difference in coordinates
	    int dY = Math.abs( fireTo.y-fireFrom.y );
	    int checkX = fireFrom.x;
	    int checkY = fireFrom.y;
	    
	    int xIncr, yIncr;
	    if( fireFrom.x > fireTo.x ) 
		xIncr=-1;
	    else
		xIncr=1;
	    
	    if( fireFrom.y > fireTo.y )
		yIncr=-1;
	    else
		yIncr=1;
	    
	    if( dX >= dY )
	    {
		int dPr = dY*2;		// Amount to increment decision to right
		int dPru = dPr - (dX*2); 	// Amount to increment up
		int p = dPr - dX;		// decision start value
		
		while( dX > 0 && !blocked)
		{
		    if( p > 0 )	// Is the next one up and right 
		    {
			checkX+=xIncr;
			checkY+=yIncr;
			p+=dPru;
		    }
		    else		// Or just right
		    {
			checkX+=xIncr;
			p+=dPr;
		    }
		    //dX--;
		    // Check the cell
		    points.add(  new Point( checkX, checkY ) );
		    
		    if( !theGame.getMap().isViewable( checkX, checkY ) || theGame.getMap().isBlockedByInhabitant( checkX, checkY, getBlockingSize() ) )
		    {
			blocked = true;
		    }
		    if( points.size() >= baseRange )
		    {
			blocked = true;
		    }
		}
	    }
	    else
	    {
		int dPr = dX*2;
		int dPru = dPr - (dY*2);
		int p = dPr-dY;
		
		while( dY>0 && !blocked )
		{
		    if( p > 0 )
		    {
			checkX+=xIncr;
			checkY+=yIncr;
			p+=dPru;
		    }
		    else
		    {
			checkY+=yIncr;
			p+=dPr;
		    }
		    //dY--;
		    //Check the cell
		    points.add(  new Point( checkX, checkY ) );
		    
		    if( !theGame.getMap().isViewable( checkX, checkY ) || theGame.getMap().isBlockedByInhabitant( checkX, checkY, getBlockingSize() ) )
		    {
			blocked = true;
		    }
		    if( points.size() >= baseRange )
		    {
			blocked = true;
		    }
		    
		}
	    }
	}
        System.out.println( "Plasma blaster area: " + points.size() );
        area = new Point[ points.size() ];
        for( int i=0; i < points.size(); i++ )
        {
            area[ i ] = (Point)points.elementAt( i );
        }
        return area;
    }
    
    public Point [] getLastArea()
    {
        return area;
    }
    
    public Point getStart()
    {
	return start;
    }
    
    public Point getEnd()
    {
	return end;
    }
    
    public boolean getOverheated()
    {
	return overheated;
    }
    
    public String [] getFiringModeNames()
    {
        return firingModes;
    }
    
    public int getNumberOfFiringModes()
    {
        return 1;
    }
    
    public int getFiringMode()
    {
        return 0;
    }
    
    public void setFiringMode( int mode )
    {
    }
    
    public int getRequiredAttribute()
    {
        return Marine.HEAVY_WEAPONS;
    }
    
    public Image getIcon()
    {
        return icon;
    }
}