// PlasmaBlasterSprite.java
// Started on September 3rd, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Image;
import java.util.Vector;
import java.awt.Point;
import java.awt.Graphics;
import java.applet.AudioClip;
import javax.swing.RepaintManager;
import javax.swing.ImageIcon;


public class PlasmaBlasterSprite extends WeaponSprite
{
    private static AudioClip fire = null;
    private static AudioClip grenade = null;
    
    private static Image [] images = null;
    private static Image [] blastImages = null;
    
    private PlasmaBlaster weapon;
    private Point fireTo = new Point( 0, 0 );
    private Point [] area = null;
    
    
    public PlasmaBlasterSprite()
    {
        if( fire == null && GameMapPanel.playSounds )
        {
            fire = ResourceManager.loadAudioClip( "PlasmaBlaster.wav" );
	    grenade = ResourceManager.loadAudioClip( "grenade.wav" );
        }
        if( images == null )
        {
            images = new Image[ 5 ];
	    Vector temp = new Vector();
	    ResourceManager.cropImage( "WeaponGraphics/plasma.png", temp, 48 );
	    for( int i=0; i < 5; i++ )
	    {
		images[i] = (Image)temp.elementAt( i );
	    }	
	    blastImages = new Image[5];
	    temp = new Vector();
	    ResourceManager.cropImage( "WeaponGraphics/plasmaexplosion.png", temp, 48*3 );
	    for( int i=0; i < 5; i++ )
	    {
		blastImages[i] = (Image)temp.elementAt( i );
	    }	
	    
	}
    }
    
    public void setWeapon( Weapon weapon )
    {
        this.weapon = (PlasmaBlaster)weapon;
    }
    
    
    public boolean affectsLights( )
    {
        return true;
    }
    
    public void prepareFrame( Point fireFrom, Point fireTo, int frame, GameMapPanel panel )
    {
	int cellSize = panel.getCellSize();

	if( weapon.getOverheated() )
	{
	    RepaintManager.currentManager( panel ).addDirtyRegion( panel, fireFrom.x*cellSize-cellSize, fireFrom.y*cellSize-cellSize, cellSize*3, cellSize*3 );
	}
	else
	{
	    area = weapon.getLastArea();
	    int minX = Integer.MAX_VALUE;
	    int minY = Integer.MAX_VALUE;
	    int maxX = -1;
	    int maxY = -1;
	    
	    Point point;
	    for( int i = 0; i < area.length; i++ )
	    {
		point = area[i];
		if( minX > point.x ) { minX = point.x; }
		if( minY > point.y ) { minY = point.y; }
		if( maxX < point.x ) { maxX = point.x; }
		if( maxY < point.y ) { maxY = point.y; }
	    }
	    this.fireTo = fireTo;
	    RepaintManager.currentManager( panel ).addDirtyRegion( panel, minX*cellSize-cellSize, minY*cellSize-cellSize, maxX*cellSize+2*cellSize, maxY*cellSize+2*cellSize );
	}
    }
    
    
    public void drawFrame( Graphics g, GameMapPanel host, int frame )
    {
        Point point;
        int cellSize = host.getCellSize();
	int frameIndex = frame-1;
	
	if( weapon.getOverheated() )
	{
	    if( frame < 5)
	    {
		//System.out.println( "Animating grenade blast at " + fireTo.x + ", " + fireTo.y );
		g.drawImage( blastImages[ frame ], (weapon.getStart().x-1)*host.getCellSize(), (weapon.getStart().y-1)*host.getCellSize(), host );
	    }
	    
	}
	else
	{
	    for( int i = 0; i < weapon.getLastArea().length; i++ )
	    {
		point = area[i];
		//System.out.println( "Animating plasma blaster at " + point.toString() );
		//if( (frame+5)/5 >= i-5 && (frame+5)/5 <= i-1 )
		g.setColor( java.awt.Color.red );
		{
		    //g.fillOval( point.x*cellSize, point.y*cellSize, 48, 48 );
		}
		if( frame >= i )
		{
		    //frameIndex=(frameIndex+1)%5;//((frame-5)/5)%5+(frame)%5;
		    frameIndex = frame-i;
		    if( frameIndex < 5 )
		    {
			g.drawImage( images[ frameIndex ], point.x*cellSize, point.y*cellSize, host );
			g.setColor( java.awt.Color.black );
			//g.drawString( String.valueOf( frameIndex ), point.x*cellSize+10,point.y*cellSize+10 );
		    }
		}
	    }
	}
    }
    
    public int getFrames()
    {
	if( weapon.getOverheated() )
	{
	    return 6;
	}
	return weapon.getLastArea().length+6;
    }
    
    public AudioClip getFireSoundEffect()
    {
	if( weapon.getOverheated() )
	{
	    return grenade;
	}
        return fire;
    }
    
    public AudioClip getMissSoundEffect()
    {
	return null;
    }
    
}