// RecoverDataObjective.java
// Started on March 25th, 2002
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.io.*;

public class RecoverDataObjective extends Objective
{
    private String [] dataTypes = { "sensitive documents", "base security blueprints", "stolen credit card numbers", "stolen operating system source codes", "security passwords" };
    public static final int SUCCESSES_NEEDED = 3;
    
    private boolean cracked = false;
    private boolean destroyed = false;
    private String dataType;
    private ComputerSprite computerSprite;
    private int direction;
    private int hitPoints = 4;
    private int successes = 0;
    private Point location;
    private Character hacker = null;

    private String [] propertyNames = { Objective.LOCATION_KEY };
    
    public RecoverDataObjective()
    {
        dataType = dataTypes[ (int)(Math.random()*dataTypes.length) ];
        computerSprite = new ComputerSprite();
    }

    public void setGame( Game game )
    {
        activeGame = game;
        Point point = null;
        while( point == null )
        {
            point = activeGame.getRandomPointForMonster();
            if( activeGame.getMap().isPassable( point, Map.NORTH ) )
            {
                location = point;
                direction = Map.NORTH;
            }
            else if( activeGame.getMap().isPassable( point, Map.EAST ) )
            {
                location = point;
                direction = Map.EAST;
            }
            else if( activeGame.getMap().isPassable( point, Map.SOUTH ) )
            {
                location = point;
                direction = Map.SOUTH;
            }
            else if( activeGame.getMap().isPassable( point, Map.WEST ) )
            {
                location = point;
                direction = Map.WEST;
            }
            else
            {
                point = null;
            }
        }
        computerSprite.setDirection( direction );
    }

    public int getNumberOfSprites()
    {
        return 1;
    }    
    
    public Sprite getSprite( int x, int y )
    {
        return (Sprite)computerSprite;
    }

    public String [] getPropertyNames()
    {
        return propertyNames;
    }

    
    public String getProperty( String key )
    {
        if( key.equals( Objective.LOCATION_KEY ) )
        {
            return Objective.pointToString( location );
        }
        return null;
    }

    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
        if( key.equals( Objective.LOCATION_KEY ) )
        {
            setLocation( Objective.pointFromString( value ) );
        }
    }    
        
    public String getDescriptionClip()
    {
	return dataType;
    }
   
    public boolean isCompleted()
    {
        return ( destroyed );
    }
    
    public boolean isSuccessful()
    {
	return ( cracked && destroyed );
    }

    public int getSize()
    {
        return 20;
    }

    public boolean isLocatedIn( Point point )
    {
        if( location.x == point.x && location.y == point.y )
            return true;
        return false;
    }

    public boolean isLocatedIn( int x, int y )
    {
        if( location.x == x && location.y == y )
            return true;
        return false;
    }

    public boolean isDestructible()
    {
        if( destroyed )
            return false;
        return true;
    }

    public boolean isShownOnMap()
    {
        if( destroyed )
            return false;
        return true;
    }

    public boolean handleDamage( Point point, int damage )
    {
        if( isLocatedIn( point ) && !destroyed )
        {
            hitPoints -= damage;
            if( hitPoints <= 0 )
            {
                destroyed = true;
                computerSprite.setDestroyed();
                if( !cracked )
                {
                    System.out.println( "Computer broken before data was rescued" );
                    activeGame.handleMessage( new Message( null, "The " + dataType + " are lost!" ) );
                }
                return true;
            }
        }
        return false;
    }

    public boolean handleAction( GameAction action )
    {
        boolean handled = false;
        Marine user;
        if( action instanceof UseAction )
        {
            if( ((UseAction)action).getActor() instanceof Marine )
            {
                user = (Marine)((UseAction)action).getActor();
                if( location.equals( ((UseAction)action).getPoint() ) )
                {
                    if( user.getCurrentActionPoints() >= 3 )
                    {
                        handled = true;
                        ((UseAction)action).setCost( 3 );
                        if( user.getAttribute( Marine.ENGINEERING ) + (int)(Math.random()*10) >= 9 )
                        {
                            successes++;
			    if( successes < SUCCESSES_NEEDED )
			    {
				activeGame.handleMessage( new Message( (Character)user, Marine.ranks[ user.getRank() ] + " " + user.getName()+" is working on the computer, but isn't ready yet. Try again." ) );
			    }
                        }
			else
			{
			    activeGame.handleMessage( new Message( (Character)user, Marine.ranks[ user.getRank() ] + " " + user.getName()+" failed to crack the computer. Try again." ) );   
			}
                        if( successes >= SUCCESSES_NEEDED && !cracked )
                        {
                            System.out.println( "Computer cracked!" );
                            cracked = true;
			    hacker = user;
                            user.grantExperienceTo( Marine.ENGINEERING, 2 );
                            activeGame.handleMessage( new Message( (Character)user, Marine.ranks[ user.getRank() ] + " " + user.getName()+" has recovered the " + dataType +"." ) );
                        }
                    }
                }
            }
        }
        return handled;
    }
    
    public boolean isUsable()
    {
	return true;
    }
    
    
}