// Objective.java
// Started on October 27th, 2001
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.io.*;

/**
 * This Objectives adds to the game an object that
 * must be recovered. Default object is a randomly
 * placed cybernetic brain. The object counts as
 * recovered when one of the player characters is
 * carrying it.
 */
public class RecoverObjective extends Objective
{
    public static final String [] spotMessages = { "found", "located", "spotted", "discovered" };
    
    protected Game activeGame;
    private Point location = new Point(-1,-1);
    private Sprite sprite;
    private boolean seen = true;
    private boolean carried = false;
    private Character holder = null;
    private boolean visibleOnMap = true;
    private String objectName = "Brain.png";
    private String description = "cybernetically preserved brain";

    public static final String IS_SHOWN_ON_MAP_KEY = "Show on map";
    public static final String DESCRIPTION_KEY = "Description";
    private String [] propertyNames = { Objective.LOCATION_KEY, DESCRIPTION_KEY, "Object", IS_SHOWN_ON_MAP_KEY };
 
    public RecoverObjective()
    {
         sprite = new RecoverObjectiveSprite();
    }
 
    public void setGame( Game game )
    {
	activeGame = game;
	seen = false;
        if( location.x == -1 )
        {
            location = activeGame.getRandomPointForMonster();
        }
    }
          
    public int getNumberOfSprites()
    {
        if( carried || !seen )
            return 0;
	return 1;
    }
    
    public Sprite getSprite( int x, int y )
    {
	return sprite;
    }

    public String [] getPropertyNames()
    {
        return propertyNames;
    }

    public String getProperty( String key )
    {
        if( key.equals( Objective.LOCATION_KEY ) )
        {
            if( location.x == -1 && location.y == -1 )
            {
                return "Random";
            }
            return Objective.pointToString( location );
        }
        if( key.equals( DESCRIPTION_KEY ) )
        {
            return description;
        }
        if( key.equals( "Object" ) )
        {
            return objectName;
        }
        if( key.equals( IS_SHOWN_ON_MAP_KEY ) )
        {
            if( visibleOnMap )
            {
                return "True";
            }
            else
            {
                return "False";
            }
        }	
        
        return null;
    }
    
    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
        if( key.equals( Objective.LOCATION_KEY ) )
        {
            if( value.equalsIgnoreCase( "Random" ) )
            {
                setLocation( -1, -1 );
            }
            else
            {
                setLocation( Objective.pointFromString( value ) );
            }
        }
        if( key.equals( DESCRIPTION_KEY ) )
        {
            description = value;
        }
        if( key.equals( "Object" ) )
        {
            objectName = new String( value );
            try
            {
                sprite = new RecoverObjectiveSprite( objectName, 48, 48 );
            }
            catch( Exception e )
            {
                System.out.println( e.toString() );
                objectName = "Brain.png";
                sprite = new RecoverObjectiveSprite();
            }
        }
        if( key.equals( IS_SHOWN_ON_MAP_KEY ) )
        {
            if( value.equalsIgnoreCase( "True" ) )
            {
                visibleOnMap = true;
            }
            else
            {
                visibleOnMap = false;
            }
        }
    }

    public void setLocation( Point point )
    {
        location = point;
    }
    
    public void setLocation( int x, int y )
    {
	setLocation( new Point( x, y ) );
    }
    
    public String getDescriptionClip()
    {
        return description;
    }
        
    public boolean isCompleted()
    {
        if( carried )
        {
            if( !holder.isAlive() )
            {
                // Drop the objective
                carried = false;
                location = holder.getLocation();
                holder = null;
            }
        }
	return carried;
    }
    
    public boolean isLocatedIn( Point point )
    {
        if( location.x == point.x && location.y == point.y )
            return true;
	return false;
    }
 
    public boolean isLocatedIn( int x, int y )
    {
        if( location.x == x && location.y == y )
            return true;
	return false;
    }
   
    public boolean isPrimary()
    {
	return true;
    }
    
    public boolean isDestructible()
    {
	return false;
    }
    
    public boolean isShownOnMap()
    {
        return ( ( visibleOnMap || seen ) && !carried );
    }
    
    public boolean handleDamage( Point point, int damage )
    {
	return false;
    }

    public boolean handleAction( GameAction action )
    {
        Character character = action.getActor();
        if( !carried && character instanceof Marine )
        {
	    if( !seen )
	    {
		for( int i=0; i < activeGame.getPlayerCharacters().size() && !seen; i++ )
		{
		    Character check = activeGame.getPlayerCharacter( i );
		    if( check.isAlive() && activeGame.isCellVisibleToCharacter( check, location ) )
		    {
			seen = true;
			if( check instanceof Marine )
			{
			    String message = Marine.ranks[ ((Marine)check).getRank() ] + " " + check.getName() + " has " + spotMessages[ (int)(Math.random()*spotMessages.length) ] + " the " + description; 
			    activeGame.setMapChanged();
			    activeGame.handleMessage( new Message( check, message ) );
			}
		    }
		}
	    }
            if( action instanceof MoveAction )
            {
                if( isLocatedIn( ((MoveAction)action).getEndPoint() ) )
                {
                    carried = true;
                    holder = character;
                    return true;
                }
            }
            if( action instanceof UseAction )
            {
                Point point = ((UseAction)action).getPoint();
                if( isLocatedIn( point ) && Map.isNeighbor( location, character.getLocation() ) )
                {
		    ((UseAction)action).setConsumed( true );
                    carried = true;
                    holder = character;
                    return true;
                }
            }
        }
        if( carried )
        {
            if( !holder.isAlive() )
            {
                // Drop the objective
                carried = false;
                location = holder.getLocation();
                holder = null;
            }
        }

        return false;
    }
    
    public boolean isUsable()
    {
	return true;
    }

}