// ReleaseLavaObjective.java
// Started on December 22nd, 2001
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.io.*;

public class ReleaseLavaObjective extends Objective
{
    public static final int CLOSED = 0;
    public static final int OPEN = 1;
    public static final int BROKEN = 2;

    private String [] baseStart = { "Neptune", "Lunar", "Acheron", "Centauri", "Pluto", "Martian", "Europa" };
    private String [] baseMiddle = { "research base", "Observatory", "outpost", "Colony", "refinery station", "mining colony", "rehabilitation center" };
    private String [] baseEnd = { "1A", "III", "IV", "52", "7", "Alpha", "Beta", "Gamma", "TLH-858", "TK-421" };

    private String [] propertyNames = { "Location" };

    protected Game activeGame = null;
    private Point location = new Point(-1,-1);
    private Sprite closedSprite;
    private Sprite openSprite;
    private Sprite brokenSprite;
    private int armourPoints = 4;
    private int status = CLOSED;
    private String baseName = "";
    private LavaFlowObjective lavaFlow;

    public ReleaseLavaObjective()
    {
        closedSprite = new PipeSprite( CLOSED );
        openSprite = new PipeSprite( OPEN );
        brokenSprite = new PipeSprite( BROKEN );
        int start = (int)(Math.random()*baseStart.length);
        int middle = (int)(Math.random()*baseMiddle.length);
        int end = (int)(Math.random()*baseEnd.length);
        baseName = baseStart[ start ] + " " + baseMiddle[ middle ] + " " + baseEnd[ end ];
        lavaFlow = new LavaFlowObjective();
    }

    public void setGame( Game game )
    {
        if( activeGame == null )
        {
            activeGame = game;
            if( location.x < 0 )
            {
                location = game.getRandomPointForMonster();
            }
            lavaFlow.setLocation( location );
            game.addObjective( lavaFlow );
        }
    }

    public int getNumberOfSprites()
    {
        return 1;
    }

    public Sprite getSprite( int x, int y )
    {
        if( status == OPEN )
            return openSprite;
        if( status == BROKEN )
            return brokenSprite;
        return closedSprite;
    }

    public String [] getPropertyNames()
    {
        return propertyNames;
    }

    public String getProperty( String key )
    {
        if( key.equals( "Location" ) )
        {
            return Objective.pointToString( getLocation() );
        }
        return null;
    }
    
    public String getDescriptionClip()
    {
	return baseName;
    }
    
    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
        if( key.equals( "Location" ) )
        {
            setLocation( Objective.pointFromString( value ) );
        }
    }

    public void setLocation( Point point )
    {
        location = point;
        lavaFlow.setLocation( point );
    }

    public Point getLocation()
    {
        return location;
    }

    public void setLocation( int x, int y )
    {
        setLocation( new Point( x, y ) );
    }

    public boolean isCompleted()
    {
        if( status == OPEN || status == BROKEN )
            return true;
        return false;
    }

    public int getSize()
    {
        return 55;
    }

    public boolean isLocatedIn( Point point )
    {
        if( location.x == point.x && location.y == point.y )
            return true;
        return false;
    }

    public boolean isLocatedIn( int x, int y )
    {
        if( location.x == x && location.y == y )
            return true;
        return false;
    }

    public boolean isDestructible()
    {
        if( status == BROKEN || status == OPEN )
            return false;
        return true;
    }

    public boolean isShownOnMap()
    {
        if( status == BROKEN || status == OPEN )
            return false;
        return true;
    }

    public boolean handleDamage( Point point, int damage )
    {
        if( isLocatedIn( point ) && status == CLOSED )
        {
            if( damage > armourPoints )
            {
                armourPoints--;
                if( armourPoints <= 0 )
                {
                    System.out.println( "Pipe broken!" );
                    releaseLava();
                    status = BROKEN;
                    return true;
                }
            }
        }
        return false;
    }

    public boolean handleAction( GameAction action )
    {
        UseAction uAction;
        if( action instanceof UseAction )
        {
            uAction = (UseAction)action;
            if( uAction.getPoint().equals( location ) )
            {
                if( status == CLOSED )
                {
                    status = OPEN;
		    uAction.setConsumed( true );
                    releaseLava();
                    return true;
                }
            }
        }
        return false;
    }
    
    public boolean isUsable()
    {
	return true;
    }

    private void releaseLava()
    {
        Map map = activeGame.getMap();
        for( int i = 0; i < map.getHorizontalSize(); i++ )
            for( int j = 0; j < map.getVerticalSize(); j++ )
                map.setLight( i, j, (byte)(Map.LIGHT_LEVELS-1) );
        lavaFlow.startFlooding();
	
	// Open the airlock doors. We don't want anyone to get potentially trapped without their own fault, do we...
	for( int i=0; i < activeGame.getNumberOfObjectives(); i++ )
	{
	    Objective objective = activeGame.getObjective( i );
	    if( objective instanceof AirlockDoorsObjective )
	    {
		((AirlockDoorsObjective)objective).openBoth();
	    }
	}
        map.lockAllDoors();
        activeGame.setMapChanged();
    }

    
}