// RipperSpawnObjective.java
// Started on December 9th, 2001
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.io.*;

public class RipperSpawnObjective extends Objective
{
    protected Game activeGame;
    public final static String SPAWN_POINT_KEY = "Spawn point";
    public final static String RANDOM_SPAWN_POINT_KEY = "Random location";
    public final static String MONSTER_CLASSNAME_KEY = "Monster class";
    public final static String STARTS_SEEKING_KEY = "Start seeking";
    public final static String ACTIVE_KEY = "Active";

    private String [] properties = { SPAWN_POINT_KEY, RANDOM_SPAWN_POINT_KEY, MONSTER_CLASSNAME_KEY, ACTIVE_KEY, STARTS_SEEKING_KEY };
    
    private Point spawnPoint = new Point( -1, -1 );
    private boolean randomSpawnPoint = true;
    private String monsterClassName = "Ripper";
    private boolean active = true;
    private boolean startSeeking = true;
    private Character dummy = new Ripper();
 
    public void setGame( Game game )
    {
	activeGame = game;
    }
          
    public String getDescriptionClip()
    {
	return dummy.getType();
    }
    
    public int getNumberOfSprites()
    {
	return 0;
    }
    
    public Sprite getSprite( int x, int y )
    {
	return null;
    }

    public String [] getPropertyNames()
    {
        return properties;
    }

    public Class getPropertyClass( String key )
    {
	if( key.equals( SPAWN_POINT_KEY ) )
	{
	    return Point.class;
	}
	if( key.equals( ACTIVE_KEY ) )
	{
	    return Boolean.class;   
	}
	if( key.equals( STARTS_SEEKING_KEY ) )
	{
	    return Boolean.class;
	}
	return String.class;
    }
    
    public String getProperty( String key )
    {
        if( key.equals( SPAWN_POINT_KEY ) )
        {
            return Objective.pointToString( spawnPoint );
        }
        if( key.equals( ACTIVE_KEY ) )
        {
            if( active )
            {
                return "True";
            }
            else
            {
                return "False";
            }
        }
        if( key.equals( STARTS_SEEKING_KEY ) )
        {
            if( startSeeking )
            {
                return "True";
            }
            else
            {
                return "False";
            }
        }
        if( key.equals( RANDOM_SPAWN_POINT_KEY ) )
        {
            if( randomSpawnPoint )
            {
                return "True";
            }
            else
            {
                return "False";
            }
        }
        if( key.equals( MONSTER_CLASSNAME_KEY ) )
        {
            return monsterClassName;
        }
        return null;
    }

    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
        if( key.equals( SPAWN_POINT_KEY ) )
        {
            spawnPoint = Objective.pointFromString( value );
        }
        if( key.equals( RANDOM_SPAWN_POINT_KEY ) )
        {
            if( value.equalsIgnoreCase( "True" ) )
            {
                randomSpawnPoint = true;
            }
            else
            {
                randomSpawnPoint = false;
            }	
        }
        if( key.equals( ACTIVE_KEY ) )
        {
            if( value.equalsIgnoreCase( "True" ) )
            {
                active = true;
            }
            else
            {
                active = false;
            }
        }
        if( key.equals( STARTS_SEEKING_KEY ) )
        {
            if( value.equalsIgnoreCase( "True" ) )
            {
                startSeeking = true;
            }
            else
            {
                startSeeking = false;
            }
        }
        
        if( key.equals( MONSTER_CLASSNAME_KEY ) )
        {
            try
            {
		String monsterClass = value;
		if( !value.startsWith( "TheGame." ) )
		{
		    monsterClass = "TheGame."+value;
		}
                dummy = (Character)(Class.forName( monsterClass ).newInstance());
            }
            catch( Exception e )
            {
                throw new IllegalArgumentException( "Character class not found for: " + value + "!" );
            }
            monsterClassName = value;
        }
       
    }
    
    public void setLocation( Point point )
    {
        spawnPoint = new Point( point );
    }
    
    public void setLocation( int x, int y )
    {
	setLocation( new Point( x, y ) );
    }
    
    public boolean isCompleted()
    {
        if( active )
        {
	    //System.out.println( "Trying to spawn " + monsterClassName );
            try
            {
		if( !monsterClassName.startsWith( "TheGame." ) )
		{
		    monsterClassName = "TheGame."+monsterClassName;
		}
		
                Monster ripper = (Monster)Class.forName( monsterClassName ).newInstance();
                ripper.setGame( activeGame );
                Point location = new Point( spawnPoint );
                if( randomSpawnPoint || ( spawnPoint.x < 0 ) )
                {
                    location = activeGame.getRandomPointForMonster();
                }
                ripper.setLocation( location );
                if( startSeeking )
                {
                    Character character = activeGame.getPlayerCharacter( (int)(Math.random()*activeGame.getPlayerCharacters().size() ) );
                    ripper.setSpottedPlayer( character, true );
                }
                activeGame.spawnMonster( ripper );
            }
            catch( Exception e )
            {
		System.out.println( "Error spawning monster!" );
		//e.printStackTrace();
		e.toString();
	    }
        }
        return true;
    }
    
    public boolean isLocatedIn( Point point )
    {
	return false;
    }
    
    public boolean isLocatedIn( int x, int y )
    {
	return false;
    }
        
    public boolean isDestructible()
    {
	return false;
    }
    
    public boolean isShownOnMap()
    {
        return false;
    }
    
    public boolean handleDamage( Point point, int damage )
    {
	return false;
    }
    
    public boolean handleAction( GameAction action )
    {
        return false;
    }

    
}