// RobotFactoryObjective.java
// Started on September 16th, 2003
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.io.*;

public class RobotFactoryObjective extends Objective
{
    public static final String [] spotMessages = { "found", "located", "spotted", "discovered" };
    
    protected Game activeGame;
    public final static String SPAWN_POINT_1_KEY = "Spawn point 1";
    public final static String SPAWN_POINT_2_KEY = "Spawn point 2";
    public final static String RANDOM_SPAWN_POINT_KEY = "Random location";
    public final static String MONSTER_CLASSNAME_KEY = "Monster class";
    public final static String STARTS_SEEKING_KEY = "Start seeking";
    public final static String TURNS_TO_BUILD_KEY = "Turns to build";
    public final static String COMPUTER_LOCATION_KEY = "Computer location";
    public final static String KEYCARD_LOCATION_KEY = "Key card location";
    public final static String ACTIVE_KEY = "Active";

    private String [] properties = { SPAWN_POINT_1_KEY, SPAWN_POINT_2_KEY, RANDOM_SPAWN_POINT_KEY, MONSTER_CLASSNAME_KEY, COMPUTER_LOCATION_KEY, KEYCARD_LOCATION_KEY, ACTIVE_KEY, STARTS_SEEKING_KEY, TURNS_TO_BUILD_KEY };
    
    private Point spawnPoint1 = new Point( -1, -1 );
    private Point spawnPoint2 = new Point( -1, -1 );
    private Point computerLocation = new Point( -1, -1 );
    private Point keyCardLocation = new Point( -1, -1 );
    private boolean cardSeen = true;
    private boolean randomSpawnPoint = true;
    private String monsterClassName = "TheGame.BattleRobot";
    private boolean active = true;
    private boolean startSeeking = false;
    private RobotFactorySprite[] robotFactorySprites;
    private RecoverObjectiveSprite keyCardSprite;
    private Character keyCardHolder = null;
    private ComputerSprite computerSprite;
    private int counter = 0;
    private int turnsToBuild = 5;

    public RobotFactoryObjective()
    {
        computerSprite = new ComputerSprite();
        keyCardSprite = new RecoverObjectiveSprite( "KeyCard.png", 48, 48 );
        robotFactorySprites = new RobotFactorySprite[ 7 ];
        for( int i = 0; i < 7; i++ )
        {
            robotFactorySprites[i] = new RobotFactorySprite( i );
        }
    }
 
    public void setGame( Game game )
    {
	activeGame = game;
	cardSeen = false;
        if( computerLocation.x < 0 )
        {
            computerLocation = activeGame.getRandomPointForMonster();
        }
        if( keyCardLocation.x < 0 )
        {
            keyCardLocation = new Point( computerLocation );
            while( keyCardLocation.equals( computerLocation ) )
            {
                keyCardLocation = activeGame.getRandomPointForMonster();
            }
        }
        
    }
          
    public int getNumberOfSprites()
    {
	return 8;
    }
    
    public Sprite getSprite( int x, int y )
    {
        if( computerLocation.x == x && computerLocation.y == y )
        {
            return computerSprite;
        }
        Sprite sprite = null;
        sprite = getSpriteForSpawnPoint( spawnPoint1, x, y );
        if( sprite != null )
        {
            return sprite;
        }
        sprite = null;
        sprite = getSpriteForSpawnPoint( spawnPoint2, x, y );
        if( sprite != null )
        {
            return sprite;
        }
        if( keyCardHolder == null && keyCardLocation.x == x && keyCardLocation.y == y )
        {
            return (Sprite)keyCardSprite;
        }
	return null;
    }

    private Sprite getSpriteForSpawnPoint( Point spawnPoint, int x, int y )
    {
        if( ( x == spawnPoint.x) && (y == spawnPoint.y+1) )
        {
            return robotFactorySprites[1];
        }
        if( ( x == spawnPoint.x) && (y == spawnPoint.y+2) )
        {
            return robotFactorySprites[4];
        }
        if( ( x == spawnPoint.x-1 ) && (y == spawnPoint.y+1) )
        {
            if( active )
            {
                return robotFactorySprites[0];
            }
            else
            {
                return robotFactorySprites[6];
            }
        }
        if( ( x == spawnPoint.x-1 ) && (y == spawnPoint.y+2) )
        {
            return robotFactorySprites[3];
        }
        if( ( x == spawnPoint.x+1 ) && (y == spawnPoint.y+1) )
        {
            return robotFactorySprites[2];
        }
        if( ( x == spawnPoint.x+1 ) && (y == spawnPoint.y+2) )
        {
            return robotFactorySprites[5];
        }
        return null;
    }

    public String [] getPropertyNames()
    {
        return properties;
    }

    public String getProperty( String key )
    {
        if( key.equals( SPAWN_POINT_1_KEY ) )
        {
            return Objective.pointToString( spawnPoint1 );
        }
        if( key.equals( SPAWN_POINT_2_KEY ) )
        {
            return Objective.pointToString( spawnPoint2 );
        }
        if( key.equals( KEYCARD_LOCATION_KEY ) )
        {
            return Objective.pointToString( keyCardLocation );
        }
        if( key.equals( COMPUTER_LOCATION_KEY ) )
        {
            return Objective.pointToString( computerLocation );
        }        
        if( key.equals( ACTIVE_KEY ) )
        {
            if( active )
            {
                return "True";
            }
            else
            {
                return "False";
            }
        }
        if( key.equals( STARTS_SEEKING_KEY ) )
        {
            if( startSeeking )
            {
                return "True";
            }
            else
            {
                return "False";
            }
        }
        if( key.equals( RANDOM_SPAWN_POINT_KEY ) )
        {
            if( randomSpawnPoint )
            {
                return "True";
            }
            else
            {
                return "False";
            }
        }
        if( key.equals( MONSTER_CLASSNAME_KEY ) )
        {
            return monsterClassName;
        }
        if( key.equals( TURNS_TO_BUILD_KEY ) )
        {
            return String.valueOf( turnsToBuild );
        }
        return null;
    }

    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
        if( key.equals( SPAWN_POINT_1_KEY ) )
        {
            spawnPoint1 = Objective.pointFromString( value );
        }
        if( key.equals( SPAWN_POINT_2_KEY ) )
        {
            spawnPoint2 = Objective.pointFromString( value );
        }
        if( key.equals( KEYCARD_LOCATION_KEY ) )
        {
            keyCardLocation = Objective.pointFromString( value );
        }
        if( key.equals( COMPUTER_LOCATION_KEY ) )
        {
            computerLocation = Objective.pointFromString( value );
        }
        
        if( key.equals( RANDOM_SPAWN_POINT_KEY ) )
        {
            if( value.equalsIgnoreCase( "True" ) )
            {
                randomSpawnPoint = true;
            }
            else
            {
                randomSpawnPoint = false;
            }	
        }
        if( key.equals( ACTIVE_KEY ) )
        {
            if( value.equalsIgnoreCase( "True" ) )
            {
                active = true;
            }
            else
            {
                active = false;
            }
        }
        if( key.equals( STARTS_SEEKING_KEY ) )
        {
            if( value.equalsIgnoreCase( "True" ) )
            {
                startSeeking = true;
            }
            else
            {
                startSeeking = false;
            }
        }
        
        if( key.equals( MONSTER_CLASSNAME_KEY ) )
        {		
	    String monsterClass = value;
            try
            {
		if( !monsterClass.startsWith( "TheGame." ) )
		{
		    monsterClass = "TheGame."+value;
		}
                Class testClass = Class.forName( monsterClass );
            }
            catch( ClassNotFoundException cnfe )
            {
                throw new IllegalArgumentException( "Character class not found for: " + value + "!" );
            }
            monsterClassName = monsterClass;
        }
        if( key.equals( TURNS_TO_BUILD_KEY ) )
        {
            try
            {
                turnsToBuild = Integer.parseInt( value );
            }
            catch( Exception e )
            {
                turnsToBuild = 5;
                throw new IllegalArgumentException( "Not a number: " + value );
            }
        }
       
    }
    
    public void setLocation( Point point )
    {
        spawnPoint1 = new Point( point );
    }
    
    public void setLocation( int x, int y )
    {
	setLocation( new Point( x, y ) );
    }
    
    public boolean isCompleted()
    {
        if( active && ++counter == turnsToBuild )
        {
            try
            {
                Monster monster = (Monster)Class.forName( monsterClassName ).newInstance();
                monster.setGame( activeGame );
                Point location = new Point( spawnPoint1 );
                if( randomSpawnPoint || ( spawnPoint1.x < 0 ) )
                {
                    location = activeGame.getRandomPointForMonster();
                }
                monster.setLocation( location );
                Character character;
                if( startSeeking )
                {
                    character = activeGame.getPlayerCharacter( (int)(Math.random()*activeGame.getPlayerCharacters().size() ) );
                    monster.setSpottedPlayer( character, true );
                }
                activeGame.spawnMonster( monster );
                if( spawnPoint2.x > 0 )
                {
                    monster = (Monster)Class.forName( monsterClassName ).newInstance();
                    monster.setGame( activeGame );
                    location = new Point( spawnPoint2 );
                    monster.setLocation( location );
                    if( startSeeking )
                    {
                        character = activeGame.getPlayerCharacter( (int)(Math.random()*activeGame.getPlayerCharacters().size() ) );
                        monster.setSpottedPlayer( character, true );
                    }
                    activeGame.spawnMonster( monster );                    
                }
            }
            catch( Exception e )
            {
		e.printStackTrace();
	    }
            counter = 0;
        }
        return !active;
    }
    
    public boolean isLocatedIn( Point point )
    {
        if( keyCardHolder == null )
        {
            if( point.equals( keyCardLocation ) )
            {
                return true;
            }
        }
        if( point.equals( computerLocation ) )
        {
            return true;
        }
        int x = point.x;
        int y = point.y;
        if( Math.abs( spawnPoint1.x-x ) <= 1 && ( y==(spawnPoint1.y+1) || y==( spawnPoint1.y+2) ) )
        {
            return true;
        }
        if( Math.abs( spawnPoint2.x-x ) <= 1 && ( y==(spawnPoint2.y+1) || y==( spawnPoint2.y+2) ) )
        {
            return true;
        }        
	return false;
    }
    
    public boolean isLocatedIn( int x, int y )
    {
	return isLocatedIn( new Point( x, y ) );
    }
        
    public boolean isDestructible()
    {
	return false;
    }
    
    public boolean isShownOnMap()
    {
        return true;
    }
    
    public boolean handleDamage( Point point, int damage )
    {
	return false;
    }
    
    public boolean handleAction( GameAction action )
    {
        Character character = action.getActor();
	if( !cardSeen )
	{
	    for( int i=0; i < activeGame.getPlayerCharacters().size(); i++ )
	    {
		Character check = activeGame.getPlayerCharacter( i );
		if( check.isAlive() && activeGame.isCellVisibleToCharacter( check, keyCardLocation ) )
		{
		    cardSeen = true;
		    if( check instanceof Marine )
		    {
			String message = Marine.ranks[ ((Marine)check).getRank() ] + " " + check.getName() + " has " + spotMessages[ (int)(Math.random()*spotMessages.length) ] + " the key card."; 
			activeGame.handleMessage( new Message( check, message ) );
		    }
		}
	    }
	}	
        if( keyCardHolder != null )
        {
            // Dead people drop their key cards
            if( !keyCardHolder.isAlive() )
            {
                keyCardLocation = keyCardHolder.getLocation();
                keyCardHolder = null;
                return true;
            }
            if( character == keyCardHolder )
            {
                if( action instanceof UseAction )
                {
                    Point point = ((UseAction)action).getPoint();
                    if( point.equals( computerLocation ) && activeGame.getMap().isNeighbor( point, character.getLocation() ) )
                    {
			((UseAction)action).setConsumed( true );
                        if( character.getCurrentActionPoints() >= 3 )
                        {
                            ((UseAction)action).setCost( 3 );
                            active = false;
                            activeGame.handleMessage( new Message( character, ((Marine)character).getName() + " shut down the robot factory." ) );
                            return true;
                        }
                        else
                        {
                            activeGame.handleMessage( new Message( character, ((Marine)character).getName() + " needs to concentrate more." ) );
                            return true;
                        }
                    }
                }                
            }
        }
        if( keyCardHolder == null )
        {
            if( character instanceof Marine )
            {
                if( action instanceof MoveAction )
                {
                    Point point = ((MoveAction)action).getEndPoint();
                    if( point.equals( keyCardLocation ) )
                    {
                        keyCardHolder = character;
                        activeGame.handleMessage( new Message( character, ((Marine)character).getName() + " got the key card. Push it into the main control computer to shut down the factory." ) );
                        return true;
                    }
                }
                if( action instanceof UseAction )
                {
                    Point point = ((UseAction)action).getPoint();
                    if( point.equals( keyCardLocation ) && activeGame.getMap().isNeighbor( point, character.getLocation() ) )
                    {
                        keyCardHolder = character;
			((UseAction)action).setConsumed( true );
                        activeGame.handleMessage( new Message( character, ((Marine)character).getName() + " got the key card. Push it into the main control computer to shut down the factory." ) );
                        return true;
                    }
                }
            }
        }
        return false;
    }

    public boolean isUsable()
    {
	return true;
    }
}