// Scratcher.java
// Started on September 1st, 2003
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class Scratcher extends Monster
{
    private boolean attacking = false;
    private boolean seeking = false;
    private int target = -1;
    private boolean hasMoreActions = false;

    public Scratcher()
    {
        location = new Point( 0, 0 );
        direction = Map.NORTH;
        alive = true;
        lastAction = new MoveAction( this, location, location );
        maximumActionPoints = 8;
        currentActionPoints = 8;
        maximumHitPoints = 5;
        currentHitPoints = 5;
    }

    public GameAction playAction()
    {
        int x;
        int y;

        Point cell;

        attacking = false;
        seeking = false;
        target = -1;

        if( alive )
        {
            if( currentActionPoints > 0 )
            {
                // If next to a player, attack
                for( int i=0; i < 8; i++ )
                {
                    for( int j=0; j < activeGame.getPlayerCharacters().size(); j++ )
                    {
                        if( activeGame.getPlayerCharacter(j).isAlive() )
                        {
                            cell = activeGame.getPlayerCharacter(j).getLocation();
                            if( cell.equals( activeGame.getMap().getNeighbor( location, i ) ) )
                            {
                                direction = i;
                                //System.out.println( "Scratcher attacking " + ((Marine)activeGame.getPlayerCharacter(j)).getName() );
                                lastAction = new MeleeAction( this, activeGame.getPlayerCharacter(j) );
                                attacking = true;
                                return lastAction;
                            }
                            else
                            {
                                attacking = false;
                            }
                        }
                        else
                        {
                            attacking = false;
                        }
                    }
                }
                if( !attacking )
                {
                    // If can see a player, move closer and attack
                    for( int i=0; i < activeGame.getPlayerCharacters().size(); i++ )
                    {
                        if( activeGame.getMap().hasLineOfSight( location, activeGame.getPlayerCharacter( i ).getLocation() ) && activeGame.getPlayerCharacter( i ).isAlive() )
                        {
                            //System.out.println( "SmartOid seeking player " + ((Marine)activeGame.getPlayerCharacter( i )).getName() );
                            seeking = true;
                            target = i;
                            break;
                        }
                    }
                    if( target > -1 )
                    {
                        Vector path = activeGame.getMap().findShortestPath( this, activeGame.getPlayerCharacter( target ).getLocation(), false );
                        if( path.size() >= 2 )
                        {
                            cell = ((MapNode)path.elementAt( 1 )).getPoint();
                            direction = Map.getDirection( location, cell, direction );
                            lastAction = new MoveAction( this, new Point( location ), new Point( cell ) );
                            //System.out.println( "Scratcher walking from " + location.toString() + " to " + cell.toString() );
                            return lastAction;
                        }
                        else
                        {
                            seeking = false;
                            target = -1;
                        }
                    }
                }
                if( !attacking && !seeking )
                {
                    // Roam around mindlessly like a common Oid
                    do
                    {
                        x = location.x;
                        y = location.y;
                        x = x + (int)(Math.random()*3)-1;
                        y = y + (int)(Math.random()*3)-1;
                    } while( !activeGame.getMap().isPassable( x, y ) );
                    lastAction = new MoveAction( this, new Point( location ), new Point( x, y ) );
		    //System.out.println( "Scratcher roaming" );
                }
            }
            else
            {
                hasMoreActions = false;
                lastAction = new MoveAction( this, new Point( location ), new Point( location ) );
            }
        }
        return lastAction;
    }

    public GameAction getLastAction()
    {
        //System.out.println( "Get action from Scratcher " +index );
        return lastAction;
    }

    public int getMeleeAbility()
    {
	return 3;
    }
    
    public int getMeleeCost()
    {
	return 4;
    }
    
    public void resetActionsPoints()
    {
        attacking = false;
        seeking = false;
        target = -1;
        currentActionPoints = maximumActionPoints;
    }

    public boolean hasMoreActions()
    {
        if( currentActionPoints > 0 )
        {
            //System.out.println( "Scratcher has " + currentActionPoints + " action points left, attacking: " + attacking +", seeking: " + seeking );
            return true;
        }
        return false;
    }

    public boolean applyDamage( int damage )
    {
        currentHitPoints = currentHitPoints-damage;
        lastAction = new MoveAction( this, location, location );
        if( currentHitPoints <= 0 )
        {
            alive = false;
            lastAction = new DeadAction( this, new Point( location ) );
            maximumActionPoints = 0;
            currentActionPoints = 0;
        }
        return alive;
    }

    public int getSize()
    {
        return 35;
    }
}