// SentryGun.java
// Started on August 11th, 2003
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class SentryGun extends Monster implements Armed
{
    public static final int ROAMING = 0;
    public static final int SEEKING = 1;
    public static final int SHOOTING = 2;
    public static final int WAITING = 3;

    private Character target = null;
    private boolean hasMoreActions = false;
    private int state = ROAMING;
    private Weapon weapon;
    private boolean blasted = false;
    private boolean armed = false;
    private boolean setByPlayer = true;
    private int burstsLeft = 36;

    public SentryGun()
    {
        location = new Point( 0, 0 );
        direction = Map.NORTH;
        alive = true;
        lastAction = new MoveAction( this, location, location );
        maximumActionPoints = 6;
        currentActionPoints = 6;
        maximumHitPoints = 8;
        currentHitPoints = 8;
        weapon = new SmartGun();
    }

    public GameAction doAction( GameAction action )
    {
        if( action.getActor() != this )
        {
            System.out.println( "Illegal action passed to " + getClass().getName() );
            return null;
        }
        if( action instanceof MoveAction )
        {
            //System.out.println( getClass().getName() + " walking from " + location.toString() + " to " + ((MoveAction)action).getEndPoint().toString() );
            direction = Map.getDirection( new Point( location ), new Point( ((MoveAction)action).getEndPoint() ), direction );
            //System.out.println( "Direction: " + direction );
            location = ((MoveAction)action).getEndPoint();
	    // Sentry guns can only move by pushing, so they won't spend action points
        }
        if( action instanceof ShootAction )
        {
            direction = Map.getDirection( new Point( location ), new Point( ((ShootAction)action).getEndPoint() ), direction );
            currentActionPoints-=((ShootAction)action).getWeapon().getActionsToFire();
        }
        if( action instanceof UseAction )
        {
            direction = Map.getDirection( new Point( location ), new Point( ((UseAction)action).getPoint() ), direction );
            currentActionPoints-= ((UseAction)action).getCost();
        }
        lastAction = action;
        return action;
    }
    

    public void setLocation( Point point )
    {
        location = new Point( point );
        lastAction = new MoveAction( this, location, location );
    }

    public void setState( int state )
    {
        this.state = state;
    }

    public void setSetByPlayer( boolean setByPlayer )
    {
        this.setByPlayer = setByPlayer;
    }

    public boolean isSetByPlayer()
    {
        return setByPlayer;
    }

    public Weapon getWeapon()
    {
        return weapon;
    }

    public void setWeapon( Weapon weapon )
    {}

    public GameAction playAction()
    {
        Vector enemies = new Vector();
        Character check;

        
        if( alive )
        {
            hasMoreActions = true;
            state = WAITING;

            for( int i=0; i < activeGame.getPlayerCharacters().size(); i++ )
            {
                check = activeGame.getPlayerCharacter( i );
                if( activeGame.getMap().hasLineOfSight( check.getLocation(), location ) && !setByPlayer && check.isAlive() )
                    enemies.add( check );
            }
            for( int i=0; i < activeGame.getMonsters().size(); i++ )
            {
                check = activeGame.getMonster( i );
                if( activeGame.getMap().hasLineOfSight( check.getLocation(), location ) && isFoe( check ) && check.isAlive() )
                    enemies.add( check );
            }
            
            //System.out.println( "SentryGun: playAction" );
            if( currentActionPoints <= 0 || burstsLeft <= 0 )
            {
                lastAction = new WaitAction( this, new Point( location ) );
                return lastAction;
            }
            // If can see enemy, shoot it
            if( enemies.size() > 0 )
            {
                target = activeGame.getClosestCharacter( this, enemies );
                //System.out.println( "SentryGun shooting at " + target.toString() );
                if( activeGame.getMap().getLineOfFire( location, target.getLocation(), weapon.getBlockingSize() ).equals( target.getLocation() ) && ( location.distance( target.getLocation() ) <= 20 ) && currentActionPoints > 1 )
                {
                    state = SHOOTING;
                    burstsLeft--;
                    //System.out.println( "Sentrygun has " + burstsLeft + " bursts left" );
                    direction = Map.getDirection( location, target.getLocation(), direction );
                    lastAction = new ShootAction( this, weapon, target.getLocation(), true );
                    return lastAction;
                }
                else
                {
                    state = WAITING;
                }
            }
            if( state == WAITING )
            {
                currentActionPoints--;
                lastAction = new WaitAction( this, new Point( location ) );
            }
        }
        return lastAction;
    }

    public GameAction getLastAction()
    {
        return lastAction;
    }

    public void resetActionsPoints()
    {
        currentActionPoints = maximumActionPoints;
    }

    public int getMeleeAbility()
    {
	return -1;
    }
    
    public int getWeaponSkill()
    {
        return 12;
    }

    public boolean hasMoreActions()
    {
        return false;
    }

    public boolean isAlive()
    {
        return alive;
    }

    public boolean isFoe( Character character )
    {
        boolean enemy = true;
        boolean marine = ( character instanceof Marine );
        if( setByPlayer && ( marine || character instanceof Bob ) )
        {
            enemy = false;
        }
        if( !setByPlayer )
        {
            if( marine  )
            {
                enemy = true;
            }
            else
            {
                enemy = false;
            }
        }
        if( character instanceof SentryGun )
        {
            enemy = false;
        }
        return enemy;
    }

    public boolean isShownOnMap()
    {
        return true;
    }

    public boolean isArmed()
    {
        return armed;
    }

    public void setArmed( boolean armed )
    {
        this.armed = armed;
    }
    
    public void addKill()
    {}
    
    public void registerHit()
    {}

    public boolean applyDamage( int damage )
    {
        damage = damage-2;
        if( damage < 0 )
        {
            damage = 0;
        }
        currentHitPoints -= damage;
        if( currentHitPoints <= 0 )
        {
            alive = false;
            if( !blasted )
            {
                blasted = true;
                hasMoreActions = false;
            }
            lastAction = new DeadAction( this, new Point( location ) );
            maximumActionPoints = 0;
            currentActionPoints = 0;
        }
        return alive;
    }

    public int getSize()
    {
        return 20;
    }
}