// Slasher.java
// Started on September 1st, 2003
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class Slasher extends Monster
{
    public static final int ROAMING = 0;
    public static final int SEEKING = 1;
    public static final int ATTACKING = 2;
    
    private Character target = null;
    private boolean hasMoreActions = false;
    private int state;
    private boolean rightHanded = true;

    public Slasher()
    {
	location = new Point( 0, 0 );
	direction = Map.NORTH;
	alive = true;
	lastAction = new WaitAction( this, location );
	maximumActionPoints = 7;
	currentActionPoints = 7;
	maximumHitPoints = 6;
	currentHitPoints = 6;
        state = ROAMING;
        if( Math.random() < 0.5 )
        {
            rightHanded = false;
        }
    }
 
    public void setSpottedPlayer( Character character, boolean canSee )
    {
	super.setSpottedPlayer( character, canSee );
        if( canSee )
        {
            if( target == null )
            {
                target = character;
            }
            state = SEEKING;
            //System.out.println( "Ripper spotted player: " + character.toString() );
        }
    }

          
    public GameAction playAction()
    {
	int x; 
	int y;

	Point cell;
        Character [] enemies;
        Character check;
        
	hasMoreActions = true;
	
	if( alive )
	{
            enemies = activeGame.getVisibleEnemies( this );
            //System.out.println( "Ripper: doAction" );
            if( target != null )
                if( !target.isAlive() )
                {
                    target = null;
                    state = ROAMING;
                }
                else
                {
                    state = SEEKING;
                }
	    if( currentActionPoints > 0 && state != ATTACKING )
	    {
		// If next to an enemy, attack
                //System.out.println( "Ripper checking neighbours" );

                for( int i=0; i < enemies.length; i++ )
                {
                    check = enemies[ i ];
                    if( check.isAlive() )
                    {
                        cell = check.getLocation();
                        if( activeGame.getMap().isNeighbor( location, cell ) )
                        {
                            if( currentActionPoints >= getMeleeCost() )
                            {
                                direction = Map.getDirection( location, cell, direction );
                                //System.out.println( "Ripper attacking " + check.toString() );
                                target = check;
                                lastAction = new MeleeAction( this, check );
				state = ATTACKING;
                                return lastAction;
                            }
                            else
                            {
                                lastAction = new WaitAction( this, new Point( location ) );
                                hasMoreActions = false;
                                return lastAction;
                            }
                        }
                        else
                        {
                            state = SEEKING;
                        }
                    }
                }
                if( target == null && enemies.length > 0 )
                {
                    // Seek the closest enemy
                    state = SEEKING;
                    double range = Double.MAX_VALUE;
                    double checkRange;
                    for( int i = 0; i < enemies.length; i++ )
                    {
                        check = enemies[ i ];
                        checkRange = location.distance( check.getLocation() );
                        if( checkRange < range )
                        {
                            target = check;
                            range = checkRange;
                        }
                    }
                }
		if( state == SEEKING )
		{
                    //System.out.println( "Ripper seeking targets" );
		    if( target != null )
		    {
                        //System.out.println(" Ripper seeking: " + target.toString() );
                        GameAction action = SeekerAI.seek( this, target.getLocation() );
                        if( action != null )
                        {
                            lastAction = action;
                            return lastAction;
                        }
                        else
                        {
                            state = ROAMING;
                            target = null;
                        }
                    }
		}
		if( state == ROAMING )
		{
		    // Roam around mindlessly
                    GameAction action = RoamerAI.roam( this, 0.25 );
		    if( action == null )
		    {
			lastAction = new WaitAction( this, new Point( location ) );
		    }
		    else
		    {
			lastAction = action;
		    }
                }
	    }
	    else
	    {
		hasMoreActions = false;
		lastAction = new WaitAction( this, new Point( location ) );
	    }
	}
	return lastAction;
    }
    
    public GameAction getLastAction()
    {
	//System.out.println( "Get action from smartoid " +index );
	return lastAction;
    }
	
    public int getMeleeAbility()
    {
	return 3;
    }
	
    public int getMeleeCost()
    {
	return 3;
    }
        
    public void resetActionsPoints()
    {
	currentActionPoints = maximumActionPoints;
    }
    
    public boolean hasMoreActions()
    {
	if( currentActionPoints > 0 && hasMoreActions )
	{
	    //System.out.println( "Ripper has " + currentActionPoints + " action points left, attacking: " + attacking +", seeking: " + seeking );
	    return true;
	}
	return false;   
    }

    public boolean applyDamage( int damage )
    {
	currentHitPoints = currentHitPoints-damage;
	lastAction = new WoundAction( this, location );
	if( currentHitPoints <= 0 )
	{
	    alive = false;
	    lastAction = new DeadAction( this, new Point( location ) );
	    maximumActionPoints = 0;
	    currentActionPoints = 0;
	}
	return alive;
    }

    public int getSize()
    {
	return 60;
    }
}