// Vipidiot.java
// Started on August 12th, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;

public class Vipidiot extends Monster implements Subordinate
{
    public static final int PROFESSOR = 0;
    public static final int OFFICER = 1;
    public static final int DEMAGOGUE = 2;
    
    public static final String [] professorRanks = { "Doctor", "Professor" };
    public static final String [] professorNames = { "Bobinheimer", "Bobstein", "Bobinstein", "Bobiarty", "Bobyll", "Brown", "Emmett", "Farnsworth", "Griffin", "Luthor", "Mortimer Oriarty", "Namtar", "Rains", "Rotwang", "Soong", "Walsh", "West" };
    public static final String [] officerRanks = { "Lieutenant", "Major", "Captain", "General", "Lieutenant commander" }; 
    public static final String [] officerNames = { "Ballmer", "Bester", "Binks", "Burke", "Dexter", "Gomez", "Gorman", "Heller", "Hermann", "Rummy", "Shrubber", "Wellch" };
    public static final String [] taunts = { "Hmph!", "Hmph!", "Hmph!", "All right, all right!", "If you insist!", "Damn grunt!", "You are not here to order me around!", "Excuse me?", "Silence, fool!", "Very well, this once." };
    
    private int type = PROFESSOR;
    private Point target = null;
    private Point tempTarget = null;
    private int meleeAbility=0;
    private boolean traitor = false;
    
    public Vipidiot()
    {
	currentActionPoints = 4;
	maximumActionPoints = 4;
	currentHitPoints = 6;
	maximumHitPoints = 6;
	location = new Point( -1, -1 );
	direction = 0;
	lastAction = new WaitAction( this, location );
	meleeAbility = 3+(int)(Math.random()*3);
	alive = true;
    }
    
    public Point getTarget()
    {
	return tempTarget;
    }
    
    public String getTaunt()
    {
	return taunts[ (int)(Math.random()*taunts.length) ];
    }
    
    public void setTarget( Point point )
    {
	if( point != null )
	{
	    tempTarget = new Point( point );
	}
	else
	{
	    tempTarget = null;
	}
    }
    
    public void setTheTarget( Point point )
    {
	target = new Point( point );
    }
    
    public int getMeleeAbility()
    {
	return meleeAbility;
    }
    
    public int getMeleeCost()
    {
	return 1;
    }
    
    public int getVipType()
    {
	return type;
    }
    
    public boolean isFoe( Character character )
    {
	if( traitor && activeGame.isPlayerCharacter( character ) )
	{
	    return true;
	}
	if( activeGame.isPlayerCharacter( character ) || character.getType().equals( "Bob" ) || traitor || dormant )
	{
	    return false;
	}
	return true;
    }
    
    public boolean move()
    {	
	int newDirection = direction;
	boolean moved = false;
	
	if( traitor )
	{
	    tempTarget = target;
	}
	
	if( tempTarget != null )
	{
	    // Run to the target
	    GameAction action = SeekerAI.seek( this, tempTarget );
	    if( action != null )
	    {
		if( action instanceof MoveAction )
		{
		    newDirection = Map.getDirection( location, ((MoveAction)action).getEndPoint(), direction );
		    activeGame.handleMessage( new Message( this, "\"" + getTaunt() + "\"" ) );
		    activeGame.moveCharacter( this, newDirection );
		    moved = true;
		}
	    }
	}
	else
	{
	    if( Math.random() < 0.4 )
	    {
		newDirection = (int)(Math.random()*8);   
	    }
	    for( int i=0; (i < 8) && !moved; i++ )
	    {
		if( activeGame.canCharacterMove( this, newDirection, false ) )
		{
		    moved = true;
		    activeGame.handleMessage( new Message( this, "\"" + getTaunt() + "\"" ) );
		    activeGame.moveCharacter( this, newDirection );
		}
		newDirection = (newDirection+1)%8;
	    }
	}
	if( !moved )
	{
	    activeGame.handleOutOfAP( this );
	}
	return moved;
    }    
    
    public GameAction playAction()
    {
	if( alive && !dormant )
	{
	    Character [] visibleEnemies = activeGame.getVisibleEnemies( this );
	    if( ( visibleEnemies.length > 0 && Math.random() < 0.5 && !traitor ) || currentActionPoints <= 0 )
	    {
		lastAction = new WaitAction( this, location );
		//System.out.println( "Vipidiot staying put" );
	    }
	    else
	    {
		Point targetPoint = tempTarget;
		if( targetPoint == null )
		{
		    targetPoint = target;
		}
		lastAction = SeekerAI.seek( this, targetPoint );
		//System.out.println( "Vipidiot proceeding" );
		if( lastAction == null )
		{
		    lastAction = RoamerAI.roam( this, 0.5 );
		    //System.out.println( "Vipidiot roaming" );
		}
	    }
	}
	return lastAction;
    }
    
    public void postAction()
    {
	tempTarget = null;
    }    
    
    public void randomize()
    {
	type = (int)(Math.random()*2);
	if( type == PROFESSOR )
	{
	    name = professorRanks[ (int)(Math.random()*professorRanks.length) ] + " " + professorNames[ (int)(Math.random()*professorNames.length) ];
	}
	if( type == OFFICER )
	{
	    name = officerRanks[ (int)(Math.random()*officerRanks.length) ] + " " + officerNames[ (int)(Math.random()*officerNames.length) ];
	}
    }
    
    public void set( int type, String name )
    {
	this.type = type;
	this.name = name;
    }
    
    public void setTraitor( boolean traitor )
    {
	this.traitor = traitor;
    }
}