// VipidiotArrestObjective.java
// Started on September 19th, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.Point;
import java.util.Vector;

public class VipidiotArrestObjective extends Objective
{
    private static final String [] typeNames = { "Professor", "Officer", "Demagogue" };
    private static final String [] messages = { "All right, I have seen enough.", 
	"I have seen enough.", "Seems good enough.", "Fine, this will do.", "All right, let's return.", 
	"Bored now.", "This is acceptable.", "Nothing interesting here.", "This will not do. No use continuing." };
    
    private static final Point SAFE_POINT = new Point( -10, -10 );
    public static final String START_POINT_KEY = "Start point";
    public static final String ADD_POINT_KEY = "Add point";
    public static final String REMOVE_POINT_KEY = "Remove point";
    public static final String END_POINT_KEY = "End point";
    public static final String STARTS_DORMANT_KEY = "Dormant";
    public static final String TRAITOR_KEY = "Traitor";
    public static final String DEATH_MESSAGE_KEY = "Death message";
    public static final String TYPE_KEY = "Type";
    public static final String NAME_KEY = "Name";
    
    private Vipidiot vip = null;
    private Point startPoint = null;
    private Point endPoint = null;
    private Vector targets = new Vector();
    private boolean startsDormant = false;
    private boolean seen = false;
    private boolean arrested = false;
    private boolean captured = false;
    private int type = -1;
    private String name = null;
    private boolean complete = false;
    private boolean messagedDeath = false;
    private String deathMessage = "$NAME$ has died!";
    
    public VipidiotArrestObjective()
    {
	vip = new Vipidiot();
	vip.randomize();
    }
    
    public void setGame( Game game )
    {
	activeGame = game;
	MapNode initial = (MapNode)game.getPlayerStartingPoints().elementAt( 0 );
	Point playerStart = new Point( initial.getPoint() );
	
	if( startPoint == null )
	{
	    Point point = new Point( playerStart );
	    int direction = initial.getDirection();
	    do
	    {
		point = game.getMap().getNeighbor( point, direction );
	    } while( !game.getMap().isPassable( point ) );
	    startPoint = point;
	    if( targets.size() < 1 )
	    {
		int targetCount = 1+(int)(Math.random()*3);
		//System.out.println( "Vipidiot has " + targetCount + " targets" );
		for( int i=0; i < targetCount; i++ )
		{
		    targets.add( game.getRandomPointForMonster() );
		}
	    }
	}
	if( startPoint.x < 0 )
	{
	    startPoint = game.getRandomPointForMonster();
	}
	if( targets.size() > 0 )
	{
	    for( int i=0; i < targets.size(); i++ )
	    {
		Point targetPoint = (Point)targets.elementAt( i );
		if( targetPoint.x < 0 )
		{
		    targetPoint = game.getRandomPointForMonster();
		    targets.set( i, targetPoint );
		}
	    }
	}
	if( endPoint == null )
	{
	    endPoint = new Point( playerStart );
	}
	if( endPoint.x < 0 )
	{
	    endPoint = game.getRandomPointForMonster();
	}
	if( type >= 0 && name != null )
	{
	    vip.set( type, name );
	}
	vip.setLocation( startPoint );
	vip.setDirection( Map.getDirection( startPoint, playerStart, 0 ) );
	vip.setTraitor( true );
	vip.setGame( game );
	vip.setDormant( startsDormant );
	if( targets.size() > 0 )
	{
	    vip.setTheTarget( (Point)targets.elementAt( 0 ) );
	}
	else
	{
	    vip.setTheTarget( endPoint );
	}
	vip.setId( "VIP-"+id );
	System.out.println( "Vip ID: " + vip.getId() );
	game.addMonster( vip );
    }
    
    public String getDescriptionClip()
    {
	return vip.getName();
    }
    
    public boolean isCompleted()
    {
	if( complete || !vip.isAlive() )
	{
	    return true;
	}
	return false;
    }
    
    public boolean isSuccessful()
    {
	return captured;
    }
    
    public boolean handleAction( GameAction action )
    {
	Character check;
	if( !seen )
	{
	    for( int i=0; i < activeGame.getPlayerCharacters().size(); i++ )
	    {
		check = activeGame.getPlayerCharacter( i );
		if( check.isAlive() && activeGame.isCellVisibleToCharacter( check, vip.getLocation() ) )
		{
		    seen = true;
		    String message = Marine.ranks[ ((Marine)check).getRank() ] + " " + check.getName() + " has spotted " + vip.getName() + "!";
		    message += "\nWalk up to him to make the arrest.";
		    activeGame.handleMessage( new Message( check, message  ) );
		}
	    }
	}
	for( int i=0; i < activeGame.getPlayerCharacters().size() && !arrested; i++ )
	{
	    check = activeGame.getPlayerCharacter( i );
	    if( activeGame.getMap().isNeighbor( check.getLocation(), vip.getLocation() ) )
	    {
		arrested = true;
		vip.setTraitor( false );
		endPoint = activeGame.getPlayerStartingPoint( 0 );
		targets.clear();
		vip.setTheTarget( endPoint );
		String message = Marine.ranks[ ((Marine)check).getRank() ] + " " + check.getName() + " has arrested " + vip.getName() + ".";
		message += "\nNow escort him back to the base.";
		activeGame.handleMessage( new Message( check, message ) );
	    }
	}
	    
	if( action.getActor() == vip )
	{
	    if( targets.size() > 0 )
	    {
		Point target = (Point)targets.elementAt( 0 );
		if( Map.isNeighbor( vip.getLocation(), target ) )
		{
		    targets.removeElementAt( 0 );
		    if( targets.size() > 0 )
		    {
			vip.setTheTarget( (Point)targets.elementAt( 0 ) );
		    }
		    else
		    {
			vip.setTheTarget( endPoint );
		    }
		}
	    }
	    if( Map.isNeighbor( vip.getLocation(), endPoint ) && targets.size() < 1 )
	    {
		vip.setLocation( SAFE_POINT );
		vip.setDormant( true );
		complete = true;
		if( arrested )
		{
		    captured = true;
		}
		if( !arrested )
		{
		    activeGame.handleMessage( new Message( vip, vip.getName() + " has escaped!" ) );
		}
		else
		{
		    activeGame.handleMessage( new Message( vip, vip.getName() + " has been brought to justice." ) );
		}
	    }
	}
	if( !messagedDeath && !vip.isAlive() )
	{
	    String messageString = deathMessage.replaceAll( "\\$NAME\\$", vip.getName() );
	    activeGame.handleMessage( messageString );
	    messagedDeath = true;
	}
        return false;
    }
    
    public String getMidMessage()
    {
	return vip.getName() + ": \"Hmm...\"";
    }
    
    
    public String getMessage()
    {
	return vip.getName() + ": \"" + messages[ (int)(Math.random()*messages.length) ] + "\"";
    }
    
    public String [] getPropertyNames()
    {
	Vector names = new Vector();
	names.add( START_POINT_KEY );
	names.add( END_POINT_KEY );
	names.add( TYPE_KEY );
	names.add( NAME_KEY );
	names.add( STARTS_DORMANT_KEY );
	names.add( TRAITOR_KEY );
	names.add( DEATH_MESSAGE_KEY );
	names.add( ADD_POINT_KEY );
	names.add( REMOVE_POINT_KEY );
	for( int i=0; i < targets.size(); i++ )
	{
	    names.add( "Target_" + i );
	}
	String [] propertyNames = new String[ names.size() ];
	for( int i=0; i < names.size(); i++ )
	{
	    propertyNames[i] = (String)names.elementAt( i );   
	}
	return propertyNames;
    }
    
    public String getProperty( String key )
    {
	if( key.equals( START_POINT_KEY ) )
	{
	    return Objective.pointToString( startPoint );
	}
	if( key.equals( END_POINT_KEY ) )
	{
	    return Objective.pointToString( endPoint );
	}
	if( key.equals( STARTS_DORMANT_KEY ) )
	{
	    return String.valueOf( startsDormant );
	}
	if( key.equals( DEATH_MESSAGE_KEY ) )
	{
	    return deathMessage;
	}
	if( key.equals( NAME_KEY ) )
	{
	    return name;
	}
	if( key.equals( TYPE_KEY ) )
	{
	    if( type >= 0 )
	    {
		return typeNames[ type ];
	    }
	}
	if( key.startsWith( "Target_" ) )
	{
	    int index = Integer.parseInt( key.substring( 7 ) );
	    return Objective.pointToString( (Point)targets.elementAt( index ) );
	}
	return null;
    }
    
    public Class getPropertyClass( String key )
    {
	if( key.equals( START_POINT_KEY ) )
	{
	    return Point.class;
	}
	if( key.equals( END_POINT_KEY ) )
	{
	    return Point.class;
	}
	if( key.startsWith( "Target_" ) )
	{
	    return Point.class;
	}
	if( key.equals( ADD_POINT_KEY ) )
	{
	    return Point.class;
	}
	if( key.equals( REMOVE_POINT_KEY ) )
	{
	    return Point.class;
	}
	return String.class;
    }
    
    
    public void setProperty( String key, String value )
    {
	if( key.equals( START_POINT_KEY ) )
	{
	    startPoint = Objective.pointFromString( value );   
	}
	if( key.equals( END_POINT_KEY ) )
	{
	    endPoint = Objective.pointFromString( value );   
	}
	if( key.equals( STARTS_DORMANT_KEY ) )
	{
	    startsDormant = new Boolean( value ).booleanValue();
	}
	if( key.equals( DEATH_MESSAGE_KEY ) )
	{
	    deathMessage = value;
	}
	if( key.equals( NAME_KEY ) )
	{
	    name = value;
	}
	if( key.equals( TYPE_KEY ) )
	{
	    int theType = Vipidiot.PROFESSOR;
	    for( int i=0; i < typeNames.length; i++ )
	    {
		if( typeNames[i].equals( value ) )
		{
		    type = i;
		}
	    }
	}
	if( key.startsWith( "Target_" ) )
	{
	    int index = Integer.parseInt( key.substring( 7 ) );
	    if( targets.size() > index )
	    {
		targets.set( index, Objective.pointFromString( value ) );
	    }
	    else
	    {
		targets.add( Objective.pointFromString( value ) );
	    }
	}
	if( key.equals( ADD_POINT_KEY ) )
	{
	    if( value != null )
	    {
		targets.add( Objective.pointFromString( value ) );
	    }
	}
	if( key.equals( REMOVE_POINT_KEY ) )
	{
	    if( value != null )
	    {
		Point check = Objective.pointFromString( value );
		for( int i=0; i < targets.size(); i++ )
		{
		    if( ((Point)targets.elementAt( i )).equals( check ) )
		    {
			targets.removeElementAt( i );
			return;
		    }
		}
	    }
	}
	if( name != null && type >= 0 )
	{
	    vip.set( type, name );
	}
    }
    
}

