// WeaponSwitchObjective.java
// Started on September 5th, 2005
// Copyright (c) Virtu Halttunen

package TheGame;

import java.awt.*;
import javax.swing.ImageIcon;
import java.io.*;

/**
 * This Objectives adds to the game an object that
 * must be recovered. Default object is a randomly
 * placed cybernetic brain. The object counts as
 * recovered when one of the player characters is
 * carrying it.
 */
public class WeaponSwitchObjective extends Objective
{
    public static final String [] spotMessages = { "found", "located", "spotted", "discovered" };
    
    protected Game activeGame;
    private Point location = new Point(-1,-1);
    private Sprite sprite;
    private boolean seen = true;
    private boolean carried = false;
    private Character holder = null;
    private boolean visibleOnMap = true;
    private Weapon weapon = null;
    private Weapon tempWeapon = null;
    private String weaponClassName = "PlasmaBlaster";

    public static final String IS_SHOWN_ON_MAP_KEY = "Show on map";
    public static final String WEAPON_CLASS_KEY = "Weapon class";
    private String [] propertyNames = { WEAPON_CLASS_KEY, Objective.LOCATION_KEY, IS_SHOWN_ON_MAP_KEY };
 
    public WeaponSwitchObjective()
    {
	sprite = new RecoverObjectiveSprite( "PlasmaBlasterObjectiveIcon.png", 48, 48 );
    }
 
    public void setGame( Game game )
    {
	activeGame = game;
	try
	{
	    weapon = (Weapon)ResourceManager.findClass( "TheGame."+weaponClassName ).newInstance();
	}
	catch( Exception e )
	{
	    e.printStackTrace();
	}
	seen = false;
        if( location.x == -1 )
        {
            location = activeGame.getRandomPointForMonster();
        }
    }
          
    public int getNumberOfSprites()
    {
        if( carried || !seen )
            return 0;
	return 1;
    }
    
    public Sprite getSprite( int x, int y )
    {
	return sprite;
    }

    public String [] getPropertyNames()
    {
        return propertyNames;
    }

    public String getProperty( String key )
    {
        if( key.equals( Objective.LOCATION_KEY ) )
        {
            if( location.x == -1 && location.y == -1 )
            {
                return "Random";
            }
            return Objective.pointToString( location );
        }
	if( key.equals( WEAPON_CLASS_KEY ) )
        {
            return weaponClassName;
        }
        if( key.equals( IS_SHOWN_ON_MAP_KEY ) )
        {
            if( visibleOnMap )
            {
                return "True";
            }
            else
            {
                return "False";
            }
        }	
        
        return null;
    }
    
    public void setProperty( String key, String value ) throws IllegalArgumentException
    {
	if( key.equals( WEAPON_CLASS_KEY ) )
	{
	    try
	    {
		System.out.println( "WeaponSwitchObjective: Trying to set weapon class" );
		weapon = (Weapon)ResourceManager.findClass( "TheGame."+value ).newInstance();
		weaponClassName = value;
		//System.out.println( "Creating sprite" );
		//sprite = new RecoverObjectiveSprite( "WeaponGraphics/"+weaponClassName+"Icon.png", 100, 30 );
	    }
	    catch( Exception e )
	    {
		throw new IllegalArgumentException( "Weapon not found: " + value );
	    }
	}
        if( key.equals( Objective.LOCATION_KEY ) )
        {
            if( value.equalsIgnoreCase( "Random" ) )
            {
                setLocation( -1, -1 );
            }
            else
            {
                setLocation( Objective.pointFromString( value ) );
            }
        }
        if( key.equals( IS_SHOWN_ON_MAP_KEY ) )
        {
            if( value.equalsIgnoreCase( "True" ) )
            {
                visibleOnMap = true;
            }
            else
            {
                visibleOnMap = false;
            }
        }
    }

    public void setLocation( Point point )
    {
        location = point;
    }
    
    public void setLocation( int x, int y )
    {
	setLocation( new Point( x, y ) );
    }
    
    public String getDescriptionClip()
    {
	if( weapon != null )
	{
	    return weapon.getName();
	}
        return "";
    }
        
    public boolean isCompleted()
    {
        if( carried )
        {
            if( !holder.isAlive() )
            {
                // Drop the objective
                carried = false;
                location = holder.getLocation();
                holder = null;
            }
        }
	return carried;
    }
    
    public boolean isSuccessful()
    {
	if( holder != null && tempWeapon != null )
	{
	    ((Armed)holder).setWeapon( tempWeapon );
	}
	return isCompleted();
    }
    
    public boolean isLocatedIn( Point point )
    {
        if( location.x == point.x && location.y == point.y )
            return true;
	return false;
    }
 
    public boolean isLocatedIn( int x, int y )
    {
        if( location.x == x && location.y == y )
            return true;
	return false;
    }
   
    public boolean isPrimary()
    {
	return true;
    }
    
    public boolean isDestructible()
    {
	return false;
    }
    
    public boolean isShownOnMap()
    {
        return ( ( visibleOnMap || seen ) && !carried );
    }
    
    public boolean handleDamage( Point point, int damage )
    {
	return false;
    }

    public boolean handleAction( GameAction action )
    {
        Character character = action.getActor();
        if( !carried && character instanceof Marine )
        {
	    if( !seen )
	    {
		for( int i=0; i < activeGame.getPlayerCharacters().size() && !seen; i++ )
		{
		    Character check = activeGame.getPlayerCharacter( i );
		    if( check.isAlive() && activeGame.isCellVisibleToCharacter( check, location ) )
		    {
			seen = true;
			if( check instanceof Marine )
			{			    
			    activeGame.setMapChanged();
			    String message = Marine.ranks[ ((Marine)check).getRank() ] + " " + check.getName() + " has " + spotMessages[ (int)(Math.random()*spotMessages.length) ] + " the " + weapon.getName(); 
			    activeGame.handleMessage( new Message( check, message ) );
			}
		    }
		}
	    }
            if( action instanceof MoveAction )
            {
                if( isLocatedIn( ((MoveAction)action).getEndPoint() ) )
                {
                    carried = true;
                    holder = character;
		    tempWeapon = ((Armed)holder).getWeapon();
		    ((Armed)holder).setWeapon( weapon );
		    String message = Marine.ranks[ ((Marine)character).getRank() ] + " " + character.getName() + " has picked up the " + weapon.getName(); 
		    activeGame.handleMessage( new Message( character, message ) );
                    return true;
                }
            }
            if( action instanceof UseAction )
            {
                Point point = ((UseAction)action).getPoint();
                if( isLocatedIn( point ) && Map.isNeighbor( location, character.getLocation() ) )
                {
		    ((UseAction)action).setConsumed( true );
                    carried = true;
                    holder = character;
		    tempWeapon = ((Armed)holder).getWeapon();
		    ((Armed)holder).setWeapon( weapon );
		    String message = Marine.ranks[ ((Marine)character).getRank() ] + " " + character.getName() + " has picked up the " + weapon.getName(); 
		    activeGame.handleMessage( new Message( character, message ) );
                    return true;
                }
            }
        }
        if( carried )
        {
            if( !holder.isAlive() )
            {
                // Drop the objective
                carried = false;
                location = holder.getLocation();
                holder = null;
            }
        }

        return false;
    }
    
    public boolean isUsable()
    {
	return true;
    }

}